/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStApi CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECTestOutput_h
#define _CECTestOutput_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <ctime.hpp>

#include <string>
#include <list>

using std::list;
using std::string;
///////////////////////////////////////////////////////////////////////////////
// @class   CECTestOutput | This class is the base class for all Test Output.
//          It provides only a single method.  This method is a pure virtual
//          method that transforms the object into a string version of itself.
//          This string will be stored as the test output.
class CYR_EXPORT CECTestOutput
{
    // @access Public Members
public:
    CECTestOutput(long t_offsets);            // @cmember Constructor
    virtual ~CECTestOutput();   // @cmember Destructor

    virtual string GetType() = 0;    // @cmember Returns type of output
    virtual string ToString() = 0;   // @cmember Serialize data to a string
    CTTime GetCreatedTime() const;      // @cmember Get time of timer creation

    // @access Protected Members
protected:
    CTTime    m_time;    // @cmember Creation time field
}
;

///////////////////////////////////////////////////////////////////////////////
// @class   CECTimerOutput | This class is used to create Timer Output records.
// @base    public | CECTestOutput
class CYR_EXPORT CECTimerOutput : public CECTestOutput
{
    // @access Public Members
public:
    // @cmember Constructor to create Test Output
    CECTimerOutput( long t_offsets, const char* desc, long value, long active );
    // @cmember Constructor to retrieve Test Output
    CECTimerOutput( const char* str );
    virtual ~CECTimerOutput();          // @cmember Destructor

    static const char* OUTPUT_TYPE;     // @cmember Type of output
    virtual string GetType();        // @cmember Returns type of output
    virtual string ToString();       // @cmember Serialize data to a string

    string GetDescription() const;   // @cmember Get timer description
    long GetValue() const;              // @cmember Get timer value
    long GetActiveThreads() const;      // @cmember Get number of active threads

    // @access Protected Members
protected:
    string m_desc;    // @cmember Description field
    long      m_value;   // @cmember Value field
    long      m_active;  // @cmember Number of active threads
}
;

///////////////////////////////////////////////////////////////////////////////
// @class   CECTestLog | This is used to create test log records.
// @base    public | CECTestOutput
class CYR_EXPORT CECTestLog : public CECTestOutput
{
    // @access Public Members
public:
    CECTestLog( long t_offsets, const char* text, bool create = true );  // @cmember Construtor
    virtual ~CECTestLog();          // @cmember Destructor

    static const char* OUTPUT_TYPE; // @cmember Type of output
    virtual string GetType();    // @cmember Returns type of output
    virtual string ToString();   // @cmember Serialize data to a string

    string GetText() const;      // @cmember Returns test log text

    // @access Protected Members
protected:
    string m_text;  // @cmember Text entry
}
;

class CYR_EXPORT CECTestReport : public CECTestLog
{
    // @access Public Members
public:
    CECTestReport( long t_offsets, const char* text, bool create = true );  // @cmember Constructor

    static const char* OUTPUT_TYPE;     // @cmember Type of output
    virtual string GetType();        // @cmember Returns type of output
}
;

class CYR_EXPORT CECTestDiagnostic : public CECTestLog
{
    // @access Public Members
public:
    CECTestDiagnostic( const char* text, bool create = true );  // @cmember Constructor

    static const char* OUTPUT_TYPE;     // @cmember Type of output
    virtual string GetType();        // @cmember Returns type of output
}
;

class CYR_EXPORT CECTestHistory : public CECTestLog
{
    // @access Public Members
public:
    CECTestHistory( long t_offsets, const char* text, bool create = true );  // @cmember Constructor

    static const char* OUTPUT_TYPE;     // @cmember Type of output
    virtual string GetType();        // @cmember Returns type of output
}
;

///////////////////////////////////////////////////////////////////////////////
// OTA Test Output
///////////////////////////////////////////////////////////////////////////////
typedef struct tagsRESULT
{
    int start;
    int stop;
    int result;
}
RESULT_S, *PRESULT_S;

typedef list<PRESULT_S> ResultData;

///////////////////////////////////////////////////////////////////////////////
// @class   CECOtaSimpleQuery | This class represents a simple set of query
//          results generated by an OTA query.
// @base    public | CECTestOutput
class CYR_EXPORT CECOtaSimpleQuery : public CECTestOutput
{
    // @access Public Members
public:
    // @cmember Constructor for retrieval
    CECOtaSimpleQuery( const char* str );
    // @cmember Constructor for creation of data entry
    CECOtaSimpleQuery( long t_offsets, const char* node, int index, int start, int stop, int result );
    virtual ~CECOtaSimpleQuery();   // @cmember Destructor

    static const char* OUTPUT_TYPE; // @cmember Type of output
    virtual string GetType();    // @cmember Returns type of output
    virtual string ToString();   // @cmember Serialize data to a string

    string GetNodeName();        // @cmember Returns name of node
    string GetDescription();     // @cmember Returns description
    int GetStartTime();
    int GetStopTime();
    int GetResult();
    int GetIndex();

    // @access Private Members
private:
    int         m_index;        // @cmember Query index
    bool        m_header;       // @cmember True if header
    string   m_node;         // @cmember Name of node where query executed
    string   m_desc;         // @cmember Description for query
    int         m_start;        // @cmember Start time
    int         m_stop;         // @cmember Start time
    int         m_result;       // @cmember Result
}
;


///////////////////////////////////////////////////////////////////////////////
// @class   CECOtaSimpleQuery | This class represents a simple set of query
//          results generated by an OTA query.
// @base    public | CECTestOutput
class CYR_EXPORT CECOtaHeader : public CECTestOutput
{
    // @access Public Members
public:
    // @cmember Constructor for retrieval
    CECOtaHeader( const char* str );
    // @cmember Constructor for creation of header
    CECOtaHeader( long t_offsets, const char* node, int index, const char* desc );
    virtual ~CECOtaHeader();   // @cmember Destructor

    static const char* OUTPUT_TYPE; // @cmember Type of output
    virtual string GetType();    // @cmember Returns type of output
    virtual string ToString();   // @cmember Serialize data to a string

    string GetNodeName();        // @cmember Returns name of node
    string GetDescription();     // @cmember Returns description
    int GetIndex();                 // @cmember Returns index

    // @access Private Members
private:
    string   m_node;         // @cmember Name of node where query executed
    string   m_desc;         // @cmember Description for query
    int m_index;
};


class CYR_EXPORT CECVULogOutput : public CECTestOutput
{
    // @access Public Members
public:
    CECVULogOutput(long t_offsets, int nVUsers);            // @cmember Constructor
	CECVULogOutput(const char* strEntry);

    virtual ~CECVULogOutput();   // @cmember Destructor
    static const char* OUTPUT_TYPE; // @cmember Type of output

    CTTime GetTime();      // @cmember Get time of timer creation
    int GetVUsers();    // @cmember Returns type of output

    string GetType();    // @cmember Returns type of output
    string ToString();   // @cmember Serialize data to a string

    // @access Protected Members
protected:
	int m_nVUsers;
}
;
#endif
