/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrFFApi
// @module  Test Run Implementation | This module contains the implementation
//          of the CECFFTestRun class.  This class is simply a C++ wrapper
//          around the OpenSTARep COM interface IStFolder and IStFile.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrffApi/CECFFTestRun.h>
#include <CyrffApi/CECFFFile.h>
#include <Repository/CyrStHelper.inl>
#include <CyrffApi/CECFFTestResultsFlusher.h>
#include <CyrHelper.inl>
#include <iostream.h>
#include <CECSimpleParse.h>
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <direct.h>

///////////////////////////////////////////////////////////////////////////////
// Flush Limit
#define FLUSH_OUTPUT_SIZE 50
#define FLUSH_MEMORY_LIMIT 500

// Create a temporary list to use in the flusher thread...
CECFFTestResultsFlusher* FlusherThread = NULL; // Flusher Thread

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor is used to retrieve a Test Run from the OpenSTARep
//          database.
// @parm    Pointer to the parent Test Store.
// @parm    The name of the Test Run to retrieve.
CECFFTestRun::CECFFTestRun( CECFFFolder* pParent, const char* name )
        : CECFFFolder( pParent, name )
{
    // Prepare arrays of test output data
    PrepareArrays();

    // Parse fields in description to fill member variables
    string str_parse( GetDescription() );
    string::size_type startpos=0;
    string::size_type pipe_pos=startpos;
    // Extract fields
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_desc = str_parse.substr(pipe_pos,startpos-pipe_pos);
    pipe_pos=++startpos;
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_testName = str_parse.substr(pipe_pos,startpos-pipe_pos);
    pipe_pos=++startpos;
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_testVersion = str_parse.substr(pipe_pos,startpos-pipe_pos);
    pipe_pos=++startpos;
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_timeStart = str_parse.substr(pipe_pos,startpos-pipe_pos);
    pipe_pos=++startpos;
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_timeStop = str_parse.substr(pipe_pos,startpos-pipe_pos);
    pipe_pos=++startpos;
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_nodeName = str_parse.substr(pipe_pos,startpos-pipe_pos);
    pipe_pos=++startpos;
    startpos=str_parse.find_first_of(SEPARATOR ,pipe_pos);
    m_execName = str_parse.substr(pipe_pos,startpos-pipe_pos);
    m_readonly = true;
    m_new = false;

    // Read any previous entries back into test run
    for ( OutputTable::iterator i = m_output.begin(); i != m_output.end(); i++ )
        OpenList( i->first, i->second );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor is used when a brand new Test Run needs to be
//          created.
// @devnote Currently the Test version is hard-coded to be 1.  This is because
//          the COM interface does not appear to offer a method to get the
//          Folder version.  It will need changing when the interface does
//          allow this functionality, and also when Cyrano put versioned Tests
//          into Impact.
// @parm    Pointer to the parent Test Store.
// @parm    A description for this test run.
// @parm    The name of the node executing the test run.
// @parm    The name of the executor running the test.
CECFFTestRun::CECFFTestRun( CECFFFolder* pParent, const char* desc,
                            const char* node, const char* executor )
        : CECFFFolder( pParent, BuildTestRunName( pParent ).c_str(), desc )
{
    // Prepare arrays of test output data
    PrepareArrays();

    // Generate current time
    CTTime tm ;
    tm.now();
    char buf[20];
    sprintf( buf, "%d", tm.getSeconds() );
    m_timeStart = buf;
    m_timeStop  = m_timeStart;

    m_desc = desc;
    m_testName = pParent->GetName();
    m_testVersion = "1";
    m_nodeName = node;
    m_execName = executor;

    // Update description field in folder
    string fullDesc = BuildFullDescription();
    PutDescription( fullDesc.data() );

    m_readonly = false;
    m_new = true;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor writes out any collected test output to the
//          relevant files.
CECFFTestRun::~CECFFTestRun()
{}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method creates an array of test output types for use during the
//          test run.  Each type of test output will be placed in its own file.

void CECFFTestRun::PrepareArrays()
{
    // Add entries to output array
    m_output.insert( OutputTable::value_type(CECTestLog::OUTPUT_TYPE,new OutputList()) );
    m_output.insert( OutputTable::value_type(CECTimerOutput::OUTPUT_TYPE,new OutputList() ));
    m_output.insert( OutputTable::value_type(CECTestDiagnostic::OUTPUT_TYPE,new OutputList() ));
    m_output.insert( OutputTable::value_type(CECTestHistory::OUTPUT_TYPE,new OutputList() ));
    m_output.insert( OutputTable::value_type(CECTestReport::OUTPUT_TYPE,new OutputList() ));
    m_output.insert( OutputTable::value_type(CECOtaSimpleQuery::OUTPUT_TYPE,new OutputList() ));
    m_output.insert( OutputTable::value_type(CECOtaHeader::OUTPUT_TYPE,new OutputList() ));
    m_output.insert( OutputTable::value_type(CECVULogOutput::OUTPUT_TYPE,new OutputList() ));

    // And Initialize the offsets - these keep track of how many items have been flushed
    // in each array during a test run.
    m_OffsetMap[CECTestLog::OUTPUT_TYPE]            = 0;
    m_OffsetMap[CECTimerOutput::OUTPUT_TYPE]        = 0;
    m_OffsetMap[CECTestDiagnostic::OUTPUT_TYPE]     = 0;
    m_OffsetMap[CECTestHistory::OUTPUT_TYPE]        = 0;
    m_OffsetMap[CECTestReport::OUTPUT_TYPE]         = 0;
    m_OffsetMap[CECOtaSimpleQuery::OUTPUT_TYPE]     = 0;
    m_OffsetMap[CECOtaHeader::OUTPUT_TYPE]          = 0;
    m_OffsetMap[CECVULogOutput::OUTPUT_TYPE]          = 0;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the Test being run.
// @rdesc   The name of the Test.
inline string CECFFTestRun::GetTestName() const
{
    return m_testName;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns a description of the test run.
// @rdesc   The Test Run description.
inline string CECFFTestRun::GetDescription() const
{
    return m_desc;
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method sets a new description for the test run.
// @parm    The new test run description.
inline void CECFFTestRun::SetDescription( const char* desc )
{
    m_desc = desc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the node that is currently
//          executing the Test.
// @rdesc   The node name.
inline string CECFFTestRun::GetNodeName() const
{
    return m_nodeName;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the executor that is currently
//          executing the Test.
// @rdesc   The executor name.
inline string CECFFTestRun::GetExecutorName() const
{
    return m_execName;
}


///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the time the Test Run was started.  The time
//          format is suitable for creation of a CTTime object.
// @rdesc   The start time.
inline CTTime CECFFTestRun::GetTimeStarted() const
{
    return CTTime( atol( m_timeStart.c_str() ));
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the time the Test Run was stopped.  The time
//          format is suitable for creation of a CTTime object.
// @rdesc   The stop time.
inline CTTime CECFFTestRun::GetTimeStopped() const
{
    return CTTime( atol( m_timeStop.c_str() ));
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   As the time a Test Run stops cannot be determined at creation, this
//          method is required.  It necessarily causes an update to the
//          underlying COM interface.
void CECFFTestRun::StopTestRun()
{
    // Update parameter
    CTTime tm;
    tm.now();
    char buf[20];
    sprintf( buf, "%d", tm.getSeconds() );
    m_timeStop = buf;
    string fullDesc = BuildFullDescription();
    // Get underlying folder reference
    PutDescription( fullDesc.data() );

    m_readonly = true;
    OutputTable::iterator i;

    // Tidy memory
    m_iterator.clear();
    for ( i = m_output.begin(); i != m_output.end(); i++ )
    {
        // If a new array has been created then we need to write contents to a file
        if ( m_new == true )
            SaveList( i->first, i->second );
        delete i->second;
    }
    m_output.clear();
    // Delete the Flusher Object
    if (FlusherThread != NULL)
    {
        delete FlusherThread;
        FlusherThread = NULL;
    }
    //check everything in
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method builds all the Test Run parameters into a single
//          string.  This is done so that they can be stored in the description
//          filed of a Folder entry.
// @devnote This method is currently used because the Test Run is bodged to be
//          a Folder.  When the correct Test Run component is added then it
//          should be removed.
// @rdesc   The complete string.

inline string CECFFTestRun::BuildFullDescription()
{
    string fullDesc( m_desc );
    fullDesc += SEPARATOR;
    fullDesc += m_testName;
    fullDesc += SEPARATOR;
    fullDesc += m_testVersion;
    fullDesc += SEPARATOR;
    fullDesc += m_timeStart;
    fullDesc += SEPARATOR;
    fullDesc += m_timeStop;
    fullDesc += SEPARATOR;
    fullDesc += m_nodeName;
    fullDesc += SEPARATOR;
    fullDesc += m_execName;
    fullDesc += SEPARATOR;

    return fullDesc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method has only one purpose - to generate a unique name for
//          the test run.
// @rdesc   The test run name.
string CECFFTestRun::BuildTestRunName( CECFFFolder* pParent )
{
    // Determine name of Folder
    CTTime Tm;
    Tm.now();
    string name( Tm.Format( "%d-%m-%Y %H-%M-%S" ));

    // Ensure name is unique.  This is done by attempting to
    // create a new folder based on the parent directory path, plus
    // the current time and date.  To prevent multiple folders being
    // created on the same day a numerical extension is added to the
    // name.
    int count = 1, error = -1;
    char buf[5];
    while ( error == -1 )
    {
        sprintf( buf, ".%03d", count );
        string fullPath( pParent->GetFullPath() );
        fullPath += name;
        fullPath += buf;
        error = _mkdir( fullPath.c_str() );
        count++;
    }
    name += buf;  // Determine final name of Folder
    return name;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method retrieves a sequence of test output data as a
//          collection of strings.  This method assumes that all test output
//          can be stored as a single line in a text file as this is how the
//          data will be eventually stored.
// @rdesc   The sequence of strings.
// @parm    The type of output to retrieve.
inline OutputList* CECFFTestRun::GetOutputList( const char* outType )
{
    // Get correct entry from output table
    return m_output[outType];
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method adds a new test output entry to the relevant file it
//          is stored in.  This method assumes that all test output can be
//          stored as a single line in a text file.
// @devnote To prevent constant opening and closing of files the file is opened
//          at the first call to this method and remains open until the Test
//          Run object is destroyed.
// @parm    The type of output to add.
// @parm    The stringified output value.
void CECFFTestRun::AddTestOutput( const char* outType, const char* entry )
{

    // If the Test Run is read only then we cannot add any new data to the
    // class.
    if ( m_readonly == true )
    {
        return;
    }

    // Get correct entry from output table
    OutputList* outList = m_output[outType];

    // Create a new entry in the OutputList
    if ( outList != NULL )
    {
        outList->push_back( entry );

        // Having added the item to the list we now check to see if we have breached our
        // internal "memory" limit.
        if (outList->size() > FLUSH_MEMORY_LIMIT)
        {
            // Obtain the name of the list we are flushing
            string rwFilename = outType;

            // Currently this purge procedure does not affect the OTA headers
            // file which remains in memory. The reason for this is to ensure
            // that TestCommander (or any similar monitoring tool may dynamically
            // know what stats are being collected.
            if ( strcmp( rwFilename.c_str(), "OtaHeader.txt" ) != 0 )
            {
                // As we have collected sufficient info - flush it out to the results files
                // by creating a seperate flusher thread

                // First we see if we are still doing it from the last time we breached this limit
                if (FlusherThread == NULL)
                {
                    FlusherThread=new CECFFTestResultsFlusher();
                }

                // If idle then reserve it as we are about to use it...
                if (FlusherThread->useIfIdle())
                {
                    // Just go through the list breached - purging any entries above our "memory" limit...
                    if (PurgeList( rwFilename, outList))
                    {
                        // Keep track of the number of items we have removed so we can provide
                        // an offset
                        m_OffsetMap[outType] += FLUSH_OUTPUT_SIZE;

                        string fullpath = GetRootDirectory().c_str();
                        fullpath += GetFolderHierarchy();

                        // And kick off the Flusher Thread
                        FlusherThread->kickoff(fullpath, rwFilename, m_PurgeList);
                        return;
                    }
                    else
                    {                       // Nothing to purge so release our reservation on it so that other
                        // lists can use it
                        FlusherThread->setIdle();
                    }
                }
            }
        }
    }

}

inline void CECFFTestRun::AddTestOutput( CECTestOutput* pOutput )
{
    AddTestOutput( pOutput->GetType().c_str(), pOutput->ToString().c_str() );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method purges any written entries of the output list.
//
// @parm    The filename of the current set of output.
// @parm    The list of output strings.
bool CECFFTestRun::PurgeList( string rwFilename, OutputList*& outList)
{
    // As a sanity check we first ensure that there are some entries to purge
    if (outList->empty() )
    {
        return false;
    }

    // Ensure that nothing is left in from a previous purge
    while (!m_PurgeList.empty())
    {
        m_PurgeList.pop_front();
    }


    // Now remove the number of entries which have been written out via the
    // flusher thread.

    // It must be stressed that the flusher thread does the output of the data
    // to the files - not this routine. All we can do is "consider it done" for
    // us and proceed to purge the items we consider written

    // The amount of entries we purge is determined by the FLUSH_OUTPUT_SIZE define
    // which tells us how many items are being written (and hence need removing) per flush
    for (int i=0; i<FLUSH_OUTPUT_SIZE; i++)
    {
        // Remove it from the main output list
        string rwString = *(outList->begin());
        outList->pop_front();

        // And add it to our purge list (which is used by the the flusher thread). This
        // independent lists contains every string that we need purging so that we do not
        // pass around the main output list.
        m_PurgeList.push_back(rwString);
    }

    return (m_PurgeList.empty()) ? false : true;

}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method recreates a list of previously saved Test Output
//          information.  The specified file is opened and each line is read in
//          and becomes a single entry in the list of Test Output values.
// @parm    The name of the file containing the Test Output.
// @parm    The list that the Test Output will be stored in.
void CECFFTestRun::OpenList( string filename, OutputList*& outList )
{
    try
    {
        // Retrieve relevant file
        CECFFFile* pFile = new CECFFFile( this, filename.c_str() );
        pFile->CheckOut( false );

        // Try to open file
        string fullpath = pFile->GetFullPath();
        std::ifstream* f = new std::ifstream( fullpath.c_str());

        if ( f->good() )
        {
            // Determine size of file
            f->seekg(0L,std::ios::end);
            std::streampos size = f->tellg();
            f->seekg(0L,std::ios::beg);

            // Read contents into single tokenized string
            char* buf = new char[(long)size+1];
            buf[size] = '\0';
            f->read( buf, size );
            string str(buf);
            CECSimpleParse parse_str(str);

            // Read through string removing any tokens
            string entry = parse_str.getNext( '\n' );
            while( !entry.empty() )
            {
                // Remove any leading/trailing whitespace
                while(entry[0]==' '||entry[0]=='\r')entry=entry.erase(0,1);
                while(entry[entry.size()]==' '||entry[entry.size()]=='\r')entry=entry.erase(entry.size(),1);

                // If we still have an entry left then add it to string list
                if ( entry.length() > 0 )
                    outList->push_back( entry );

                // Get next entry
                entry = parse_str.getNext( '\n' );
            }

            delete [] buf;
        }

        // Close file and exit function
        delete f;
        delete pFile;
    }
    catch( ... )
    {
        // If it does not exist then just ignore exception
    }

}




///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method saves the entire contents of the output list to
//          several files within the Test Output folder.  Each string is saved
//          one per line in a standard ASCII file.  The file is then checked
//          into OpenSTARep.
// @parm    The filename of the current set of output.
// @parm    The list of output strings.
void CECFFTestRun::SaveList( string filename, OutputList*& outList )
{
    // If there are no entries in the sequence then do not check it into OpenSTARep
    if ( outList->empty() )
        return;

    // Get correct filename for file
    string fullpath = GetRootDirectory().c_str();
    fullpath += GetFolderHierarchy();
    string rwTmppath = fullpath;

    // Not forgetting that all purged entries go to a temporary file indentified
    // with a leading underscore
    rwTmppath += '_';
    rwTmppath += filename;

    fullpath += filename;

    // Wait for the Flusher thread to complete ...
    if (FlusherThread != NULL)
    {
        while (FlusherThread->isRunning())
        {
            CyrSleep(500);
        }
    }

    // If a purged file does exist - copy it to the correct file name - not worrying
    // about failure as the test may not of generated any purging at all any way - (And
    // the OTAHeaders file will definitely not have a temporary file associated with
    // it at all)
    if (::CopyFile(rwTmppath.c_str(), fullpath.c_str(), TRUE))
    {
        // And delete the temporary file
        CyrDeleteFile(rwTmppath.c_str());
    }

    try
    {

        // Open file for output and write contents to file
        std::ofstream* f = new std::ofstream( fullpath.c_str(), std::ios::out|std::ios::app|std::ios::binary);
        if( f->good() )
        {
            while( !outList->empty() )
            {
                string str = *(outList->begin());
                outList->pop_front();
                f->write( str.c_str(), str.length() );
                f->write( "\r\n", 2 );
            }
        }

        // Delete file - flushing contents to disk
        delete f;
    }
    catch(...)
    {
    }

    // Create file reference and check file into OpenSTARep
    CECFFFile* pFile;

    pFile = new CECFFFile( this, filename.c_str(), filename.c_str() );
    pFile->CheckIn( true );

    delete pFile;
}

inline string CECFFTestRun::GetFirstOutputString( const char* outType )
{
    // Reset iterator and start retrieving items
    m_iterator[outType] = m_output[outType]->begin();
    string ret  = GetNextOutputString( outType );
    return ret;
}

inline string CECFFTestRun::GetNextOutputString( const char* outType )
{
    // If we successfully retrieve an entry then continue
    OutputListIt i = m_iterator[outType]++;
    if ( i != m_output[outType]->end() )
    {
        return *i;
    }
    else
    {
        return "";
    }
}
inline string CECFFTestRun::GetOutputString( const char* outType, long Iter)
{
    string ret="";

    // And retrieve the correct entry based on how many items we have flushed...

    // i.e. The monitoring app knows nothing of the offset (what has been flushed
    // so far) and so we simply alter the requested item to account for this.
    Iter -= m_OffsetMap[outType];

    // And return if in invalid entry is specified...
    if (Iter < 0)
    {
        return ret;
    }

    // If we successfully retrieve an entry then continue
    OutputList* OL = m_output[outType];
    if(OL)
    {
        if(Iter<=OL->size())
        {
            ret=OL->at(Iter);
        }
    }

    return ret;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method retrieves the number of items that has been purged for
//          the specified list so far.
// @rdesc   The name of the output list
inline long CECFFTestRun::GetOutputOffset(const char* outType)
{

    long lOffset = m_OffsetMap[outType];

    return lOffset;
}

inline long CECFFTestRun::GetNoOutputEntries( const char* outType)
{
    // If we successfully retrieve an entry then continue
    OutputList* OL = m_output[outType];
    if(OL)
    {
        return (OL->size() + m_OffsetMap[outType]);
    }
    else
    {
        return 0;
    }
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method resets the iteration for timer output values.  It then
//          creates and returns the first timer in the sequence.  Note that
//          the returned pointer must be freed by the calling application else
//          there will be memory leaks.
// @rdesc   Pointer to the first timer output in the sequence.
CECTimerOutput* CECFFTestRun::GetFirstTimer()
{
    CECTimerOutput* timer = NULL;

    // If we successfully retrieve an entry then continue
    string str = GetFirstOutputString( CECTimerOutput::OUTPUT_TYPE );
    if ( !str.empty() )
        timer = new CECTimerOutput( str.c_str() );
    return timer;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method retrieves the next timer output value in the sequence.
//          It calls the underlying CORBA Test Run object to retrieve a single
//          string from the remote OpenSTARep database.  A timer output object is
//          then created from that string and returned to the calling code.
//          Note that the returned pointer must be deleted by the calling
//          application to prevent memory leaks.
// @rdesc   The next timer value in the sequence.
CECTimerOutput* CECFFTestRun::GetNextTimer()
{
    CECTimerOutput* timer = NULL;

    // If we successfully retrieve an entry then continue
    string str = GetNextOutputString( CECTimerOutput::OUTPUT_TYPE );
    if ( !str.empty() )
        timer = new CECTimerOutput( str.c_str() );
    return timer;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method resets the iteration for test log output values.  It
//          then creates and returns the first test log in the sequence.  Note
//          that the returned pointer must be freed by the calling application
//          else there will be memory leaks.
// @rdesc   Pointer to the first test log output in the sequence.
CECTestLog* CECFFTestRun::GetFirstTestLog()
{
    CECTestLog* log = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetFirstOutputString( CECTestLog::OUTPUT_TYPE );
    if ( !str.empty() )
        log = new CECTestLog( 0, str.c_str(), false );

    return log;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method retrieves the next test log value in the sequence.
//          It calls the underlying CORBA Test Run object to retrieve a single
//          string from the remote OpenSTARep database.  A test log object is
//          then created from that string and returned to the calling code.
//          Note that the returned pointer must be deleted by the calling
//          application to prevent memory leaks.
// @rdesc   The next test log value in the sequence.
CECTestLog* CECFFTestRun::GetNextTestLog()
{
    CECTestLog* log = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetNextOutputString( CECTestLog::OUTPUT_TYPE );
    if ( !str.empty() )
        log = new CECTestLog( 0, str.c_str(), false );

    return log;
}

CECOtaSimpleQuery* CECFFTestRun::GetFirstSimpleQuery()
{
    CECOtaSimpleQuery* query = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetFirstOutputString( CECOtaSimpleQuery::OUTPUT_TYPE );
    if ( !str.empty() )
        query = new CECOtaSimpleQuery( str.c_str() );

    return query;
}

CECOtaSimpleQuery* CECFFTestRun::GetNextSimpleQuery()
{
    CECOtaSimpleQuery* query = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetNextOutputString( CECOtaSimpleQuery::OUTPUT_TYPE );
    if ( !str.empty() )
        query = new CECOtaSimpleQuery( str.c_str() );

    return query;
}

CECTestHistory* CECFFTestRun::GetFirstHistory()
{
    CECTestHistory* history = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetFirstOutputString( CECTestHistory::OUTPUT_TYPE );
    if ( !str.empty() )
        history = new CECTestHistory( 0, str.c_str(), false );

    return history;
}

CECTestHistory* CECFFTestRun::GetNextHistory()
{
    CECTestHistory* history = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetNextOutputString( CECTestHistory::OUTPUT_TYPE );
    if ( !str.empty() )
        history = new CECTestHistory( 0, str.c_str(), false );

    return history;
}

CECTestReport* CECFFTestRun::GetFirstReport()
{
    CECTestReport* report = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetFirstOutputString( CECTestReport::OUTPUT_TYPE );
    if ( !str.empty() )
        report = new CECTestReport( 0, str.c_str(), false );

    return report;
}

CECTestReport* CECFFTestRun::GetNextReport()
{
    CECTestReport* report = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetNextOutputString( CECTestReport::OUTPUT_TYPE );
    if ( !str.empty() )
        report = new CECTestReport( 0, str.c_str(), false );
    return report;
}

CECOtaHeader* CECFFTestRun::GetFirstOTAHeader()
{
    CECOtaHeader* query = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetFirstOutputString( CECOtaHeader::OUTPUT_TYPE );
    if ( !str.empty() )
        query = new CECOtaHeader( str.c_str() );

    return query;
}

CECOtaHeader* CECFFTestRun::GetNextOTAHeader()
{
    CECOtaHeader* query = NULL;

    // If we have successfully retrieved an entry then continue
    string str = GetNextOutputString( CECOtaHeader::OUTPUT_TYPE );
    if ( !str.empty() )
        query = new CECOtaHeader( str.c_str() );

    return query;
}
