/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     Cyrdmn
// @module  Cyrano Daemon | This module contains the implementation of the
//          Cyrano Daemon.  The Cyrano Daemon starts up the CORBA services and
//          instantiates an instance of a <c FactoryImpl>.  It is intended for
//          use in a console application, but there is no reason it cannot be
//          integrated into any other style of application.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrHelper.inl>
#include <Daemon/CyrDmn.h>
#pragma warning (disable:4786)
#pragma warning (disable:4503)
#include <Logging/CECLog.h>

extern CECLog* flogger;
void * CyrDaemon_ReaperThread(void* args)
{
    CyrDaemon* Dmn=reinterpret_cast<CyrDaemon*>(args);
    Dmn->ReaperThread();
    return NULL;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The constructor is responsible for starting the CORBA Factory and
//          loading all the relevant Provider modules.
// @parm    Pointer to CORBA helper functionality.
// @parm    The name to be given to the Factory object.
CyrDaemon::CyrDaemon( CORBAServer* pServer, const char* name )
{
    // Initialise member variables
    m_name = name;
    m_running = true;
    m_server = pServer;

    // Start CORBA stuff
    StartFactory();

    // Load all Providers
    LoadProviderModules();

    // Start Reaper Thread
    StartReaper();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor is responsible for unloading the Provider modules
//          and shutting down any CORBA objects started by this class.
CyrDaemon::~CyrDaemon()
{
    // Stop Reaper Thread
    StopReaper();

    // Unload provider modules
    UnloadProviderModules();

    // Stop CORBA stuff
    StopFactory();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method starts the daemon process running.  The daemon can
//          block further input or allow processing to continue.  If processing
//          is stopped there is NO way to stop the daemon other than to
//          terminate the application.
// @parm    Determines whether the daemon continues processing or not.
void CyrDaemon::Run( bool cont /* = true */ )
{
    // The daemon has been started
    m_running = true;

    // Inform CORBA that everything is ready and then block or not
    // depending on the passed parameter.
    m_server->Run( );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is responsible for starting the CORBA Factory object.
// @xref    <mf CyrDaemon::StopFactory>
void CyrDaemon::StartFactory()
{
    const char* where = "CyrDaemon::StartFactory";

    try
    {
        // Start Factory with specified name
        m_factory = new FactoryImpl( m_server, m_name.c_str() );
    }
    catch( CORBA::Exception& )
    {
        throw CyrDaemonException( "Error starting CORBA services.",
                                  where, CYR_CORBA_INIT_FAILED );
    }
    catch( CyrCORBAException ex )
    {
        throw CyrDaemonException( ex.what().c_str(), where, ex.status() );
    }
    catch( ... )
    {
        throw CyrDaemonException( "Unknown exception", where, CYR_UNKNOWN_ERROR );
    }

    // Output success message and return
    flogger->AddTrace( CYR_DAEMON, 1, "Factory started." );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method shuts down any running CORBA services - including the
//          ORB and the BOA.
// @xref    <mf CyrDaemon::StartFactory>
void CyrDaemon::StopFactory()
{
    const char* where = "CyrDaemon::StopCORBA";

    try
    {
        // Get rid of Factory object
        m_factory->Shutdown();
        m_factory->_remove_ref();
    }
    catch( CORBA::Exception& ex )
    {
        string error = CorbaExceptionString( ex );
        throw CyrDaemonException( error.c_str(), where, CYR_CORBA_EXCEPTION );
    }
    catch( CyrCORBAException ex )
    {
        throw CyrDaemonException( ex.what().c_str(), where, ex.status() );
    }
    catch( ... )
    {
        throw CyrDaemonException( "Unknown exception", where, CYR_UNKNOWN_ERROR );
    }

    // Successfully shutdown CORBA Factory.
    flogger->AddTrace( CYR_DAEMON, 1, "Factory stopped." );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc    This method is responsible for loading any Provider modules.  It
//           works by determining a list of likely libraries to load which
//           will contain a Provider class.  Each library is then loaded, and
//           an initialisation function searched for.  If successful the
//           initialisation function is called and the Provider module is
//           ready.  The successful libraries are stored until the Daemon
//           shuts down when they will be unloaded.
// @xref     <mf CyrDaemon::UnloadProviderModules>
// @todo     This method currently works only for windows platforms.  Further
//           work is required for other platforms.  This may also require
//           further work on the CyrTools2 library.
void CyrDaemon::LoadProviderModules()
{
#ifdef _WIN32
#ifdef _DEBUG
    const char* pattern = "CyrVDK???d.dll";
#else
const char* pattern = "CyrVDK???.dll";
#endif
#else
    const char* pattern= "libCyrVDK???.so";
#endif

    const char* func    = "CyrVdkInitialise";

    // Calculate path for libraries
    string path = CyrGetModuleName();
    path = path.substr( 0, path.find_last_of( DIR_SLASH ) + 1 );

    // Construct a list of all libraries to load
    NameList lstName = CyrFindFiles( path.c_str(), pattern );

    // If list is empty then log event and return
    if ( lstName.empty() )
    {
        flogger->AddTrace( CYR_DAEMON, 1, "No Provider modules found." );
        return;
    }

    // Loop through library list and load them all
    for( NameList::iterator i=lstName.begin();i!=lstName.end();i++)
    {
        CTLibrary* lib = NULL;
        CTProcedure* proc = NULL;

        try
        {
            // Load library and locate relevant function
            const string& libName = *i;
            lib = new CTLibrary( libName.c_str() );
            proc = new CTProcedure( *lib, func );

            // Call initialisation function
            PFN_LIB_INIT pfn = (PFN_LIB_INIT)proc->getAddress();
            (*pfn)( m_name.c_str(), m_server );

            // Store module handle
            m_lstLib.push_back( lib );
        }
        catch(CTException& e)
        {
            string msg=string("Exception caught while loading library :")+e.what();
            flogger->AddTrace( CYR_DAEMON, 1, msg.c_str());
            if ( lib  != NULL )
                delete lib;

        }

        catch( ... )
        {
            string msg="Exception caught while loading library";
            flogger->AddTrace( CYR_DAEMON, 1, msg.c_str());
            if ( lib  != NULL )
                delete lib;
        }

        // Tidy memory
        if ( proc != NULL )
            delete proc;
    }

    // Successfully loaded all Provider modules
    flogger->AddTrace( CYR_DAEMON, 1, "All Providers loaded and initialised." );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is responsible for unloading any Provider modules that
//          were loaded using the <mf CyrDaemon::LoadProviderModules> method.
// @xref    <mf CyrDaemon::LoadProviderModules>
void CyrDaemon::UnloadProviderModules()
{
    // Successfully loaded all Provider modules
    flogger->AddTrace( CYR_DAEMON, 50, "All Providers being unloaded." );

    // Name of the uninitialisation function within the library
    const char* func = "CyrVdkUninitialise";

    // Unload all the previously loaded libraries
    for( LibList::iterator i=m_lstLib.begin();i!=m_lstLib.end();i++)
    {
        // Successfully loaded all Provider modules
        flogger->AddTrace( CYR_DAEMON, 100, "Another Provider being unloaded." );

        const CTLibrary* lib = *i;
        CTProcedure* proc = NULL;

        try
        {
            // Find uninitialisation function in library
            proc = new CTProcedure( *lib, func );

            // Call initialisation function
            PFN_LIB_UNINIT pfn = (PFN_LIB_UNINIT)proc->getAddress();
            (*pfn)();

            // Successfully loaded all Provider modules
            flogger->AddTrace( CYR_DAEMON, 100, "Successfully unloaded Provider." );

        }
        catch( ... )
        {
            // Only exception should occur is when function cannot
            // be found in library. Build and log error message
            string error( "Error unloading Provider: " );
            error += lib->getName();
            flogger->AddTrace( CYR_DAEMON, 1, error.c_str() );
        }

        // delete procedure
        if ( proc != NULL )
            delete proc;

        // Unload library

        // Successfully loaded all Provider modules
        flogger->AddTrace( CYR_DAEMON, 100, "Deleting lib object." );

        delete lib;
        // Successfully loaded all Provider modules
        flogger->AddTrace( CYR_DAEMON, 100, "Deleted lib object." );
    }

    // Empty module vector
    m_lstLib.clear();

    // Output debug message
    flogger->AddTrace( CYR_DAEMON, 1, "All Providers unloaded." );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is the Reaper thread.  This thread runs as a separate
//          process and destroys objects on a regular basis.
// @xref    <mf CyrDaemon::StartReaper>, <mf CyrDaemon::StopReaper>
void CyrDaemon::ReaperThread()
{
    // Log Reaper starting
    flogger->AddTrace( CYR_DAEMON, 1, "Reaper thread started." );

    // Loop until the daemon is being shut down
    while( m_running )
    {
        try
        {

            // Sleep for alloted time
            for ( int i = 0; i < TEN_MINUTES; i += TEN_SECONDS )
            {
                omni_thread::sleep( TEN_SECONDS );
                //if told to stop break out of this loop and fall through outer loop
                if(!m_running)break;
            }

            // Call Reaper function on factory
            flogger->AddTrace( CYR_DAEMON, 1, "Reaper thread activated." );
            //         m_factory->Reaper();
        }

        catch( CORBA::Exception& ex )
        {
            // We have caught a CORBA exception and need to tell the
            // main process.
            string error = CorbaExceptionString( ex );
            throw CyrDaemonException( error.c_str(), "CyrDaemon::ReaperThread",
                                      CYR_CORBA_EXCEPTION );
        }
        catch( ... )
        {
            flogger->AddTrace( CYR_DAEMON, 1, "Unknown exception caught" );
        }
    }

    // Log that the reaper has terminated
    flogger->AddTrace( CYR_DAEMON, 1, "Reaper thread terminated." );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method starts the Reaper thread.  The reaper thread is a
//          separate thread used for disposing of objects that exist after
//          their lease has run out.
// @xref    <mf CyrDaemon::ReaperThread>, <mf CyrDaemon::StopReaper>
void CyrDaemon::StartReaper()
{
    try
    {
        // Create Reaper thread and start it running.
        reaper = new omni_thread(&CyrDaemon_ReaperThread,this);
        reaper->start();
    }
    catch( ... )
    {
        flogger->AddTrace( CYR_DAEMON, 1, "Unable to start reaping thread." );
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method stops the Reaper thread. It does so by causing an
//          exception in the reaper code.  This exception is caught and the
//          thread should terminate.  This method waits for the reaper thread
//          to terminate before continuing.
// @xref    <mf CyrDaemon::StartReaper>, <mf CyrDaemon::ReaperThread>
void CyrDaemon::StopReaper()
{
    flogger->AddTrace( CYR_DAEMON, 1, "Stopping reaper thread." );

    // Request termination of thread.
    m_running = false;
    reaper->join(NULL);
}

