/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrStRepository
// @module  Folder Implementation | This module contains the implementation of
//          the CyrStFolder CORBA object.  This CORBA object is simply a CORBA
//          wrapper around the OpenSTARep COM interface IFolder.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStFolder_i.h>
#include <Repository/CyrStHelper.inl>

#ifdef __VDK002__
#include <Logging/CECLog.h>
extern CECLog* v2logger;
#endif

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor is for use by derived Folder types only.  It simply
//          prepares the underlying object for linking and does not retrieve any
//          values from the OpenSTARep database.  It is intended for items such as
//          TestRuns where the name is not known beforehand and specialised
//          construction is required.
// @parm    Pointer to the parent folder object.
CyrStFolderImpl::CyrStFolderImpl( CyrStProvider *pProvider )
        : CyrStLinkImpl( pProvider )
{
    // And add some tracing
#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 50, "Constructing a CyrStFolderImpl (for Derived Folders)" );
#endif
    m_com = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor creates a new Folder object based on a list
//          of construction parameters.  The Folder can retrieved either as a
//          root or child Folder.
// @parm    Pointer to the parent Provider object.
// @parm    List of construction parameters.
CyrStFolderImpl::CyrStFolderImpl( CyrStProvider* pProvider, ParamList_ptr params )
        : CyrStLinkImpl( pProvider )
{

    // And add some tracing
#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 50, "Constructing a CyrStFolderImpl (for New Folders)" );
#endif
    const char* where = "CyrStFolderImpl::CyrStFolderImpl";

    try
    {
        // Determine if we are retrieving a root folder
        CORBA::String_var strProject = params->GetValue( CyrStFolder::PARAM_PROJECT );

        // And add some tracing
#ifdef __VDK002__
        if ( strlen( strProject ) != 0 )
        {
            string msg ("Project Name is: " );
            msg+= strProject;
            v2logger->AddTrace( CYR_VDK002, 50, msg.c_str());
        }
#endif

        // If the project parameter was specified then we should retrieve a root
        // folder object.
        try
        {
            if ( strlen( strProject ) != 0 )
            {
#ifdef __VDK002__
                string msg("Retrieving existing project: " );
                msg+= strProject;
                v2logger->AddTrace( CYR_VDK002, 100,  msg.c_str() );
#endif
                // Locate correct Server and Project interfaces
                CECFFProject* pProject = (CECFFProject*)m_provider->GetPtr( strProject.in() );

                m_regName = pProject->GetName();

#ifdef __VDK002__
                string msg1 = "Registered name is: " + m_regName;
                v2logger->AddTrace( CYR_VDK002, 100,  msg1.c_str() );
#endif

                m_com = new CECFFFolder( pProject );
#ifdef __VDK002__
                v2logger->AddTrace( CYR_VDK002, 100, "m_com has been newed sucessfully"  );
#endif

                m_regName += "_root_folder";
            }
            else
            {
                // Try to retrieve named project
                CORBA::String_var strFolderName = params->GetValue( CyrFactory::PARAM_NAME );
                CORBA::String_var strParentFolder = params->GetValue( CyrStFolder::PARAM_PARENT );
                CORBA::String_var strFolderDesc = params->GetValue( CyrStFolder::PARAM_DESC );
                if ( strlen( strFolderName ) == 0 )
                    throw CyrRepositoryException( "Invalid name parameter", where, CYR_FOLDER_NOT_FOUND );
                if ( strlen( strParentFolder ) == 0 )
                    throw CyrRepositoryException( "Invalid parent folder parameter", where, CYR_FOLDER_NOT_FOUND );

                // Retrieve parent Folder interface
                CECFFFolder* pParentFolder = (CECFFFolder*)m_provider->GetPtr( strParentFolder.in() );
                if ( strlen( strFolderDesc ) == 0 )
                    m_com = new CECFFFolder( pParentFolder, strFolderName );
                else
                    m_com = new CECFFFolder( pParentFolder, strFolderName, strFolderDesc );
                m_regName = BuildItemPath( m_com );
                m_regName+=strFolderName;

                // And add some tracing
#ifdef __VDK002__
                string msg = "Retrieving root folder for named project: " + m_regName;
                v2logger->AddTrace( CYR_VDK002, 100,msg.c_str() );
#endif
            }
        }
        catch( CyrRepositoryException ex )
        {
#ifdef __VDK002__
            const char* what = ex.what().c_str();
            const char* where = ex.where().c_str();

            string msg = "Caught CyrRepositoryException Exception ";
            v2logger->AddTrace( CYR_VDK002, 100, msg.c_str() );
            v2logger->AddTrace( CYR_VDK002, 100, what );
            v2logger->AddTrace( CYR_VDK002, 100, where );
#endif
            return;
        }
        catch (...)
        {
#ifdef __VDK002__
            string msg = "Caught General Exception ";
            v2logger->AddTrace( CYR_VDK002, 100,  msg.c_str() );
#endif
            return;
        }

        // Calculate current user name
#ifdef __VDK002__
        string msg = "Calculate UserName";
        v2logger->AddTrace( CYR_VDK002, 100, msg.c_str() );
#endif

        CECFFServer* pServer = m_com->GetParentServer();
#ifdef __VDK002__
        msg = "retrieved Parent Server";
        v2logger->AddTrace( CYR_VDK002, 100, msg.c_str() );
#endif

        string user = pServer->GetUserName();
#ifdef __VDK002__
        msg = "UserName :"+user+" And store pointer";
        v2logger->AddTrace( CYR_VDK002, 100, msg.c_str() );
#endif


        // Store project interface in parent Provider
        m_regName = m_provider->StorePtr( m_com, m_regName.c_str(), user.c_str() );
    }
    catch (...)
    {
        // And add some tracing
#ifdef __VDK002__
        v2logger->AddTrace( CYR_VDK002, 50, "Caught exception on obtaining a CyrStFolder::PARAM_PROJECT" );
#endif
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor destroys the underlying COM interface as part of its
//          tidying up.
CyrStFolderImpl::~CyrStFolderImpl()
{
    // And add some tracing
#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 100, "Destroying CyrStFolderImpl" );
#endif
    // Tidy up any allocated memory
    if ( m_com != NULL )
    {
        delete m_com;
        m_com = NULL;
    }


#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 100, "Still Destroying CyrStFolderImpl" );
#endif

}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns a list of child Folders directly beneath this
//          Folder.  The list will be separated by '\|' characters and
//          terminated by two '\|' characters.
// @rdesc   The separated list of folder names.
// @parm    Will contain the number of Folders.
char* CyrStFolderImpl::GetFolders( long& count )
{
    return CORBA::string_dup( m_com->GetFolders( count ).c_str());
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc  This method returns a list of Files contained by this Folder.  The
//         list will be '\|' separated and terminated by two '\|' characters.
// @rdesc  The separated list of file names.
// @parm   Will contain the number of Files within this Folder.
char* CyrStFolderImpl::GetFiles( long& count )
{
    return CORBA::string_dup( m_com->GetFiles( count ).c_str());
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the Folder as recognised by
//          OpenSTARep.  It should be the same name as the one passed into the
//          constructor and therefore should not be needed.  However, it has
//          been added here for completeness.
// @rdesc   The name of the Folder.
char* CyrStFolderImpl::GetName()
{
    return CORBA::string_dup( m_com->GetName().c_str() );
}

void CyrStFolderImpl::Refresh()
{
    // And add some tracing
#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 50, "Performing Folder Refresh" );
#endif
    m_com->Refresh();
}
