/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrStRepository
// @module  Project Implementation | This module contains the implementation of
//          the CyrStProject CORBA object.  This CORBA object is simply a CORBA
//          wrapper around the OpenSTARep COM wrapper interface.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStProject_i.h>
#include <Repository/CyrStHelper.inl>
#include <CyrException.h>

#ifdef __VDK002__
#include <Logging/CECLog.h>
extern CECLog* v2logger;
#endif

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor extracts the relevant information from the
//          parameter list and either uses it to construct a new Project or
//          retrieve an existing one.
// @parm    Pointer to the parent Provider object.
// @parm    List of construction parameters.
CyrStProjectImpl::CyrStProjectImpl( CyrStProvider* pProvider, ParamList_ptr params )
{
    // And add some tracing
#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 50, "Constructing a CyrStProjectImpl." );
#endif
    const char* where = "CyrStProjectImpl::CyrStProjectImpl";

    // Assign member variables
    m_provider = pProvider;

    // Retrieve member variables
    CORBA::String_var name = params->GetValue( CyrFactory::PARAM_NAME );
    CORBA::String_var server = params->GetValue( CyrStProject::PARAM_SERVER );

    // If no name has been given then throw an exception
    if ( strlen( name ) == 0 )
        throw CyrRepositoryException( "Invalid name parameter", where, CYR_PROJECT_NOT_FOUND );
    if ( strlen( server ) == 0 )
        throw CyrRepositoryException( "Invalid server parameter", where, CYR_SERVER_NOT_FOUND );

    // Attempt to retrieve Project from OpenSTARep database.
    CECFFServer* pServer = (CECFFServer*)m_provider->GetPtr( server.in() );
    m_com = new CECFFProject( pServer, name );

    // Calculate current user name
    string user = pServer->GetUserName();

    // Store project interface in parent Provider
    m_regName = m_provider->StorePtr( m_com, name, user.c_str() );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor is responsible for destroying the underlying COM
//          interface.
CyrStProjectImpl::~CyrStProjectImpl()
{
    // And add some tracing
#ifdef __VDK002__
    v2logger->AddTrace( CYR_VDK002, 50, "Destroying a CyrStProjectImpl." );
#endif
    // Destroy underlying server reference
    m_provider->RemovePtr( m_regName );
    delete m_com;
    m_com = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Within the Reaper mechanism of the Factory, each object must be
//          given a unique name.  This method returns that name for this
//          particular object.  The name itself is generated by the Provider
//          when the underlying COM object is stored.  This name can be used
//          to increase the lifetime of the CORBA object.  The name itself is
//          generated from the name of the Project, plus the name of the
//          current user.  This should guarantee uniqueness unless the same
//          user wishes to create the same object again - in which case they
//          are responsible for producing a different name.
// @rdesc   The registered name.
char* CyrStProjectImpl::GetRegisteredName()
{
    // And add some tracing
#ifdef __VDK002__
    string msg = "Obtaining registered project name via CyrStProjectImpl: " + m_regName;
    v2logger->AddTrace( CYR_VDK002, 100,  msg.c_str());
#endif
    return CORBA::string_dup( m_regName.c_str() );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the Project as recognised by
//          OpenSTARep.  It should be the same name as the one passed into the
//          constructor and therefore should not be needed.  However, it has
//          been added here for completeness.
// @rdesc   The name of the Project.
char* CyrStProjectImpl::GetName()
{
    // And add some tracing
#ifdef __VDK002__
    string msg = "Obtaining project name via CyrStProjectImpl: " + m_com->GetName();
    v2logger->AddTrace( CYR_VDK002, 100,  msg.c_str());
#endif
    return CORBA::string_dup( m_com->GetName().c_str() );
}

