/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Server.cpp: implementation of the CYRServer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Server.h"
#include "Project.h"
#include "CECRegistryTools.h"
#include "solutions.h"
#include "CYRFFAPI\CECFFImpactFolders.h"

#define MAX_FOLDER_DEPTH 5
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
/////////////////////////////////////////////////////////////////////

CYRServer::CYRServer():
        m_bUsingRepository(false),
        m_pOpenProject(NULL)
{
    CECRegistryTools reg;
    m_strWorkingDirectory = reg.GetStringValue(HKEY_CURRENT_USER ,SOLUTIONS_SERVER_KEY ,"ProjectDir");
    if(m_strWorkingDirectory.Right(1) != "\\")
        m_strWorkingDirectory += "\\";
}

CYRServer::~CYRServer()
{
    CloseProject();
}

BOOL CYRServer::GetProjectList(CStringArray& arstrProjectList)
{
	CECRegistryTools reg;
    m_strWorkingDirectory = reg.GetStringValue(HKEY_CURRENT_USER ,SOLUTIONS_SERVER_KEY ,"ProjectDir");
    if(m_strWorkingDirectory.Right(1) != "\\")
        m_strWorkingDirectory += "\\";

    arstrProjectList.RemoveAll();

    BOOL bResult = true;
    GetProjectListFromDisk(arstrProjectList);

    m_nProjectCount = arstrProjectList.GetSize();
    return bResult;
}

void CYRServer::GetProjectListFromDisk(CStringArray& arstrProjectList)
{
    ASSERT(!m_bUsingRepository);

    arstrProjectList.RemoveAll();

    CString strKeyName = SOLUTIONS_SERVER_KEY;
    CString strValueName = "ProjectDir";


    WIN32_FIND_DATA WFD;
    HANDLE hFindFile = FindFirstFile(m_strWorkingDirectory + "*",&WFD);
    if(hFindFile == INVALID_HANDLE_VALUE)
        return;

    do
    {
        if(WFD.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
        {
            CString strDir = WFD.cFileName;
            if (strDir.GetAt(0) != '.')
            {
                arstrProjectList.Add(strDir);
            }
        }
    }
    while(FindNextFile(hFindFile,&WFD));
}

BOOL CYRServer::DeleteProject(const CString& strProject)
{
    if(m_pOpenProject != NULL && m_pOpenProject->GetName() == strProject)
    {
        m_strLastError = "Cannot delete the current project.\nClose the project and try again.";
        return false;
    }

    CYRProject Project(strProject,this);
    if(!Project.Delete())
    {
        m_strLastError = Project.GetLastError();
        return false;
    }
    return true;
}

CString CYRServer::GetOpenProjectName()
{
    if(m_pOpenProject)
        return m_pOpenProject->GetName();
    else
        return "";
}

CString CYRServer::GetFullOpenProjectName()
{
    if(m_pOpenProject)
        return m_pOpenProject->GetName();
    else
        return "";
}

BOOL CYRServer::OpenProject(const CString& strProject,BOOL bRefresh)
{
    if(m_pOpenProject)
    {
        delete m_pOpenProject;
        m_pOpenProject = NULL;
    }

    if(strProject.IsEmpty())
        return false;

    m_pOpenProject = new CYRProject(strProject,this);
    if(bRefresh)
        m_pOpenProject->Refresh();
    if(!m_pOpenProject->IsValid())
    {
        delete m_pOpenProject;
        m_pOpenProject = NULL;
        return false;
    }
    return true;
}

CString CYRServer::GetFolderPath(ProjectFolder nFolder)
{
    CString strPath = m_strWorkingDirectory + m_pOpenProject->GetName() + "\\";

    CString strFolder = GetFolderName(nFolder);
    if(!strFolder.IsEmpty())
        strPath += strFolder + "\\";

    return strPath;
}

BOOL CYRServer::CheckOutFile(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile)
{
    CFileStatus fs;
    if(!m_bUsingRepository)
    {
        //If we're not using the repository just check if the file exists
        BOOL bResult = CFile::GetStatus(GetFolderPath(nFolder) + strSubDir + "\\"+ strFile,fs);
        if(!bResult)
            m_strLastError.Format("Could not find file \"%s\"",strFile);

        return bResult;
    }
    return false;
}

BOOL CYRServer::CheckInFile(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile,const CString& strDescription)
{
    CFileStatus fs;
    if(!m_bUsingRepository)
    {
        //If we're not using the repository just check if the file exists
        BOOL bResult = CFile::GetStatus(GetFolderPath(nFolder) + strSubDir + "\\"+ strFile,fs);
        if(!bResult)
            m_strLastError.Format("Could not find file \"%s\"",strFile);

        return bResult;
    }


    return false;
}

CString CYRServer::GetFolderName(ProjectFolder nFolder)
{
    switch(nFolder)
    {
    case enCaptureFolder:
        return CECFFCaptureFolder::FOLDER_NAME;
        break;
    case enDataFolder:
        return CECFFDataFolder::FOLDER_NAME;
        break;
    case enDependenciesFolder:
        return CECFFDependancyFolder::FOLDER_NAME;
        break;
    case enNodesFolder:
        return CECFFNodeFolder::FOLDER_NAME;
        break;
    case enObjectCodeFolder:
        return CECFFObjectCodeFolder::FOLDER_NAME;
        break;
    case enParametersFolder:
        return CECFFParameterFolder::FOLDER_NAME;
        break;
    case enSourceFolder:
        return CECFFSourceFolder::FOLDER_NAME;
        break;
    case enTestsFolder:
        return CECFFTestFolder::FOLDER_NAME;
        break;
    case enRootFolder:
        break;
    }
    return "";
}

BOOL CYRServer::GetFileList(ProjectFolder nFolder,const CString& strSubDir, CStringArray& arstrFiles)
{
    arstrFiles.RemoveAll();

    if(!m_bUsingRepository)
    {
        CString strDir(GetFolderPath(nFolder) + strSubDir + "\\");

        CFileFind ff;
        BOOL bWorking = ff.FindFile(strDir + "*.*");
        while(bWorking)
        {
            bWorking = ff.FindNextFile();
            if(ff.IsDots() || ff.IsDirectory())
                continue;
            arstrFiles.Add(ff.GetFileName());
        }
        return true;
    }

    return false;
}

BOOL CYRServer::GetFolderList(ProjectFolder nFolder,const CString& strSubDir,CStringArray& arstrFolders)
{
    arstrFolders.RemoveAll();

    if(!m_bUsingRepository)
    {
        CString strDir(GetFolderPath(nFolder) + strSubDir + "\\");

        CFileFind ff;
        BOOL bWorking = ff.FindFile(strDir + "*.*");
        while(bWorking)
        {
            bWorking = ff.FindNextFile();
            if(ff.IsDirectory() && !ff.IsDots())
                arstrFolders.Add(ff.GetFileName());
        }
        return true;
    }

    return false;
}

void CYRServer::CloseProject()
{
    delete m_pOpenProject;
    m_pOpenProject = NULL;
}

BOOL CYRServer::DoesFileExist(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile)
{
    if(!m_bUsingRepository)
    {
        CFileStatus fs;
        CString strFilePath(GetFolderPath(nFolder) + strSubDir + "\\"+strFile);
        BOOL bFound = CFile::GetStatus(strFilePath,fs);
        if(!bFound)
            m_strLastError.Format("Could not find file \"%s\"",strFile);

        return bFound;
    }


    return false;
}

BOOL CYRServer::CreateProject(const CString& strProject,const CString& strDescription)
{
    CString strProjectPath = m_strWorkingDirectory + strProject + "\\";
    BOOL bResult = CreateDirectory(strProjectPath,NULL);
    bResult &= CreateDirectory(strProjectPath + "CAPTURE",NULL);
    bResult &= CreateDirectory(strProjectPath + "DATA",NULL);
    bResult &= CreateDirectory(strProjectPath + "DEPENDENCIES",NULL);
    bResult &= CreateDirectory(strProjectPath + "NODES",NULL);
    bResult &= CreateDirectory(strProjectPath + "OBJECTCODE",NULL);
    bResult &= CreateDirectory(strProjectPath + "PARAMETERS",NULL);
    bResult &= CreateDirectory(strProjectPath + "SOURCE",NULL);
    bResult &= CreateDirectory(strProjectPath + "TESTS",NULL);

    if(!bResult)
        m_strLastError = "Failed to create project directories";
    return bResult;
}

BOOL CYRServer::DeleteFile(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile)
{
    CFileStatus fs;
    CString strFilePath(GetFolderPath(nFolder));
    if(strSubDir.IsEmpty())
        strFilePath += strFile;
    else
        strFilePath += strSubDir + "\\" + strFile;

    return ::DeleteFile(strFilePath);
}

BOOL CYRServer::DeleteTest(const CString& strTest)
{
    //Delete the test folder
    BOOL bResult = DeleteFolder(enTestsFolder,strTest);

    // Delete the Test scripts
    bResult &= DeleteFile(enSourceFolder,"",strTest+".tss");

    bResult &= DeleteFile(enSourceFolder,"",strTest+"-TH.SCL");

    // Delete the test tof
    bResult &= DeleteFile(enObjectCodeFolder,"",strTest+".tof");
    bResult &= DeleteFile(enObjectCodeFolder,"",strTest+"-TH.tof");

    // Delete the test depends
    bResult &= DeleteFile(enDependenciesFolder,"",strTest+".tsd");
    bResult &= DeleteFile(enDependenciesFolder,"",strTest+"-TH.scd");

    return bResult;
}

BOOL CYRServer::DeleteFolder(ProjectFolder nFolder,const CString& strSubDir)
{
    // ... then delete the local folder
    CString strFolder = GetFolderPath(nFolder) + strSubDir + "\\";
    DeleteDirectory(strFolder);
    return true;
}

void CYRServer::DeleteDirectory(const CString& strDir)
{
	CFileStatus fs;
	if(!CFile::GetStatus(strDir + ".",fs))
	{
		return;
	}

    CFileFind* pFF = new CFileFind();
    BOOL bWorking = pFF->FindFile(strDir + "*");
    while(bWorking)
    {
        bWorking = pFF->FindNextFile();

        if(pFF->IsDots())
            continue;

        if(pFF->IsDirectory())
        {
            DeleteDirectory(pFF->GetFilePath() + "\\");
            ::RemoveDirectory(pFF->GetFilePath());
        }
        else
            ::DeleteFile(pFF->GetFilePath());
    }

    delete pFF;

    BOOL b = ::RemoveDirectory(strDir);
    if (!b)
    {
        LPTSTR lpMsgBuf;

        FormatMessage(  FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                        NULL,
                        ::GetLastError(),
                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                        (LPTSTR) &lpMsgBuf,
                        0,
                        NULL);

        AfxMessageBox (lpMsgBuf);
    }
}

int CYRServer::GetProjectCount()
{
    static DWORD dwLastTime = 0;

    //Update this count only if 2 seconds have passed since last request
    //this is done because the count is requested every time the mouse moves
    //in the onupdatecommand handlers
    if(!dwLastTime || ((GetTickCount()-dwLastTime)>2000))
    {
        CStringArray arstrProjects;
        GetProjectList(arstrProjects);
        m_nProjectCount = arstrProjects.GetSize();
        dwLastTime = GetTickCount();
    }

    return m_nProjectCount;
}
