/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_DECODE_FKEY                 */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This module contains a routine that will return the <ESC>ape         */
/*  sequence for a PHYSICAL function key or vise versa.          */
/*                                       */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>
#include <string.h>

/*
 *  Function prototypes.
 */
#include <psl_rtndef.h>         /* Define ourselves */

/*
 *  Structure definitions.
 */
#include <shrmsg.h>         /* Message Statuses */


/************************************************************************/
/*                                  */
/*  PSL_DECODE_FKEY - Convert Escape Sequence/Physical Keys     */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*      iret = PSL_DECODE_FKEY (inp_string, inp_string_length,  */
/*              out_string, out_string_length)      */
/*                                  */
/*  Where:                              */
/*                                  */
/*      iret              = Unsigned Long PSL Routine Status:   */
/*                  PSL_NORMAL    - Match Found     */
/*                  PSL_NOMTCHFOU - NO Match Found      */
/*      inp_string        = Pointer to Character String to match.   */
/*                  Either Physical Name or Escape Sequence.*/
/*      inp_string_length = Unsigned Long Length of inp_string.     */
/*      out_string        = Pointer to Character Output String.     */
/*                  Opposite value inp_string.      */
/*                  Should be at least 11 characters.   */
/*      out_string_length = Pointer to Unsigned Long Return length  */
/*                  of out_string.              */
/*                                  */
/*  This routine is passed either the ESCAPE SEQUENCE of a function */
/*  key and the PHYSICAL NAME is returned, or the PHYSICAL NAME     */
/*  is passed and the ESCAPE SEQUENCE is returned.          */
/*                                  */
/************************************************************************/

unsigned long   PSL_DECODE_FKEY (     char *inp_string
                                      ,unsigned long inp_string_length
                                      ,         char *out_string
                                      ,unsigned long *out_string_length)
{
    /*
     *  Local data
     *  
     *      Define and fill a table to hold all of the VT... function keys and
     *      their corresponding escape sequence.  
     *  Note that in the following table, the sequence <ESC>[ is equivalent
     *  to <CSI>, and the sequence <ESC>O is equivalent to <SS3>.
     */
#define FKEY_MAX 54         /* 54 Function keys supported */
    static  struct table
    {
        char fkey_name[12];       /* Function key name */
        char escape_seq[6];       /* Escape sequence */
    }
    fkey_table[FKEY_MAX] = {
        {"HELP",  "\x1B[28~"},
        {"DO",    "\x1B[29~"},
        {"FIND",  "\x1B[1~"},
        {"INSERT-HERE","\x1B[2~"},
        {"REMOVE",    "\x1B[3~"},
        {"SELECT",    "\x1B[4~"},
        {"PREV-SCREEN","\x1B[5~"},
        {"NEXT-SCREEN","\x1B[6~"},
        {"PF1",   "\x1BOP"},
        {"PF2",   "\x1BOQ"},
        {"PF3",   "\x1BOR"},
        {"PF4",        "\x1BOS"},
        {"KP0",        "\x1BOp"},
        {"KP1",        "\x1BOq"},
        {"KP2",        "\x1BOr"},
        {"KP3",        "\x1BOs"},
        {"KP4",        "\x1BOt"},
        {"KP5",        "\x1BOu"},
        {"KP6",        "\x1BOv"},
        {"KP7",        "\x1BOw"},
        {"KP8",        "\x1BOx"},
        {"KP9",        "\x1BOy"},
        {"MINUS",      "\x1BOm"},
        {"COMMA",      "\x1BOl"},
        {"PERIOD",     "\x1BOn"},
        {"ENTER",      "\x1BOM"},
        {"UP",         "\x1B[A"},
        {"DOWN",       "\x1B[B"},
        {"RIGHT",      "\x1B[C"},
        {"LEFT",       "\x1B[D"},
        {"A-UP",       "\x1BOA"},
        {"A-DOWN",     "\x1BOB"},
        {"A-RIGHT",    "\x1BOC"},
        {"A-LEFT",     "\x1BOD"},
        {"F1",         "\x1B[11~"},
        {"F2",         "\x1B[12~"},
        {"F3",         "\x1B[13~"},
        {"F4",         "\x1B[14~"},
        {"F5",         "\x1B[15~"},
        {"F6",         "\x1B[17~"},
        {"F7",         "\x1B[18~"},
        {"F8",         "\x1B[19~"},
        {"F9",         "\x1B[20~"},
        {"F10",        "\x1B[21~"},
        {"F11",        "\x1B[23~"},
        {"F12",        "\x1B[24~"},
        {"F13",        "\x1B[25~"},
        {"F14",        "\x1B[26~"},
        {"F15",        "\x1B[28~"},
        {"F16",        "\x1B[29~"},
        {"F17",        "\x1B[31~"},
        {"F18",        "\x1B[32~"},
        {"F19",        "\x1B[33~"},
        {"F20",        "\x1B[34~"}
    };
    long    isub;           /* Loop control */
    char    *input_ptr;     /* Input pointer */
    char    *table_ptr;     /* Table pointer */
    unsigned long   input_length;       /* Input length */
    long    comp_status;        /* Comparison status */

    /*
     *  Is this an ESCAPE sequence ??
     */
    if ((*inp_string == '\x1B')     /* First character <ESC> ?? */
            || (*inp_string == '\x9B')     /* First character <CSI> ?? */
            || (*inp_string == '\x8F'))    /* First character <SS3> ?? */

        /*
         *  The Escape Sequence of the key has been supplied, so check
         *  the table for a match on escape sequence
         */
    {
        for (isub = 0; isub < FKEY_MAX; isub++)
        {
            /*
             *  Point to input and table entry
             */
            input_ptr = inp_string;
            table_ptr = fkey_table[isub].escape_seq;
            input_length = inp_string_length;

            /*
             *  If input string starts with <CSI>, check for table entry
             *  starting with <ESC>[
             */
            if (*input_ptr == '\x9B')
            {
                comp_status = strncmp ("\x1B[", table_ptr, 2);
                if (comp_status == 0)
                {
                    input_ptr += 1; /* Step over <CSI> */
                    input_length -= 1;
                    /* Reduce length accordingly */
                    table_ptr += 2; /* Step over <ESC>[ */
                }
            }
            /*
             *  If input string starts with <SS3>, check for table entry
             *  starting with <ESC>O
             */
            else if (*input_ptr == '\x8F')
            {
                comp_status = strncmp ("\x1BO", table_ptr, 2);
                if (comp_status == 0)
                {
                    input_ptr += 1; /* Step over <SS3> */
                    input_length -= 1;
                    /* Reduce length accordingly */
                    table_ptr += 2; /* Step over <ESC>O */
                }
            }
            /*
             *  Compare strings
             */
            if (strlen(table_ptr) != input_length)
            {
                comp_status = 1;    /* Mismatch if not same length */
            }
            else
            {
                comp_status = strncmp (input_ptr, table_ptr, input_length);
            }
            /*
             *  If escape sequence matched, return equivalent function key string
             *  and its length.
             */
            if (comp_status == 0)
            {
                (void) strcpy (out_string, fkey_table[isub].fkey_name);
                *out_string_length = strlen (out_string);
                return PSL_NORMAL;
            }
        }
    }
    else
        /*
         *  The function key name has been supplied, so check the table 
         *  for a match on name
         */
    {
        for (isub = 0; isub < FKEY_MAX; isub++)
        {
            table_ptr = fkey_table[isub].fkey_name;
            if (strlen(table_ptr) != inp_string_length)
            {
                comp_status = 1;    /* Mismatch if not same length */
            }
            else
            {
                comp_status = strncmp (inp_string, table_ptr,
                                       inp_string_length);
            }
            /*
             *  If function key name matched, return equivalent escape sequence
             *  and its length.
             */
            if (comp_status == 0)
            {
                (void) strcpy (out_string, fkey_table[isub].escape_seq);
                *out_string_length = strlen (out_string);
                return PSL_NORMAL;
            }
        }
    }
    /*
     *  We have reached the end of the table - no match was found...
     */
    return PSL_NOMTCHFOU;       /* Return no match status */
}
