/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_TPATRACE                */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This is a simple module for tracing PSL_TPARSE State Tables.         */
/*                                       */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  Standard ANSI header files
 */
#include <stdio.h>
#include <ctype.h>
#include <string.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL Function Prototypes */

/*
 * Structure definitions.
 */
#include <psl_tpaargdef.h>      /* TPARSE Argument Block */
#include <psl_tpatabdef.h>      /* TPARSE Table Entry */
#include <shrmsg.h>         /* PSL Messages */


/************************************************************************/
/*                                  */
/*  PSL_TPATRACE - Trace TPARSE activity                */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     PSL_TPATRACE (argblk_ptr, type)              */
/*                                  */
/*  Where:                              */
/*                                  */
/*     argblk_ptr = Pointer to PSL_TPARSE argument block        */
/*     type = short int Call Type:                  */
/*       1 => Token Match Failed                */
/*       2 => Action Routine Failed             */
/*       3 => Transition Succeeded              */
/*       4 => No Next Transition                */
/*       5 => About to enter Sub-expression         */
/*                                  */
/************************************************************************/

void    PSL_TPATRACE (struct tpaarg *argblk_ptr, /* Pointer to argument block */
                      short type)        /* Type code */
{
    /*
     *  Local Data.
     */
    struct tpatab   *state_ptr; /* Pointer to current state */
    char        *name_ptr;  /* Pointer to name */
    long        outcnt;     /* Output count */
    char        new_string[40]; /* New string */
#define REASON_OFFSET   57      /* Offset of start of reason */

    /*
     *   Set up a pointer to the TPARSE block and see if we have a State Name 
     *   to output at this point.
     */
    state_ptr = (*argblk_ptr).trans_ptrps;
    /* Point to current transition */
    name_ptr = (*state_ptr).stanam_ptrpc;
    /* Point to state name */
    if ((name_ptr != NULL)      /* If there is a name */
            && (type < 4))         /* And not secondary trace entry */
    {
        (void) printf ("\nSTATE %s\n", name_ptr);
        /* Output the state name */
    }
    outcnt = 0;         /* Initialize output count */

    /*
     *   Output "TRAN"
     */
    outcnt += printf ("TRAN  ");

    /*
     *   Case according to token type
     */
    switch ((*state_ptr).token_typeb)
    {
    case PSLTPA_KEYWORD:
        {
            /*
             *   PSLTPA_KEYWORD - Multi-character keyword
             *   Just output the keyword in quotes.
             */
            outcnt += printf ("'%s'", (*state_ptr).keyword_ptrpc);
            /* Output quoted keyword */
            break;
        }

    case PSLTPA_EOS:
        {
            /*
             *   PSLTPA_EOS - End of String
             */
            outcnt += printf ("PSLTPA_EOS");
            /* Output PSLTPA_EOS */
            break;
        }

    case PSLTPA_ANY:
        {
            /*
             *   PSLTPA_ANY - Any single character
             */
            outcnt += printf ("PSLTPA_ANY");
            /* Output PSLTPA_ANY */
            break;
        }

    case PSLTPA_ALPHA:
        {
            /*
             *   PSLTPA_ALPHA - Any alphabetic character
             */
            outcnt += printf ("PSLTPA_ALPHA");
            /* Output PSLTPA_ALPHA */
            break;
        }

    case PSLTPA_DIGIT:
        {
            /*
             *   PSLTPA_DIGIT - Any Digit 
             */
            outcnt += printf ("PSLTPA_DIGIT");
            /* Output PSLTPA_DIGIT */
            break;
        }

    case PSLTPA_STRING:
        {
            /*
             *   PSLTPA_STRING - Any alphanumeric string
             */
            outcnt += printf ("PSLTPA_STRING");
            /* Output PSLTPA_STRING */
            break;
        }

    case PSLTPA_SYMBOL:
        {
            /*
             *   PSLTPA_SYMBOL - Any symbol
             */
            outcnt += printf ("PSLTPA_SYMBOL");
            /* Output PSLTPA_SYMBOL */
            break;
        }

    case PSLTPA_BLANK:
        {
            /*
             *   PSLTPA_BLANK - Any sequence of blanks and tabs
             */
            outcnt += printf ("PSLTPA_BLANK");
            /* Output PSLTPA_BLANK */
            break;
        }

    case PSLTPA_DECIMAL:
        {
            /*
             *   PSLTPA_DECIMAL - Any decimal number
             */
            outcnt += printf ("PSLTPA_DECIMAL");
            /* Output PSLTPA_DECIMAL */
            break;
        }

    case PSLTPA_OCTAL:
        {
            /*
             *   PSLTPA_OCTAL - Any octal number
             */
            outcnt += printf ("PSLTPA_OCTAL");
            /* Output PSLTPA_OCTAL */
            break;
        }

    case PSLTPA_HEX:
        {
            /*
             *   PSLTPA_HEX - Any hexadecimal number
             */
            outcnt += printf ("PSLTPA_HEX");
            /* Output PSLTPA_HEX */
            break;
        }

    case PSLTPA_SUBXPR:
        {
            /*
             *   PSLTPA_SUBXPR - Subexpression
             *   If we're lucky then the user stored the name for us.  If not then 
             *   we output a fake name instead.
             */
            name_ptr = (*state_ptr).subnam_ptrpc;
            /* Point to subexpression name */
            if (name_ptr != NULL)   /* If there is a name */
            {
                outcnt += printf ("!%s", name_ptr);
                /* Output the name */
            }
            else
            {
                outcnt += printf ("!<subexpr>");
                /* Output dummy name */
            }
            break;
        }

    case PSLTPA_LAMBDA:
        {
            /*
             *   PSLTPA_LAMBDA - always matches
             */
            outcnt += printf ("PSLTPA_LAMBDA");
            /* Output PSLTPA_LAMBDA */
            break;
        }

    case PSLTPA_CHAR:
        {
            /*
             *   PSLTPA_CHAR - Single character
             *   See if 'printable' and, if not, output as hexadecimal value
             */
            if (isprint((*state_ptr).token_charb))
            {
                outcnt += printf ("'%c'", (*state_ptr).token_charb);
                /* Output quoted character */
            }
            else
            {
                outcnt += printf ("^X%X", (int) (*state_ptr).token_charb);
                /* Output in hexadecimal */
            }
            break;
        }

    default:
        {
            break;
        }

    }
    /*
     *   If we have a target and/or an action routine, output a
     *   divider and, if present, the target
     */
    if (((*state_ptr).target_ptrps != NULL) ||
            ((*state_ptr).action_ptrpf != NULL) ||
            ((*state_ptr).quitb != 0))  /* Target or action routine? */
    {
        outcnt += printf (", ");    /* Yes - insert the divider */
        if ((*state_ptr).target_ptrps != NULL)
            /* If there is a target */
        {
            name_ptr = (*state_ptr).tgtnam_ptrpc;
            /* Point to target name */
            if (name_ptr != NULL)   /* If there is a name */
            {
                outcnt += printf ("%s", name_ptr);
                /* Output the name */
            }
            else
            {
                outcnt += printf ("<target>");
                /* Output dummy name */
            }
        }
        else if ((*state_ptr).quitb == PSLTPA_EXIT)
        {
            outcnt += printf ("PSLTPA_EXIT");
        }
        else if ((*state_ptr).quitb == PSLTPA_FAIL)
        {
            outcnt += printf ("PSLTPA_FAIL");
        }
        /*
         *   If we have an action routine, output a divider and the routine name
         */
        if ((*state_ptr).action_ptrpf != NULL)
            /* If there is an action routine */
        {
            name_ptr = (*state_ptr).actnam_ptrpc;
            /* Point to action routine name */
            if (name_ptr != NULL)   /* If there is a name */
            {
                outcnt += printf (", %s", name_ptr);
                /* Output the name */
            }
            else
            {
                outcnt += printf (", <action_rtn>");
                /* Output dummy name */
            }
        }
    }
    /*
     *   Add reason string to end of line.  Try to align on column 80 if poss.
     */
    if (outcnt < REASON_OFFSET) /* If before the standard offset */
    {
        outcnt += printf ("%*s", (int) (REASON_OFFSET - outcnt), " ");
        /* Pad with spaces */
    }
    if (type == 1)
    {
        outcnt += printf ("! Token Match Failed");
    }
    else if (type == 2)
    {
        outcnt += printf ("! Action Rtn Failed");
    }
    else if (type == 3)
    {
        outcnt += printf ("! Transition Succeeded");
    }
    else if (type == 4)
    {
        outcnt += printf ("! End of State");
    }
    else if (type == 5)
    {
        outcnt += printf ("! Subexpression Call");
    }

    (void) printf ("\n");       /* New line */

    /*
     *   If the call is for a Successful Transition then output the
     *   new string to be parsed as well.
     */
    if (type == 3)
    {
        outcnt = (*argblk_ptr).stringcntl;
        /* Length of new string */
        if (outcnt > (sizeof (new_string) - 1))
            /* If more than will fit */
        {
            outcnt = sizeof (new_string) - 1;
            /* Truncate it */
        }
        (void) memcpy (new_string, (*argblk_ptr).string_ptrpc, outcnt);
        /* Copy string to work buffer */
        new_string [outcnt] = '\0'; /* Null-terminate it */
        (void) printf ("  New String = \"%s\"\n",new_string);
        /* Output new string */
    }

    return;
}
