/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/* scl_def_routines.c:
 * ===================
 *
 * This module contains routines to convert test definition data
 * between its internal and external representations.
 */

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL Routinesc */
#include "scl_rtndef.h"         /* SCL Routines */
#include <vtest_rtndef.h>       /* V-TEST Routines */

/*
 *    Local routine function prototypes.
 */
static unsigned long    VTEST_UNDEFAULT_TST (struct diagstk *diagstk_ptr
        ,   char    *src_test_ptr
        ,   char    *def_test_ptr
        ,   char    *out_test_ptr);

static unsigned long    VTEST_UNDEF_TRCFIL  (struct diagstk *diagstk_ptr,
        struct tlh *src_tlh_ptr,
        struct prm *src_prm_ptr,
        struct tdf *src_tdf_ptr,
        char **trcfil_pptr);

static unsigned long    VTEST_UNDEF_RECFIL  (struct diagstk *diagstk_ptr,
        struct tlh *src_tlh_ptr,
        struct prm *src_prm_ptr,
        struct tdf *src_tdf_ptr,
        char **recfil_pptr);

static void VTEST_UNDEFAULT_TLV (struct diagstk *diagstk_ptr
                                 ,struct tlv     *src_tlv_ptr
                                 ,struct tlv     *def_tlv_ptr
                                 ,struct tlv     *out_tlv_ptr);

static long     VTEST_LOGIN_TYPE_DEF    (long terminal_type);

static void VTEST_DEFAULT_TLV   (struct diagstk *diagstk_ptr
                                 ,struct tlv     *tlv_ptr);

static long     VTEST_TIMING_DEF    (long terminal_type);

static long     VTEST_PORT_DEF      (long login_type);

static void SETUP_INT_TDF_DEF   (struct diagstk *diagstk_ptr
                                 ,struct tdf     *int_tdf_ptr);

static void SETUP_INT_TLV_DEF   (struct diagstk *diagstk_ptr
                                 ,struct tlv     *int_tlv_ptr);

static void SETUP_INT_PRM_DEF   (struct diagstk *diagstk_ptr
                                 ,struct prm     *int_prm_ptr);

static void SUBSTR_COMMON       (char *wrkstr_ptr,
                                 unsigned long wrkstr_size);

/*
 *  Structure Definitions.
 */
#include <shrmsg.h>         /* PSL Messages         */
#include "scl_tlodef.h"         /* Test structures layout   */
#include <psldiag_prmdef.h>     /* Diagnostic Routine Parameters */
#include <tofprmdef.h>          /* TOF parameters */
#include <vtest_prmdef.h>       /* VTEST Parameters     */

/************************************************************************/
/*                                  */
/*    VTEST_UNDEFAULT_TST - Translate default values in a TEST      */
/*    ===================    read in from a file.                   */
/*                                  */
/*    Calling Format:                           */
/*  iret = VTEST_UNDEFAULT_TST (diagstk_ptr, src_test_ptr       */
/*              ,def_test_ptr ,out_test_ptr);       */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  src_test_ptr = Pointer to the TEST to translate             */
/*  def_test_ptr = Pointer to defaults TEST             */
/*            (May be a NULL pointer if no defaults record) */
/*  out_test_ptr = Pointer to the translated Test to set up.    */
/*            (May be same as src_test_ptr to re-use buffer)    */
/*                                  */
/*    Return Status                             */
/*  iret        = PSL Return Status                 */
/*                                  */
/*    This routine is called after reading a complete test to translate */
/*    any fields contained therein which are flagged as being default   */
/*    so that they contain actual values.               */
/*                                  */
/*    If a field within a record is defined as being 'defaulted' then   */
/*    we will check the appropriate record in the DEFAULTS record if    */
/*    one was passed. If the field in the DEFAULTS record is also set   */
/*    to default then the internal default is used.         */
/*                                  */
/*    NOTE: The DEFAULT test contains only 3 records, the TLH, PRM and  */
/*    a single TDF record. There is no DEFAULT record for TLV's at  */
/*    present.                              */
/*                                  */
/************************************************************************/

static unsigned long    VTEST_UNDEFAULT_TST (struct diagstk *diagstk_ptr
        ,   char    *src_test_ptr
        ,   char    *def_test_ptr
        ,   char    *out_test_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   iret;           /* Routine Return Status */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */
    long   isub;           /* Loop control variable.   */
    void   *void_ptr;      /* Work pointer to VOID     */

    struct   tlh    *src_tlh_ptr;       /* pointer to Source TLH    */
    struct   tlh    *out_tlh_ptr;       /* pointer to Output TLH    */
    struct   tlh    *def_tlh_ptr = NULL;    /* pointer to defaults TLH  */

    struct   tdf    *src_tdf_ptr;       /* pointer to Source TDF    */
    struct   tdf    *out_tdf_ptr;       /* pointer to Output TDF    */
    struct   tdf    *def_tdf_ptr = NULL;    /* pointer to defaults TDF  */

    struct   prm    *src_prm_ptr;       /* pointer to Source PRM    */
    struct   prm    *out_prm_ptr;       /* pointer to Output PRM    */
    struct   prm    *def_prm_ptr = NULL;    /* pointer to defaults PRM  */

    struct   tlv    *src_tlv_ptr;       /* pointer to Source TLV    */
    struct   tlv    *out_tlv_ptr;       /* pointer to Output TLV    */


    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During VTEST_UNDEFAULT_TST");

    /*
     *    Set up a pointer to the script header record of the source and output
     *    tests. So that we know where to start. Note that we may not have a 
     *    default test so we should be careful when setting up a pointer to its
     *    header record.
     */
    void_ptr = (void *) src_test_ptr;
    src_tlh_ptr = (struct tlh *) void_ptr;

    void_ptr = (void *) out_test_ptr;
    out_tlh_ptr = (struct tlh *) void_ptr;

    if (def_test_ptr != NULL)
    {
        void_ptr = (void *) def_test_ptr;
        def_tlh_ptr = (struct tlh *) void_ptr;
    }

    /*
     *    Move forward to the PRM record and convert it.
     */
    src_prm_ptr = (struct prm *) (src_tlh_ptr + 1);
    /* PRM record follows TLH   */
    out_prm_ptr = (struct prm *) (out_tlh_ptr + 1);
    /* PRM record follows TLH   */
    if (def_tlh_ptr != NULL)
    {
        def_prm_ptr = (struct prm *) (def_tlh_ptr + 1);
        /* PRM record follows TLH   */
    }

    /*
     *    ...and translate the PRM record first.
     */
    iret = SCL_UNDEFAULT_PRM (diagstk_ptr ,src_tlh_ptr
                              ,src_prm_ptr ,def_prm_ptr ,out_prm_ptr);
    if (!(psl_test_status (iret))) goto end;

    /*
     *    Step forward to the TLV/TDF records and translate them as required.
     */
    if ((*src_tlh_ptr).tlh_testtypeb == TEST_TYPE_TERMINALS)
    {                   /* If the source test contains terms */
        src_tdf_ptr = (struct tdf *) (src_prm_ptr + 1);
        /* Get pointer to first Source TDF   */
        out_tdf_ptr = (struct tdf *) (out_prm_ptr + 1);
        /* ...and to first Output TDF        */
        if (def_prm_ptr != NULL)    /* ...and if we have a defaults rec  */
        {
            def_tdf_ptr = (struct tdf *) (def_prm_ptr + 1);
            /* ...get DEFAULTS TDF (if any)      */
        }

        /*
         *    Loop through the TDF's translating each as we go.
         */
        for (isub = 0;
                isub < (*src_tlh_ptr).tlh_testcntl;
                isub++, src_tdf_ptr++ ,out_tdf_ptr++)
        {
            iret = SCL_UNDEFAULT_TDF (diagstk_ptr
                                      ,src_tlh_ptr ,src_prm_ptr
                                      ,src_tdf_ptr ,def_tdf_ptr ,out_tdf_ptr);
            if (!(psl_test_status (iret))) goto end;
        }
    }

    else
    {
        src_tlv_ptr = (struct tlv *) (src_prm_ptr + 1);
        /* Get pointer to first Source TLV   */
        out_tlv_ptr = (struct tlv *) (out_prm_ptr + 1);
        /* ...and to first Output TLV        */

        /*
         *    Loop through the TLV's translating each as we go.
         */
        for (isub = 0;
                isub < (*src_tlh_ptr).tlh_testcntl;
                isub++, src_tlv_ptr++ ,out_tlv_ptr++)
        {
            VTEST_UNDEFAULT_TLV (diagstk_ptr ,src_tlv_ptr ,NULL ,out_tlv_ptr);
        }
    }

    /*
     *    Indicate Success.
     */
    iret = PSL_NORMAL;

    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);

    /*
     *  END:
     *
     *  Return with status.
     */
end:
    return iret;
}

/************************************************************************/
/*                                  */
/*    SCL_UNDEFAULT_TDF - Translate default values in a TDF record  */
/*    =================    read in from a file.                     */
/*                                  */
/*    Calling Format:                           */
/*  iret = SCL_UNDEFAULT_TDF (diagstk_ptr,src_tlh_ptr,src_prm_ptr,  */
/*               src_tdf_ptr, def_tdf_ptr, out_tdf_ptr);    */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  src_tlh_ptr = Pointer to the TLH record to use              */
/*  src_prm_ptr = Pointer to the PRM record to use              */
/*  src_tdf_ptr = Pointer to the TDF record to translate        */
/*  def_tdf_ptr = Pointer to defaults TDF record.           */
/*  out_tdf_ptr = Pointer to the translated TDF record to set up.   */
/*            (May be a NULL pointer if no defaults record) */
/*                                  */
/*    Return Status                             */
/*  iret        = PSL Return Status                 */
/*                                  */
/*    This routine is called to copy the source TDF record into the     */
/*    output TDF record, determining which if any fields are set to     */
/*    default and translating any found. If we have a defaults record   */
/*    then this itself provides the default value. Note that it also    */
/*    could have the field set to default in which case we must use */
/*    the internal defaults.                        */
/*                                  */
/*    We use a work record to hold the TDF while we convert it and so   */
/*    we allow the SOURCE and TARGET TDF buffers to be one and the same */
/*                                  */
/*    IMPORTANT NOTE                            */
/*    ==============                            */
/*  This routine does NOT convert either the USERNAME field or the  */
/*  script parameter records. This is because these fields have     */
/*  internal defaults which are defined by the environment.     */
/*                                  */
/*  For example, the username internal default is the current user  */
/*  whereas the TDP record internal defaults are the values held    */
/*  in the script. Calling utilities should check these fields  */
/*  manually for defaulting.                    */
/*                                  */
/************************************************************************/

unsigned long   SCL_UNDEFAULT_TDF   (struct diagstk *diagstk_ptr
                                     ,struct tlh     *src_tlh_ptr
                                     ,struct prm     *src_prm_ptr
                                     ,struct tdf     *src_tdf_ptr
                                     ,struct tdf     *def_tdf_ptr
                                     ,struct tdf     *out_tdf_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   iret;           /* Routine Return Status */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    static   struct tdf wrk_tdf_rec;    /* Work TDF record      */
    static   struct tdf int_tdf_rec;    /* INTERNAL TDF defaults    */
    static  Boolean got_int_tdf = PFALSE;   /* Flag to show setup internal defs */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During VTEST_UNDEFAULT_TDF");

    /*
     *    If we have not as yet set up the Internal TDF defaults then do so now.
     */
    if (!got_int_tdf)
    {
        SETUP_INT_TDF_DEF (diagstk_ptr ,&int_tdf_rec);
        got_int_tdf = PTRUE;
    }

    /*
     *    First, copy the entire TDF record from the source to the WORK record.
     */
    (void) memcpy (&wrk_tdf_rec ,src_tdf_ptr ,sizeof (struct tdf));

    /*
     *    ...then step through each field and if it is set to default then pickup
     *    the value from the supplied default record. If this value is default or
     *    the defaults record was NOT specified then pick up the internal defaults
     *    record.
     */

    /*
     *    The first batch of fields may not have a value specified in the $DEFAULTS
     *    record and so we recover the value directly from the internal defaults if
     *    the field is marked as being defaulted.
     *
     *    SEQUENCE NUMBER field 
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_SEQ_NO) != 0)
    {
        wrk_tdf_rec.tdf_seq_nol = int_tdf_rec.tdf_seq_nol;
        /* ...pick up internal default  */
    }

    /*
     *    AND THEN THE FIELDS WHICH DO HAVE $DEFAULT VALUES
     *    =================================================
     *
     *    TERMINAL TYPE field - Some items are invalid depending on Terminal Type
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_TERMINAL_TYPE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_terminal_typel =
                int_tdf_rec.tdf_terminal_typel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_terminal_typel =
                (*def_tdf_ptr).tdf_terminal_typel;
            /* ...pick up default record value */
        }
    }

    /*
     *    LT Node field.
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_LTNODE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_ltnodec8
                           ,int_tdf_rec.tdf_ltnodec8);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_ltnodec8
                           ,(*def_tdf_ptr).tdf_ltnodec8);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    NUMBER field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_NUMBER) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_numberl = int_tdf_rec.tdf_numberl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_numberl = (*def_tdf_ptr).tdf_numberl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    PHYSICAL DEVICE field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_PHYSICAL_DEVICE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_physical_devicec36
                           ,int_tdf_rec.tdf_physical_devicec36);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_physical_devicec36
                           ,(*def_tdf_ptr).tdf_physical_devicec36);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    START field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_START) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_startl = int_tdf_rec.tdf_startl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_startl = (*def_tdf_ptr).tdf_startl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    CHARACTER SIZE field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_CHARACTER_SIZE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_character_sizel =
                int_tdf_rec.tdf_character_sizel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_character_sizel =
                (*def_tdf_ptr).tdf_character_sizel;
            /* ...pick up internal default  */
        }
    }

    /*
     *    DELAY field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_DELAY) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_delayl = int_tdf_rec.tdf_delayl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_delayl = (*def_tdf_ptr).tdf_delayl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    WIDTH field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_WIDTH) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_widthl = int_tdf_rec.tdf_widthl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_widthl = (*def_tdf_ptr).tdf_widthl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    PAGE DEPTH field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_PAGE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_pagel = int_tdf_rec.tdf_pagel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_pagel = (*def_tdf_ptr).tdf_pagel;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    BAUD RATE field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_BAUD_RATE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_baud_ratel = int_tdf_rec.tdf_baud_ratel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_baud_ratel = (*def_tdf_ptr).tdf_baud_ratel;
            /* ...get default value     */
        }
    }

    /*
     *    TERMINAL ID field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_TERMINAL_ID) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_terminal_idl = int_tdf_rec.tdf_terminal_idl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_terminal_idl = (*def_tdf_ptr).tdf_terminal_idl;
            /* ...pick up default record value */
        }
    }

    /*
     *    TRACE field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_TRACEFLAG) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_traceflagl = int_tdf_rec.tdf_traceflagl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_traceflagl = (*def_tdf_ptr).tdf_traceflagl;
        }
    }

    /*
     *    TRACE FILE NAME field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_TRACEFILE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_tracefilec52
                           ,int_tdf_rec.tdf_tracefilec52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_tracefilec52
                           ,(*def_tdf_ptr).tdf_tracefilec52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    RECORD TERMINAL field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_RECORD) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_recordl = int_tdf_rec.tdf_recordl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_recordl = (*def_tdf_ptr).tdf_recordl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    RECORDING TYPES field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_RECORD_TYPE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_record_typel = int_tdf_rec.tdf_record_typel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_record_typel = (*def_tdf_ptr).tdf_record_typel;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    TYPEAHEAD field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_TYPEAHEAD) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_typeaheadl = int_tdf_rec.tdf_typeaheadl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_typeaheadl = (*def_tdf_ptr).tdf_typeaheadl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    RECORDING FILE NAME field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_RECORDFILE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_recordfilec52
                           ,int_tdf_rec.tdf_recordfilec52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_recordfilec52
                           ,(*def_tdf_ptr).tdf_recordfilec52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    START-UP DELAY field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_STARTDELAY) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_startdelayl = int_tdf_rec.tdf_startdelayl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_startdelayl = (*def_tdf_ptr).tdf_startdelayl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    ANSWERBACK field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_ANSWERBACK) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_answerbackc32
                           ,int_tdf_rec.tdf_answerbackc32);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_answerbackc32
                           ,(*def_tdf_ptr).tdf_answerbackc32);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    DETACH ON EXIT field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_DETACH_EXIT) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_detach_exitl = int_tdf_rec.tdf_detach_exitl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_detach_exitl = (*def_tdf_ptr).tdf_detach_exitl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    HANGUP ON DETACH field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_HANGUP_DETACH) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_hangup_detachl = int_tdf_rec.tdf_hangup_detachl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_hangup_detachl = (*def_tdf_ptr).tdf_hangup_detachl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    STOP RECORD ON DETACH field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_STOPRECDET) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_stoprecdetl = int_tdf_rec.tdf_stoprecdetl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_stoprecdetl = (*def_tdf_ptr).tdf_stoprecdetl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    FKEY CONTROLS field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_FKEY_CONTROL) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_fkey_controll = int_tdf_rec.tdf_fkey_controll;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_fkey_controll = (*def_tdf_ptr).tdf_fkey_controll;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    SERVICE NAME field
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_LTSERVICE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_ltservicec20
                           ,int_tdf_rec.tdf_ltservicec20);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_ltservicec20
                           ,(*def_tdf_ptr).tdf_ltservicec20);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    AUTO-SYNC field
     */
    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_AUTO_SYNC) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_auto_syncl = int_tdf_rec.tdf_auto_syncl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_auto_syncl = (*def_tdf_ptr).tdf_auto_syncl;
        }
    }

    /*
     *    ...and then the platform-specific fields.
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_DEVICE_TYPE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_device_typel = int_tdf_rec.tdf_device_typel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_device_typel = (*def_tdf_ptr).tdf_device_typel;
            /* ...get value from DEFAULTS record */
        }
    }

    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_ENABLE_IOM) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_enable_ioml = int_tdf_rec.tdf_enable_ioml;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_enable_ioml = (*def_tdf_ptr).tdf_enable_ioml;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    TCP/IP ADDRESS field
     */
    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_TCPIPADDR) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_fqtcpip_addressc128
                           ,int_tdf_rec.tdf_fqtcpip_addressc128);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_fqtcpip_addressc128
                           ,(*def_tdf_ptr).tdf_fqtcpip_addressc128);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    TERMINFO NAME field
     */
    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_TRMINFO) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_terminfo_namec36
                           ,int_tdf_rec.tdf_terminfo_namec36);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_terminfo_namec36
                           ,(*def_tdf_ptr).tdf_terminfo_namec36);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    Some of the fields within the terminal definition cannot simply be 
     *    defaulted as their default values may rely on the value of another
     *    field. We do these here to ensure that if any dependant field is also
     *    defaulted that its default value has been set up already.
     *
     *    The Test Unit defaults to Script Name.
     */
    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_TST_UNIT) != 0)
    {
        (void) strcpy (wrk_tdf_rec.tdf_unit_namec20
                       ,wrk_tdf_rec.tdf_scriptc20);
        /* ...default is the script name */
    }

    /*
     *    The Terminal Name defaults to Test Unit Name.
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_TRM_NAME) != 0)
    {
        (void) strcpy (wrk_tdf_rec.tdf_trm_namec20
                       ,wrk_tdf_rec.tdf_unit_namec20);
        /* ...default is the test unit */
    }

    /*
     *    The default for the username is the name of the current user.  
     *    Seeing as how we do not know this we will set it up to be the text <Calc>.
     *
     *    Any utility calling this routine should be aware of this and apply a
     *    suitable default.
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_USERNAME) != 0)
    {                   /* If the USERNAME defaulted    */
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_tdf_rec.tdf_usernamec36
                           ,int_tdf_rec.tdf_usernamec36);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_tdf_rec.tdf_usernamec36
                           ,(*def_tdf_ptr).tdf_usernamec36);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    SOCKET CONNECTION
     */
    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_USE_TCP_SOCKET) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_use_tcp_socketl = int_tdf_rec.tdf_use_tcp_socketl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_use_tcp_socketl =
                (*def_tdf_ptr).tdf_use_tcp_socketl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    LOGIN TYPE field - For this call the SVXLBR default routine
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_LOGIN_TYPE) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_login_typel =
                VTEST_LOGIN_TYPE_DEF (wrk_tdf_rec.tdf_terminal_typel);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_login_typel = (*def_tdf_ptr).tdf_login_typel;
        }
    }

    /*
     *  If we have a LOGIN TYPE that is inconsistent with the TERMINAL TYPE then
     *  adjust the login type to become valid again.
     */
#ifndef WIN32
    if (((wrk_tdf_rec.tdf_terminal_typel == TOF_THRTTP_REMLAT) ||
            (wrk_tdf_rec.tdf_terminal_typel == TOF_THRTTP_REMTCPIP)) &&
            ((wrk_tdf_rec.tdf_login_typel != TOF_THRLOG_TELNET) &&
             (wrk_tdf_rec.tdf_login_typel != TOF_THRLOG_RLOGIN) &&
             (wrk_tdf_rec.tdf_login_typel != TOF_THRLOG_RAW)))
    {
        wrk_tdf_rec.tdf_login_typel = TOF_THRLOG_TELNET;
    }
#endif /* NOT WINDOWS */

    /*
     *    TIMING field - This is dependant on the Terminal type.
     */
    if (((*src_tdf_ptr).tdf_default_flags1l & TDF_THRDF1_INPUT_TIMING) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_input_timingl =
                VTEST_TIMING_DEF (wrk_tdf_rec.tdf_input_timingl);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_input_timingl = (*def_tdf_ptr).tdf_input_timingl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    PORT NUMBER field - This is dependant on the LOGIN TYPE.
     */
    if (((*src_tdf_ptr).tdf_default_flags2l & TDF_THRDF2_TCPIP_PORT) != 0)
    {
        if (def_tdf_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_tdf_rec.tdf_tcpip_portl =
                VTEST_PORT_DEF (wrk_tdf_rec.tdf_login_typel);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_tdf_rec.tdf_tcpip_portl = (*def_tdf_ptr).tdf_tcpip_portl;
            /* ...get value from DEFAULTS record */
        }
    }



    /*
     *    Copy the translated TDF record out of the work buffer and into the buffer.
     */
    (void) memcpy (out_tdf_ptr ,&wrk_tdf_rec ,sizeof (struct tdf));

    /*
     *  Expand the RECORDING FILE name.
     */
    iret = VTEST_UNDEF_RECFIL (diagstk_ptr, src_tlh_ptr,
                               src_prm_ptr, out_tdf_ptr,
                               &(*out_tdf_ptr).tdf_recfil_ptrpc);
    if (!(psl_test_status (iret))) goto end;

    /*
     *  Expand the TRACE FILE name.
     */
    iret = VTEST_UNDEF_TRCFIL (diagstk_ptr, src_tlh_ptr,
                               src_prm_ptr, out_tdf_ptr,
                               &(*out_tdf_ptr).tdf_trcfil_ptrpc);
    if (!(psl_test_status (iret))) goto end;

    /*
     *    Indicate Success.
     */
    iret = PSL_NORMAL;

    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);

    /*
     *  END:
     *
     *  Return with status.
     */
end:
    return iret;
}



/************************************************************************/
/*                                  */
/*    VTEST_UNDEF_TRCFIL  - Build TRACE FILE from defaulted values      */
/*    ==================                                            */
/*                                  */
/*    Calling Format:                           */
/*  iret = VTEST_UNDEF_TRCFIL (diagstk_ptr, src_tlh_ptr,        */
/*                 src_prm_ptr, src_tdf_ptr,        */
/*                 trcfil_pptr)             */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  src_tlh_ptr = Pointer to the TLH record to use              */
/*  src_prm_ptr = Pointer to the PRM record to use              */
/*  src_tdf_ptr = Pointer to the TDF record (already translated)    */
/*  trcfil_pptr = Pointer for returned expanded name        */
/*                                  */
/*    Return Status                             */
/*  iret        = PSL Return Status                 */
/*                                  */
/*    This routine is called to build up a TRACE FILE name within the   */
/*    test definition.  It relies on the fact that all existing TDF     */
/*    fields have their real values.                                    */
/*                                  */
/************************************************************************/

static unsigned long    VTEST_UNDEF_TRCFIL  (struct diagstk *diagstk_ptr,
        struct tlh *src_tlh_ptr,
        struct prm *src_prm_ptr,
        struct tdf *src_tdf_ptr,
        char **trcfil_pptr)
{
    /*
     *    Local Data
     */
    unsigned long   iret;           /* Routine Return Status */
    unsigned long   saved_diagctx;      /* Saved diagnostics context */

    static   char   wrkstr[512];        /* Work Buffer */
    char   *wrk_ptr;       /* Work Pointer */
    char   tstnam[256];        /* Test Name */
#if defined(UNIX) || defined(WIN32)
    char   tstnam_lc[256];     /* Test Name (lowercase) */
    char   envnam_lc[256];     /* Env. Name (lowercase) */
    char   scpnam_lc[256];     /* Script Name (lowercase) */
    char   trmnam_lc[256];     /* Terminal Name (lowercase) */
    unsigned long   loop;           /* Loop Control */
#endif

    /*
     *  First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr, "During VTEST_UNDEF_TRCFIL");

    /*
     *  Get the Test Name (null-terminated) - both upper and lowercase.
     */
    (void) memset (tstnam, '\0', sizeof (tstnam));

    /*
     *  Copy the test name if we have a TLH Pointer.
     */
    if (src_tlh_ptr != NULL)
    {
        (void) memcpy (tstnam, (*src_tlh_ptr).tlh_testnamec16,
                       sizeof ((*src_tlh_ptr).tlh_testnamec16));
        wrk_ptr = strchr (tstnam, ' ');
        if (wrk_ptr != NULL) *wrk_ptr = '\0';

        /*
         *  If <LOCAL> test use "LGN".
         */
        if (memcmp (tstnam, "<LOCAL>", 7) == 0)
        {
            (void) strcpy (tstnam, "LGN");
        }

        /*
         *      Convert the Test Name to lowercase for UNIX and WIN32.
         */
#if defined(UNIX) || defined(WIN32)
        (void) strcpy (tstnam_lc, tstnam);

        for (loop = 0; loop < strlen (tstnam_lc); loop++)
        {
            tstnam_lc[loop] = tolower (tstnam_lc[loop]);
        }
#endif
    }

    /*
     *  Get a lowercase version of the environment name.
     */
#if defined(UNIX) || defined(WIN32)
    if (src_prm_ptr != NULL)
    {
        (void) strcpy (envnam_lc, (*src_prm_ptr).prm_environmentc12);

        for (loop = 0; loop < strlen (envnam_lc); loop++)
        {
            envnam_lc[loop] = tolower (envnam_lc[loop]);
        }
    }
#endif  /* UNIX & WIN32 */

    /*
     *  Convert the Script Name to lowercase.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy (scpnam_lc, (*src_tdf_ptr).tdf_scriptc20);

    for (loop = 0; loop < strlen (scpnam_lc); loop++)
    {
        scpnam_lc[loop] = tolower (scpnam_lc[loop]);
    }
#endif /* UNIX & WIN32 */

    /*
     *  Convert the Logical Terminal Name to lowercase.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy (trmnam_lc, (*src_tdf_ptr).tdf_trm_namec20);

    for (loop = 0; loop < strlen (trmnam_lc); loop++)
    {
        trmnam_lc[loop] = tolower (trmnam_lc[loop]);
    }
#endif  /* UNIX & WIN32 */

    /*
     *  Handle the substitution parameters for the TRACE FILE
     */
    if ((memcmp ((*src_tdf_ptr).tdf_tracefilec52, "<terminal>", 10) == 0) ||
            (memcmp ((*src_tdf_ptr).tdf_tracefilec52, "<TERMINAL>", 10) == 0))
    {

        (void) strcpy (wrkstr, "^l.trc");
    }
    else
    {
        (void) strcpy (wrkstr, (*src_tdf_ptr).tdf_tracefilec52);
    }

    /*
     *  Replace "^E" - Environment Name (in uppercase & lowercase)
     */
    if (src_prm_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^E",
                           (*src_prm_ptr).prm_environmentc12);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^e", envnam_lc);
#endif
    }

    /*
     *  Replace "^T" - Test Name (in uppercase & lowercase)
     */
    if (src_tlh_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^T", tstnam);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^t", tstnam_lc);
#endif
    }

    /*
     *  Replace "^S" - Script Name (in uppercase & lowercase)
     */
    if ((*src_tdf_ptr).tdf_scriptc20[0] != '\0')
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^S",
                           (*src_tdf_ptr).tdf_scriptc20);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^s", scpnam_lc);
#endif

        /*
         *  Replace "^L" - Logical Terminal Name (in uppercase & lowercase)
         */
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^L",
                           (*src_tdf_ptr).tdf_trm_namec20);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^l", trmnam_lc);
#endif
    }

    /*
     *  Hande the 'platform' type and the nodename.
     */
    SUBSTR_COMMON (wrkstr, sizeof (wrkstr));

    /*
     *  Allocate enough space for this resultant string.
     */
    *trcfil_pptr = malloc (strlen (wrkstr) + 1);
    if (*trcfil_pptr == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_PUTDIAG (diagstk_ptr, 2, PSL_SYSERVERR,
                     "malloc for TRCFIL Buffer", iret);
        goto end;
    }
    else
    {
        (void) strcpy (*trcfil_pptr, wrkstr);
    }

    /*
     *  Indicate Success.
     */
    iret = PSL_NORMAL;

    /*
     *  Remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);

    /*
     *  END:
     *
     *  Return with status.
     */
end:
    return iret;
}




/************************************************************************/
/*                                  */
/*    VTEST_UNDEF_RECFIL  - Build RECORD FILE from defaulted values     */
/*    ==================                                            */
/*                                  */
/*    Calling Format:                           */
/*  iret = VTEST_UNDEF_RECFIL (diagstk_ptr, src_tlh_ptr,        */
/*                 src_prm_ptr, src_tdf_ptr,        */
/*                 recfil_pptr)             */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  src_tlh_ptr = Pointer to the TLH record to use              */
/*  src_prm_ptr = Pointer to the PRM record to use              */
/*  src_tdf_ptr = Pointer to the TDF record (already translated)    */
/*  recfil_pptr = Pointer for returned expanded name        */
/*                                  */
/*    Return Status                             */
/*  iret        = PSL Return Status                 */
/*                                  */
/*    This routine is called to build up a RECORD FILE name within the  */
/*    test definition.  It relies on the fact that all existing TDF     */
/*    fields have their real values.                                    */
/*                                  */
/************************************************************************/

static unsigned long    VTEST_UNDEF_RECFIL  (struct diagstk *diagstk_ptr,
        struct tlh *src_tlh_ptr,
        struct prm *src_prm_ptr,
        struct tdf *src_tdf_ptr,
        char **recfil_pptr)
{
    /*
     *    Local Data
     */
    unsigned long   iret;           /* Routine Return Status */
    unsigned long   saved_diagctx;      /* Saved diagnostics context */

    static   char   wrkstr[512];        /* Work Buffer */
    char   *wrk_ptr;       /* Work Pointer */
    char   tstnam[256];        /* Test Name */
#if defined(UNIX) || defined(WIN32)
    char   tstnam_lc[256];     /* Test Name (lowercase) */
    char   envnam_lc[256];     /* Env. Name (lowercase) */
    char   scpnam_lc[256];     /* Script Name (lowercase) */
    char   trmnam_lc[256];     /* Terminal Name (lowercase) */
    unsigned long   loop;           /* Loop Control */
#endif

    /*
     *  First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr, "During VTEST_UNDEF_RECFIL");

    /*
     *  Get the Test Name (null-terminated) - both upper and lowercase.
     */
    (void) memset (tstnam, '\0', sizeof (tstnam));

    /*
     *  Copy the test name if we have a TLH Pointer.
     */
    if (src_tlh_ptr != NULL)
    {
        (void) memcpy (tstnam, (*src_tlh_ptr).tlh_testnamec16,
                       sizeof ((*src_tlh_ptr).tlh_testnamec16));
        wrk_ptr = strchr (tstnam, ' ');
        if (wrk_ptr != NULL) *wrk_ptr = '\0';

        /*
         *  If <LOCAL> test use "LGN".
         */
        if (memcmp (tstnam, "<LOCAL>", 7) == 0)
        {
            (void) strcpy (tstnam, "LGN");
        }

        /*
         *      Convert the Test Name to lowercase.
         */
#if defined(UNIX) || defined(WIN32)
        (void) strcpy (tstnam_lc, tstnam);

        for (loop = 0; loop < strlen (tstnam_lc); loop++)
        {
            tstnam_lc[loop] = tolower (tstnam_lc[loop]);
        }
#endif
    }

    /*
     *  Get a lowercase version of the environment name.
     */
#if defined(UNIX) || defined(WIN32)
    if (src_prm_ptr != NULL)
    {
        (void) strcpy (envnam_lc, (*src_prm_ptr).prm_environmentc12);

        for (loop = 0; loop < strlen (envnam_lc); loop++)
        {
            envnam_lc[loop] = tolower (envnam_lc[loop]);
        }
    }
#endif

    /*
     *  Convert the Script Name to lowercase.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy (scpnam_lc, (*src_tdf_ptr).tdf_scriptc20);

    for (loop = 0; loop < strlen (scpnam_lc); loop++)
    {
        scpnam_lc[loop] = tolower (scpnam_lc[loop]);
    }
#endif

    /*
     *  Convert the Logical Terminal Name to lowercase.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy (trmnam_lc, (*src_tdf_ptr).tdf_trm_namec20);

    for (loop = 0; loop < strlen (trmnam_lc); loop++)
    {
        trmnam_lc[loop] = tolower (trmnam_lc[loop]);
    }
#endif

    /*
     *  Handle the substitution parameters for the RECORDING FILE
     */
    if ((memcmp ((*src_tdf_ptr).tdf_recordfilec52, "<terminal>", 10) == 0) ||
            (memcmp ((*src_tdf_ptr).tdf_recordfilec52, "<TERMINAL>", 10) == 0))
    {
        (void) strcpy (wrkstr, "^l.vrf");
    }
    else
    {
        (void) strcpy (wrkstr, (*src_tdf_ptr).tdf_recordfilec52);
    }

    /*
     *  Replace "^E" - Environment Name (in uppercase & lowercase)
     */
    if (src_prm_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^E",
                           (*src_prm_ptr).prm_environmentc12);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^e", envnam_lc);
#endif
    }

    /*
     *  Replace "^T" - Test Name (in uppercase & lowercase)
     */
    if (src_tlh_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^T", tstnam);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^t", tstnam_lc);
#endif
    }

    /*
     *  Replace "^S" - Script Name (in uppercase & lowercase)
     */
    if ((*src_tdf_ptr).tdf_scriptc20[0] != '\0')
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^S",
                           (*src_tdf_ptr).tdf_scriptc20);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^s", scpnam_lc);
#endif

        /*
         *  Replace "^L" - Logical Terminal Name (in uppercase & lowercase)
         */
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^L",
                           (*src_tdf_ptr).tdf_trm_namec20);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^l", trmnam_lc);
#endif
    }

    /*
     *  Hande the 'platform' type and the nodename.
     */
    SUBSTR_COMMON (wrkstr, sizeof (wrkstr));

    /*
     *  Allocate enough space for this resultant string.
     */
    *recfil_pptr = malloc (strlen (wrkstr) + 1);
    if (*recfil_pptr == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_PUTDIAG (diagstk_ptr, 2, PSL_SYSERVERR,
                     "malloc for RECFIL Buffer", iret);
        goto end;
    }
    else
    {
        (void) strcpy (*recfil_pptr, wrkstr);
    }

    /*
     *  Indicate Success.
     */
    iret = PSL_NORMAL;

    /*
     *  Remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);

    /*
     *  END:
     *
     *  Return with status.
     */
end:
    return iret;
}




/************************************************************************/
/*                                  */
/*    SUBSTR_COMMON - Substitute 'common' parameters            */
/*    =============                                             */
/*                                  */
/*    Calling Format:                           */
/*  SUBSTR_COMMON (wrkstr_ptr, wrkstr_size)             */
/*                                  */
/*    Where:                                */
/*  wrkstr_ptr   = Pointer to the work string                       */
/*  wrkstr_size  = Size of the work string                      */
/*                                  */
/*    This routine is called to replace the ^^, ^O & ^M substitution    */
/*    parameters within the string.                                     */
/*                                  */
/************************************************************************/

static  void    SUBSTR_COMMON (char *wrkstr_ptr,
                               unsigned long wrkstr_size)
{
    /*
     *    Local Data
     */
    char    nodename[32];       /* Nodename */
#if defined(UNIX) || defined(WIN32)
    char   nodename_lc[32];    /* Machine Name (lowercase) */
    unsigned long   loop;           /* Loop Control */
#endif

    /*
     *  "^O" - Operating System Name (In uppercase & lowercase)
     */
#ifdef VAX
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "VAX");
#elif defined(ALPHA)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "ALPHA");
#elif defined(HP)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "HP");
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^o", "hp");
#elif defined(IBM)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "IBM");
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^o", "ibm");
#elif defined(NCR)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "NCR");
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^o", "ncr");
#elif defined(OSF)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "OSF");
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^o", "osf");
#elif defined(SUN)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "SUN");
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^o", "sun");
#elif defined(WIN32)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^O", "WIN32");
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^o", "win32");
#else
#error Unsupported platform at line __LINE__
#endif

    /*
     *  Get the current nodename.
     */
    (void) PSL_GET_NODENAME (NULL, nodename, sizeof (nodename));

    /*
     *  Handle "^M" & "^m".
     */
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^M", nodename);

    /*
     *  Get a lowercase version.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy (nodename_lc, nodename);

    for (loop = 0; loop < strlen (nodename_lc); loop++)
    {
        nodename_lc[loop] = tolower (nodename_lc[loop]);
    }
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^m", nodename_lc);
#endif

    /*
     *  Replace "^^" - Single "^"
     */
#if defined(UNIX) || defined(WIN32)
    (void) PSL_SUBSTR (wrkstr_ptr, wrkstr_size, "^^", "^");
#endif

    return;
}



/************************************************************************/
/*                                  */
/*    SCL_FREE_TDF - Free up allocated data within TDF record       */
/*    ============                                              */
/*                                  */
/*    Calling Format:                           */
/*  SCL_FREE_TDF (tdf_ptr)                      */
/*                                  */
/*    Where:                                */
/*  tdf_ptr = Pointer to the TDF record containing pointers     */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to free up any allocated space pointed     */
/*    to by the fields in the TDF structure.                            */
/*                                  */
/*    These will have been allocated via the SCL_UNDEFAULT_TDF rtn. */
/*                                  */
/************************************************************************/

void    SCL_FREE_TDF    (struct tdf *tdf_ptr)
{
    /*
     *  Deallocate the allocated data (if necessary).
     */
    if ((*tdf_ptr).tdf_recfil_ptrpc != NULL)
    {
        free ((*tdf_ptr).tdf_recfil_ptrpc);
        (*tdf_ptr).tdf_recfil_ptrpc = NULL;
    }

    if ((*tdf_ptr).tdf_trcfil_ptrpc != NULL)
    {
        free ((*tdf_ptr).tdf_trcfil_ptrpc);
        (*tdf_ptr).tdf_trcfil_ptrpc = NULL;
    }

    /*
     *  ...and return.
     */
    return;
}


/************************************************************************/
/*                                  */
/*    VTEST_UNDEFAULT_TLV - Translate default values in a TLV record    */
/*    ===================    read in from a file.                   */
/*                                  */
/*    Calling Format:                           */
/*  VTEST_UNDEFAULT_TLV (diagstk_ptr, src_tlv_ptr ,def_tlv_ptr  */
/*              ,out_tlv_ptr);              */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  src_tlv_ptr = Pointer to the TLV record to translate        */
/*  def_tlv_ptr = Pointer to defaults TLV record.           */
/*  out_tlv_ptr = Pointer to the translated TLV record to set up.   */
/*            (May be a NULL pointer if no defaults record) */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to copy the source TLV record into the     */
/*    output TLV record, determining which if any fields are set to     */
/*    default and translating any found. If we have a defaults record   */
/*    then this itself provides the default value. Note that it also    */
/*    could have the field set to default in which case we must use */
/*    the internal defaults.                        */
/*                                  */
/************************************************************************/

static  void    VTEST_UNDEFAULT_TLV (struct diagstk *diagstk_ptr
                                     ,struct tlv     *src_tlv_ptr
                                     ,struct tlv     *def_tlv_ptr
                                     ,struct tlv     *out_tlv_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    static   struct tlv int_tlv_rec;    /* Pointer to INTERNAL TLV defaults */
    static  Boolean got_int_tlv = PFALSE;   /* Flag to show setup internal defs */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During VTEST_UNDEFAULT_TLV");

    /*
     *    If we have not as yet set up the Internal TLV defaults then do so now.
     */
    if (!got_int_tlv)
    {
        SETUP_INT_TLV_DEF (diagstk_ptr ,&int_tlv_rec);
        got_int_tlv = PTRUE;
    }

    /*
     *    First of all copy the entire TLV record from the source to the new record.
     */
    (void) memcpy (out_tlv_ptr ,src_tlv_ptr ,sizeof (struct tlv));

    /*
     *    LOGICAL NODE NAME field
     */
    if (((*src_tlv_ptr).tlv_default_flagsl & TLVD_LOGICAL_NODE) != 0)
    {
        if (def_tlv_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy ((*out_tlv_ptr).tlv_logical_nodec20
                           ,int_tlv_rec.tlv_logical_nodec20);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy ((*out_tlv_ptr).tlv_logical_nodec20
                           ,(*def_tlv_ptr).tlv_logical_nodec20);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}


/************************************************************************/
/*                                  */
/*    SCL_UNDEFAULT_PRM - Translate default values in a PRM record  */
/*    =================    read in from a file.                     */
/*                                  */
/*    Calling Format:                           */
/*  iret = SCL_UNDEFAULT_PRM (diagstk_ptr ,src_tlh_ptr      */
/*              ,src_prm_ptr ,def_prm_ptr ,out_prm_ptr);    */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  src_tlh_ptr = Pointer to the TLH record to use              */
/*  src_prm_ptr = Pointer to the PRM record to translate        */
/*  def_prm_ptr = Pointer to defaults PRM record.           */
/*            (May be a NULL pointer if no defaults record) */
/*  out_prm_ptr = Pointer to the translated PRM record to set up.   */
/*                                  */
/*    Return Status                             */
/*  iret        = PSL Return Status                 */
/*                                  */
/*    This routine is called to copy the source PRM record into the     */
/*    output PRM record, determining which if any fields are set to     */
/*    default and translating any found. If we have a defaults record   */
/*    then this itself provides the default value. Note that it also    */
/*    could have the field set to default in which case we must use */
/*    the internal defaults.                        */
/*                                  */
/************************************************************************/

unsigned long   SCL_UNDEFAULT_PRM   (struct diagstk *diagstk_ptr
                                     ,struct tlh     *src_tlh_ptr
                                     ,struct prm     *src_prm_ptr
                                     ,struct prm     *def_prm_ptr
                                     ,struct prm     *out_prm_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   iret;           /* Routine Return Status */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    static   struct prm wrk_prm_rec;    /* Work PRM record      */
    static   struct prm int_prm_rec;    /* Pointer to INTERNAL PRM defaults */
    static  Boolean got_int_prm = PFALSE;   /* Flag to show setup internal defs */

    static   char   wrkstr[512];        /* Work Buffer */
    char   *wrk_ptr;       /* Work Pointer */
    char   tstnam[256];        /* Test Name */
#if defined(UNIX) || defined(WIN32)
    char   tstnam_lc[256];     /* Test Name (lowercase) */
    char   envnam_lc[256];     /* Env. Name (lowercase) */
    unsigned long   loop;           /* Loop Control */
#endif

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During SCL_UNDEFAULT_PRM");

    /*
     *    If we have not as yet set up the Internal TDF defaults then do so now.
     */
    if (!got_int_prm)
    {
        SETUP_INT_PRM_DEF (diagstk_ptr ,&int_prm_rec);
        got_int_prm = PTRUE;
    }

    /*
     *    First, copy the entire PRM record from the source to the WORK record.
     */
    (void) memcpy (&wrk_prm_rec ,src_prm_ptr ,sizeof (struct prm));


    /*
     *    Copy the translated PRM record out of the work buffer and into the target 
     *    buffer. 
     */
    (void) memcpy (out_prm_ptr ,&wrk_prm_rec ,sizeof (struct prm));

    /*
     *    MESSAGE LOG field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_MESSAGE_LOG) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_message_logc52
                           ,int_prm_rec.prm_message_logc52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_message_logc52
                           ,(*def_prm_ptr).prm_message_logc52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    MAXIMUM TERMINALS field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_MAX_THREADS) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_max_threadsl = int_prm_rec.prm_max_threadsl;
        }
        else
        {               /* If no DEFAULT or also default */
            wrk_prm_rec.prm_max_threadsl = (*def_prm_ptr).prm_max_threadsl;
        }
    }

    /*
     *    STATISTICS FILE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_STATISTICS_FILE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_statistics_filec52
                           ,int_prm_rec.prm_statistics_filec52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_statistics_filec52
                           ,(*def_prm_ptr).prm_statistics_filec52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    INPUT RATE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_INPUT_RATE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_input_ratec8
                           ,int_prm_rec.prm_input_ratec8);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_input_ratec8
                           ,(*def_prm_ptr).prm_input_ratec8);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    WORKING SET QUOTA field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_WSQUOTA) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_wsquotal = int_prm_rec.prm_wsquotal;
        }
        else
        {               /* If no DEFAULT or also default */
            wrk_prm_rec.prm_wsquotal = (*def_prm_ptr).prm_wsquotal;
        }
    }

    /*
     *    ENVIRONMENT field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_ENVIRONMENT) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_environmentc12
                           ,int_prm_rec.prm_environmentc12);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_environmentc12
                           ,(*def_prm_ptr).prm_environmentc12);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    SCOPE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_SCOPE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_scopel = int_prm_rec.prm_scopel;
        }
        else
        {               /* If no DEFAULT or also default */
            wrk_prm_rec.prm_scopel = (*def_prm_ptr).prm_scopel;
        }
    }

    /*
     *    HARDCOPY LOG field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_HARDCOPY_LOG) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_hardcopy_logc52
                           ,int_prm_rec.prm_hardcopy_logc52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_hardcopy_logc52
                           ,(*def_prm_ptr).prm_hardcopy_logc52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    PRIORITY field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_PRIORITY) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_priorityl = int_prm_rec.prm_priorityl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_priorityl = (*def_prm_ptr).prm_priorityl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    HISTORY FILE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_HISTORY_FILE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_history_filec52
                           ,int_prm_rec.prm_history_filec52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_history_filec52
                           ,(*def_prm_ptr).prm_history_filec52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    REPORT FILE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_REPORT_FILE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_report_filec52
                           ,int_prm_rec.prm_report_filec52);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_report_filec52
                           ,(*def_prm_ptr).prm_report_filec52);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    MESSAGE LOG TYPE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_LOG_TYPE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_log_typel = int_prm_rec.prm_log_typel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_log_typel = (*def_prm_ptr).prm_log_typel;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    DUMP ON EXCEPTION field
     */
    if (((*src_prm_ptr).prm_default_flagsl |= PRM_TSTDF_DUMP_STYLE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_dump_stylel = int_prm_rec.prm_dump_stylel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_dump_stylel = (*def_prm_ptr).prm_dump_stylel;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    SHARED field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_SHARED) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_sharedl = int_prm_rec.prm_sharedl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_sharedl = (*def_prm_ptr).prm_sharedl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    AUTOCLOSE field
     */

    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_AUTOCLOSE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_autoclosel = int_prm_rec.prm_autoclosel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_autoclosel = (*def_prm_ptr).prm_autoclosel;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    ABORT ON FAILURE
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_ABORTFAIL) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_abortfaill = int_prm_rec.prm_abortfaill;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_abortfaill = (*def_prm_ptr).prm_abortfaill;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    MAXIMUM CONCURRENT field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_MAX_CONCURRENT) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_max_concurrentl = int_prm_rec.prm_max_concurrentl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_max_concurrentl = (*def_prm_ptr).prm_max_concurrentl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    IGNORE INFORMATIONAL field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_IGNRINF) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_ignrinfl = int_prm_rec.prm_ignrinfl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_ignrinfl = (*def_prm_ptr).prm_ignrinfl;
            /* ...get value from DEFAULTS record */
        }
    }
    /*
     *    ENABLE SERVER IPC TRACING field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_TRACE_MASK) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_trace_maskl = int_prm_rec.prm_trace_maskl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_trace_maskl = (*def_prm_ptr).prm_trace_maskl;
            /* ...get value from DEFAULTS record */
        }
    }


    /*
     *    FAIL TEST-CASE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_FAILTC) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_failtcc84
                           ,int_prm_rec.prm_failtcc84);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_failtcc84
                           ,(*def_prm_ptr).prm_failtcc84);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    PASS TEST-CASE field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_PASSTC) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_passtcc84
                           ,int_prm_rec.prm_passtcc84);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_passtcc84
                           ,(*def_prm_ptr).prm_passtcc84);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    COMPLETE TEST field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_COMPTEST) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_comptestc84
                           ,int_prm_rec.prm_comptestc84);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_comptestc84
                           ,(*def_prm_ptr).prm_comptestc84);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    ABORT TEST field
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_ABRTTEST) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_abrttestc84
                           ,int_prm_rec.prm_abrttestc84);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_abrttestc84
                           ,(*def_prm_ptr).prm_abrttestc84);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    SCRIPT PROTECTION mask.
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_SCP_PROT) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_scp_protl = int_prm_rec.prm_scp_protl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_scp_protl = (*def_prm_ptr).prm_scp_protl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    TEST PROTECTION mask.
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_TST_PROT) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_tst_protl = int_prm_rec.prm_tst_protl;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_tst_protl = (*def_prm_ptr).prm_tst_protl;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    PROCESS TYPE mask.
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_PRC_TYPE) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            wrk_prm_rec.prm_process_typel = int_prm_rec.prm_process_typel;
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            wrk_prm_rec.prm_process_typel = (*def_prm_ptr).prm_process_typel;
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    HISTORY TEMPLATE default message format
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_HISTTEM) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_histtemc84
                           ,int_prm_rec.prm_histtemc84);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_histtemc84
                           ,(*def_prm_ptr).prm_histtemc84);
            /* ...get value from DEFAULTS record */
        }
    }
    /*
     *    REPORT TEMPLATE default message format
     */
    if (((*src_prm_ptr).prm_default_flagsl & PRM_TSTDF_REPTEM) != 0)
    {
        if (def_prm_ptr == NULL)
        {               /* If no DEFAULT        */
            (void) strcpy (wrk_prm_rec.prm_reptemc84
                           ,int_prm_rec.prm_reptemc84);
            /* ...pick up internal default  */
        }
        else                /* ...otherwise...      */
        {
            (void) strcpy (wrk_prm_rec.prm_reptemc84
                           ,(*def_prm_ptr).prm_reptemc84);
            /* ...get value from DEFAULTS record */
        }
    }

    /*
     *    Copy the translated PRM record out of the work buffer and into the buffer.
     */
    (void) memcpy (out_prm_ptr ,&wrk_prm_rec ,sizeof (struct prm));

    /*
     *    Get the Test Name (null-terminated) - both upper and lowercase.
     */
    (void) memset (tstnam, '\0', sizeof (tstnam));

    /*
     *    Copy the test name if we have a TLH Pointer.
     */
    if (src_tlh_ptr != NULL)
    {
        (void) memcpy (tstnam,
                       (*src_tlh_ptr).tlh_testnamec16,
                       sizeof ((*src_tlh_ptr).tlh_testnamec16));
        wrk_ptr = strchr (tstnam, ' ');
        if (wrk_ptr != NULL) *wrk_ptr = '\0';

        /*
         *  If <LOCAL> test use "LGN".
         */
        if (memcmp (tstnam, "<LOCAL>", 7) == 0)
        {
            (void) strcpy (tstnam, "LGN");
        }

        /*
         *    Convert the Test Name to lowercase.
         */
#if defined(UNIX) || defined(WIN32)
        (void) strcpy (tstnam_lc, tstnam);

        for (loop = 0; loop < strlen (tstnam_lc); loop++)
        {
            tstnam_lc[loop] = tolower (tstnam_lc[loop]);
        }
#endif
    }

    /*
     *    Get a lowercase version of the environment name.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy (envnam_lc, (*out_prm_ptr).prm_environmentc12);

    for (loop = 0; loop < strlen (envnam_lc); loop++)
    {
        envnam_lc[loop] = tolower (envnam_lc[loop]);
    }
#endif

    /*
     *    Handle the substitution parameters for the MESSAGE LOG.
     */
    (void) strcpy (wrkstr, (*out_prm_ptr).prm_message_logc52);

    /*
     *    Replace "^E" - Environment Name (in uppercase & lowercase)
     */
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^E",
                       (*out_prm_ptr).prm_environmentc12);
#if defined(UNIX) || defined(WIN32)
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^e", envnam_lc);
#endif

    /*
     *    Replace "^T" - Test Name (in uppercase & lowercase)
     */
    if (src_tlh_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^T", tstnam);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^t", tstnam_lc);
#endif
    }

    /*
     *  Hande the 'platform' type and the nodename.
     */
    SUBSTR_COMMON (wrkstr, sizeof (wrkstr));

    /*
     *    Allocate enough space for this resultant string.
     */
    (*out_prm_ptr).prm_msglog_ptrpc = malloc (strlen (wrkstr) + 1);
    if ((*out_prm_ptr).prm_msglog_ptrpc == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_PUTDIAG (diagstk_ptr, 2, PSL_SYSERVERR,
                     "malloc for MSGLOG Buffer", iret);
        goto end;
    }
    else
    {
        (void) strcpy ((*out_prm_ptr).prm_msglog_ptrpc, wrkstr);
    }

    /*
     *    Now handle the substitution parameters for the STATISTICS FILE.
     */
    (void) strcpy (wrkstr, (*out_prm_ptr).prm_statistics_filec52);

    /*
     *    Replace "^E" - Environment Name (in uppercase & lowercase)
     */
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^E",
                       (*out_prm_ptr).prm_environmentc12);
#if defined(UNIX) || defined(WIN32)
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^e", envnam_lc);
#endif

    /*
     *    Replace "^T" - Test Name (in uppercase & lowercase)
     */
    if (src_tlh_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^T", tstnam);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^t", tstnam_lc);
#endif
    }

    /*
     *  Hande the 'platform' type and the nodename.
     */
    SUBSTR_COMMON (wrkstr, sizeof (wrkstr));

    /*
     *    Allocate enough space for this resultant string.
     */
    (*out_prm_ptr).prm_stafil_ptrpc = malloc (strlen (wrkstr) + 1);
    if ((*out_prm_ptr).prm_stafil_ptrpc == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_PUTDIAG (diagstk_ptr, 2, PSL_SYSERVERR,
                     "malloc for STAFIL Buffer", iret);
        goto end;
    }
    else
    {
        (void) strcpy ((*out_prm_ptr).prm_stafil_ptrpc, wrkstr);
    }

    /*
     *    Now handle the substitution parameters for the HISTORY FILE.
     */
    (void) strcpy (wrkstr, (*out_prm_ptr).prm_history_filec52);

    /*
     *    Replace "^E" - Environment Name (in uppercase & lowercase)
     */
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^E",
                       (*out_prm_ptr).prm_environmentc12);
#if defined(UNIX) || defined(WIN32)
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^e", envnam_lc);
#endif

    /*
     *    Replace "^T" - Test Name (in uppercase & lowercase)
     */
    if (src_tlh_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^T", tstnam);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^t", tstnam_lc);
#endif
    }

    /*
     *  Hande the 'platform' type and the nodename.
     */
    SUBSTR_COMMON (wrkstr, sizeof (wrkstr));

    /*
     *    Allocate enough space for this resultant string.
     */
    (*out_prm_ptr).prm_hisfil_ptrpc = malloc (strlen (wrkstr) + 1);
    if ((*out_prm_ptr).prm_hisfil_ptrpc == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_PUTDIAG (diagstk_ptr, 2, PSL_SYSERVERR,
                     "malloc for HISFIL Buffer", iret);
        goto end;
    }
    else
    {
        (void) strcpy ((*out_prm_ptr).prm_hisfil_ptrpc, wrkstr);
    }

    /*
     *    Now handle the substitution parameters for the REPORT FILE.
     */
    (void) strcpy (wrkstr, (*out_prm_ptr).prm_report_filec52);

    /*
     *    Replace "^E" - Environment Name (in uppercase & lowercase)
     */
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^E",
                       (*out_prm_ptr).prm_environmentc12);
#if defined(UNIX) || defined(WIN32)
    (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^e", envnam_lc);
#endif

    /*
     *    Replace "^T" - Test Name (in uppercase & lowercase)
     */
    if (src_tlh_ptr != NULL)
    {
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^T", tstnam);
#if defined(UNIX) || defined(WIN32)
        (void) PSL_SUBSTR (wrkstr, sizeof (wrkstr), "^t", tstnam_lc);
#endif
    }

    /*
     *  Hande the 'platform' type and the nodename.
     */
    SUBSTR_COMMON (wrkstr, sizeof (wrkstr));

    /*
     *    Allocate enough space for this resultant string.
     */
    (*out_prm_ptr).prm_repfil_ptrpc = malloc (strlen (wrkstr) + 1);
    if ((*out_prm_ptr).prm_repfil_ptrpc == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_PUTDIAG (diagstk_ptr, 2, PSL_SYSERVERR,
                     "malloc for REPFIL Buffer", iret);
        goto end;
    }
    else
    {
        (void) strcpy ((*out_prm_ptr).prm_repfil_ptrpc, wrkstr);
    }

    /*
     *    Indicate Success.
     */
    iret = PSL_NORMAL;

    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);

    /*
     *  END:
     *
     *  Return with status.
     */
end:
    return iret;
}


/************************************************************************/
/*                                  */
/*    SCL_FREE_PRM - Free up allocated data within PRM record       */
/*    ============                                              */
/*                                  */
/*    Calling Format:                           */
/*  SCL_FREE_PRM (prm_ptr)                      */
/*                                  */
/*    Where:                                */
/*  prm_ptr = Pointer to the PRM record containing pointers     */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to free up any allocated space pointed     */
/*    to by the fields in the PRM structure.                            */
/*                                  */
/*    These will have been allocated via the SCL_UNDEFAULT_PRM rtn.     */
/*                                  */
/************************************************************************/

void    SCL_FREE_PRM    (struct prm *prm_ptr)
{
    /*
     *  Deallocate the allocated data (if necessary).
     */
    if ((*prm_ptr).prm_msglog_ptrpc != NULL)
    {
        free ((*prm_ptr).prm_msglog_ptrpc);
        (*prm_ptr).prm_msglog_ptrpc = NULL;
    }

    if ((*prm_ptr).prm_stafil_ptrpc != NULL)
    {
        free ((*prm_ptr).prm_stafil_ptrpc);
        (*prm_ptr).prm_stafil_ptrpc = NULL;
    }

    if ((*prm_ptr).prm_hisfil_ptrpc != NULL)
    {
        free ((*prm_ptr).prm_hisfil_ptrpc);
        (*prm_ptr).prm_hisfil_ptrpc = NULL;
    }

    if ((*prm_ptr).prm_repfil_ptrpc != NULL)
    {
        free ((*prm_ptr).prm_repfil_ptrpc);
        (*prm_ptr).prm_repfil_ptrpc = NULL;
    }

    /*
     *  ...and return.
     */
    return;
}


/************************************************************************/
/*                                  */
/*    SCL_DEFAULT_TDF - Set all of the bits in TDF defaults bitmask */
/*    ===============                                           */
/*                                  */
/*    Calling Format:                           */
/*  SCL_DEFAULT_TDF (diagstk_ptr, tdf_ptr);                 */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  tdf_ptr     = Pointer to TDF record to default.         */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to set all of the bits in a TDF record     */
/*    so that the entire record is defaulted.                           */
/*                                  */
/*    In addition, this routine is responsible for setting up any   */
/*    fields which do not have a default value but rather have an   */
/*    initial value.                            */
/*                                  */
/************************************************************************/

void    SCL_DEFAULT_TDF (struct diagstk *diagstk_ptr
                         ,struct tdf     *tdf_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */
    unsigned long   isub;           /* Loop control variable    */
    struct   tdp    *tdp_ptr;       /* Test Parameter pointer   */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During SCL_DEFAULT_TDF");

    /*
     *    Setup the internal defaults.
     */
    SETUP_INT_TDF_DEF (diagstk_ptr, tdf_ptr);

    /*
     *    Set each of the bits in the defaults bitmask for the TDF record first.
     */
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_LTNODE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_NUMBER;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_PHYSICAL_DEVICE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_SEQ_NO;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_START;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_CHARACTER_SIZE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_DELAY;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_WIDTH;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_PAGE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_BAUD_RATE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_TERMINAL_TYPE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_TERMINAL_ID;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_TRACEFLAG;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_TRACEFILE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_RECORD;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_RECORD_TYPE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_RECORDFILE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_TYPEAHEAD;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_STARTDELAY;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_ANSWERBACK;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_DETACH_EXIT;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_HANGUP_DETACH;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_STOPRECDET;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_FKEY_CONTROL;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_LTSERVICE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_TRM_NAME;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_USERNAME;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_LOGIN_TYPE;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_INPUT_TIMING;
    (*tdf_ptr).tdf_default_flags1l |= TDF_THRDF1_DEVICE_TYPE;

    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_AUTO_SYNC;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_ENABLE_IOM;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_TCPIPADDR;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_TRMINFO;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_TST_UNIT;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_TCPIP_PORT;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_USE_TCP_SOCKET;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_EMULMOD;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_EMULTYP;
    (*tdf_ptr).tdf_default_flags2l |= TDF_THRDF2_RECORD_OPTS;

    /*
     *    Handle any fields which are not defaulted.
     */
    (*tdf_ptr).tdf_scpprm_countl = 0;   /* Zeroize parameter count  */

    /*
     *    ...and initialize the script parameters.
     */
    tdp_ptr = (*tdf_ptr).tdf_paramsr8;  /* Pointer to script parameters */
    for (isub = 0; isub<8; isub++, tdp_ptr++)
    {
        SCL_DEFAULT_TDP (diagstk_ptr, tdp_ptr);
    }

    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    SCL_DEFAULT_TDP - Set all of the bits in TDP defaults bitmask */
/*    ===============                                           */
/*                                  */
/*    Calling Format:                           */
/*  SCL_DEFAULT_TDP (diagstk_ptr, tdp_ptr);             */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  tdp_ptr     = Pointer to TDP record to default.         */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to set all of the bits in a TDP record     */
/*    so that the entire record is defaulted.                           */
/*                                  */
/************************************************************************/

void    SCL_DEFAULT_TDP (struct diagstk *diagstk_ptr
                         ,struct tdp     *tdp_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During SCL_DEFAULT_TDP");

    /*
     *    Initialize the script parameters.
     */
    (void) memset ((*tdp_ptr).tdp_char_paramc52 ,'\0'
                   ,sizeof ((*tdp_ptr).tdp_char_paramc52));
    /* Clear parameter value    */
    (*tdp_ptr).tdp_int_paraml = 0;
    (*tdp_ptr).tdp_paramtypel = 0;
    (*tdp_ptr).tdp_default_flagsl |= TDPD_PARAMECHO;
    /* ...echo is defaulted     */
    (*tdp_ptr).tdp_default_flagsl |= TDPD_PARAMUPCASE;
    /* ...in uppercase is defaulted */
    (*tdp_ptr).tdp_default_flagsl |= TDPD_PROMPT;
    /* ...prompt string is defaulted */
    (*tdp_ptr).tdp_default_flagsl |= TDPD_PARAMTYPE;
    /* ...parameter type is defaulted */
    (*tdp_ptr).tdp_default_flagsl |= TDPD_PARAMVAL;           /*CTA4253*/
    /* ...parameter value is defaulted */
    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    VTEST_DEFAULT_TLV - Set all of the bits in TLV defaults bitmask   */
/*    =================                                             */
/*                                  */
/*    Calling Format:                           */
/*  VTEST_DEFAULT_TLV (diagstk_ptr, tlv_ptr);           */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  tlv_ptr     = Pointer to TLV record to default.         */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to set all of the bits in a TLV record     */
/*    so that the entire record is defaulted.                           */
/*                                  */
/*    In addition, this routine is responsible for setting up any   */
/*    fields which do not have a default value but rather have an   */
/*    initial value.                            */
/*                                  */
/************************************************************************/

static  void    VTEST_DEFAULT_TLV   (struct diagstk *diagstk_ptr
                                     ,struct tlv     *tlv_ptr)

{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During VTEST_DEFAULT_TLV");

    /*
     *    Setup the internal defaults.
     */
    SETUP_INT_TLV_DEF (diagstk_ptr, tlv_ptr);

    /*
     *    Set the following fields to be defaulted:
     *  TLV_LOGICAL_NODEC20 
     */
    (*tlv_ptr).tlv_default_flagsl = 0;  /* Clear all flags initially    */
    (*tlv_ptr).tlv_default_flagsl |= TLVD_LOGICAL_NODE;
    /* ...and set LOGICAL NODE */
    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    SCL_DEFAULT_PRM - Set all of the bits in PRM defaults bitmask */
/*    ================                                          */
/*                                  */
/*    Calling Format:                           */
/*  SCL_DEFAULT_PRM (diagstk_ptr, tlv_ptr);             */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  prm_ptr     = Pointer to PRM record to default.         */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to set all of the bits in a PRM record     */
/*    so that the entire record is defaulted.                           */
/*                                  */
/*    In addition, this routine is responsible for setting up any   */
/*    fields which do not have a default value but rather have an   */
/*    initial value.                            */
/*                                  */
/************************************************************************/

void    SCL_DEFAULT_PRM (struct diagstk *diagstk_ptr
                         ,struct prm     *prm_ptr)

{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr,"During SCL_DEFAULT_PRM");

    /*
     *    Setup the internal defaults.
     */
    SETUP_INT_PRM_DEF (diagstk_ptr, prm_ptr);

    /*
     *    ...and set all of the fields in the bitmask.
     */
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_MESSAGE_LOG;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_MAX_THREADS;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_STATISTICS_FILE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_INPUT_RATE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_WSQUOTA;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_ENVIRONMENT;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_SCOPE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_HARDCOPY_LOG;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_PRIORITY;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_HISTORY_FILE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_REPORT_FILE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_LOG_TYPE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_PRIORITY;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_DUMP_STYLE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_SHARED;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_AUTOCLOSE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_ABORTFAIL;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_MAX_CONCURRENT;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_TRACE_MASK;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_IGNRINF;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_FAILTC;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_PASSTC;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_COMPTEST;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_ABRTTEST;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_SCP_PROT;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_TST_PROT;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_PRC_TYPE;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_HISTTEM;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_REPTEM;
    (*prm_ptr).prm_default_flagsl |= PRM_TSTDF_DSC;

    /*
     *    ...remove any diagnostics and return.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    SETUP_INT_TDF_DEF  - Setup the internal defaults for a TDF record */
/*    ==================                                        */
/*                                  */
/*    Calling Format:                           */
/*  SETUP_INT_TDF_DEF (diagstk_ptr, int_tdf_ptr);           */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  int_tdf_ptr = Pointer to the Internal TDF Defaults record   */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to initialize the TDF record to hold the   */
/*    internal default values.                      */
/*                                  */
/************************************************************************/

static void SETUP_INT_TDF_DEF   (struct diagstk *diagstk_ptr
                                 ,struct tdf     *int_tdf_ptr)

{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr ,"During SETUP_INT_TDF_DEF");

    /*
     *    ...and set up the internal default values for those fields which have 
     *    a fixed default.
     */
    (*int_tdf_ptr).tdf_seq_nol = 0;
    (*int_tdf_ptr).tdf_terminal_typel = TOF_THRTTP_PSEUDO;
    (*int_tdf_ptr).tdf_numberl = 1;
    (*int_tdf_ptr).tdf_physical_devicec36 [0] = '\0';
    (*int_tdf_ptr).tdf_answerbackc32 [0] = '\0';
    (*int_tdf_ptr).tdf_baud_ratel = TOF_THRBDR_9600;
    (*int_tdf_ptr).tdf_startl = 0;
    (*int_tdf_ptr).tdf_delayl = 0;
    (*int_tdf_ptr).tdf_device_typel = DEVICE_TYPE_VT300;
    (*int_tdf_ptr).tdf_terminal_idl = TOF_THRTID_VT320;
    (*int_tdf_ptr).tdf_widthl = 80;
    (*int_tdf_ptr).tdf_pagel = 24;
    (*int_tdf_ptr).tdf_traceflagl = TOF_GENSTA_DISABLED;
    (*int_tdf_ptr).tdf_recordl = TOF_GENSTA_DISABLED;
    (void) strcpy ((*int_tdf_ptr).tdf_tracefilec52 ,"<terminal>");
    (void) strcpy ((*int_tdf_ptr).tdf_recordfilec52 ,"<terminal>");
    (*int_tdf_ptr).tdf_typeaheadl = TOF_GENSTA_DISABLED;
    (*int_tdf_ptr).tdf_record_typel = (TOF_THRRAT_INPUT | TOF_THRRAT_OUTPUT);
    (*int_tdf_ptr).tdf_character_sizel = CHARACTER_SIZE_7BIT;
    (*int_tdf_ptr).tdf_detach_exitl = TOF_GENSTA_DISABLED;
    (*int_tdf_ptr).tdf_hangup_detachl = TOF_GENSTA_ENABLED;
    (*int_tdf_ptr).tdf_stoprecdetl = TOF_GENSTA_DISABLED;
    (*int_tdf_ptr).tdf_auto_syncl = TOF_GENSTA_ENABLED;
    (*int_tdf_ptr).tdf_enable_ioml = TOF_GENSTA_ENABLED;
    (*int_tdf_ptr).tdf_fkey_controll = CHARACTER_SIZE_7BIT;

    (void) memset ((*int_tdf_ptr).tdf_ltservicec20 ,'\0'
                   ,sizeof ((*int_tdf_ptr).tdf_ltservicec20));

    (void) memset ((*int_tdf_ptr).tdf_fqtcpip_addressc128 ,'\0'
                   ,sizeof ((*int_tdf_ptr).tdf_fqtcpip_addressc128));

    (void) memset ((*int_tdf_ptr).tdf_unit_namec20 ,'\0'
                   ,sizeof ((*int_tdf_ptr).tdf_unit_namec20));

    (void) memset ((*int_tdf_ptr).tdf_ltnodec8 ,'\0'
                   ,sizeof ((*int_tdf_ptr).tdf_ltnodec8));

    /*
     *    The Terminal information field has an initial value of 'vt220'
     */
    (void) memset ((*int_tdf_ptr).tdf_terminfo_namec36 ,'\0'
                   ,sizeof ((*int_tdf_ptr).tdf_terminfo_namec36));
    (void) strcpy ((*int_tdf_ptr).tdf_terminfo_namec36 ,"vt220");

    /*
     *    The username field has an internal default of "<Calc>". 
     *    This means that the username of the current user should be displayed.
     */
    (void) strcpy ((*int_tdf_ptr).tdf_usernamec36 ,"<Calc>");

    /*
     *    setup the internal defaults for
     *
     *  SOCKET CONNECTION = TOF_GENSTA_DISABLED
     */
    (*int_tdf_ptr).tdf_use_tcp_socketl = TOF_GENSTA_DISABLED;

    /*
     *    Note that there are a number of fields which have default values based 
     *    on values for other fields. These are as follows:
     *
     *    tdf_unit_namec20  - Defaults to the script name
     *    tdf_trm_namec20   - Defaults to the test unit name
     *    tdf_login_typel   - Default is based on Terminal Type
     *    tdf_input_timingl - Default is based on Terminal type
     *
     *    These should all be resolved manually by calling the appropriate routine
     *
     *    ...and return after removing diagnostics added.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    SETUP_INT_PRM_DEF  - Setup the internal defaults for a PRM record */
/*    ==================                                        */
/*                                  */
/*    Calling Format:                           */
/*  SETUP_INT_PRM_DEF (diagstk_ptr, int_prm_ptr);           */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  int_prm_ptr = Pointer to the Internal TDF Defaults record   */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to initialize the TDF record to hold the   */
/*    internal default values.                      */
/*                                  */
/************************************************************************/

static void SETUP_INT_PRM_DEF   (struct diagstk *diagstk_ptr
                                 ,struct prm     *int_prm_ptr)

{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr ,"During SETUP_INT_PRM_DEF");

    /*
     *    ...and set up the internal default values for those fields which have 
     *    a fixed default.
     */
    (*int_prm_ptr).prm_max_threadsl = 0;
    /* ...and max threads to 0  */
    (*int_prm_ptr).prm_sharedl = LGN_CALC_CALC;
    /* ...and shared to <Calc>  */
    (*int_prm_ptr).prm_autoclosel = LGN_CALC_CALC;
    /* ...and autoclose to <Calc>   */
    (void) memset ((*int_prm_ptr).prm_hardcopy_logc52, '\0'
                   ,sizeof ((*int_prm_ptr).prm_hardcopy_logc52));
    /* ...and no hardcopy log   */
    (*int_prm_ptr).prm_abortfaill = TOF_GENSTA_ENABLED;
    /* ...and abort on failure  */
    (void) memcpy ((*int_prm_ptr).prm_input_ratec8, "5.0 ", 4);
    /* ...and input rate to 5cps    */
    (*int_prm_ptr).prm_log_typel = AUDIT_TRAIL_NEW;
    /* ...and log type to 'New' */
    (*int_prm_ptr).prm_dump_stylel = TOF_GENSTA_DISABLED;
    /* ...and no dump on exception  */

    (*int_prm_ptr).prm_ignrinfl = TOF_GENSTA_DISABLED;
    /* Do not discard any messages  */


    /*
     *    The Scope/environment fields do not have a fixed default by the time
     *    they reach LGR but do for listing purposes. We therefore set the internal
     *    default value to be RUNTIME/WORLD
     */
    (*int_prm_ptr).prm_scopel = VTEST_SCOPE_WORLD;
    /* Default scope is WORLD   */
    (void) strcpy ((*int_prm_ptr).prm_environmentc12 ,"RUNTIME");
    /* ...and 'RUNTIME' environment */

    /*
     *    The run-time priority internal default is 999 which will be displayed as
     *    <Calc> by SLB during listing.
     */
    (*int_prm_ptr).prm_priorityl = 999;

    /*
     *    The WS Quota default is 0 which will be displayed as
     *    <Calc> by SLB during listing.
     */
    (*int_prm_ptr).prm_wsquotal = 0;

    /*
     *    The maximum threads default is 0 which will be displayed as
     *    <Calc> by SLB during listing.
     */
    (*int_prm_ptr).prm_max_threadsl = 0;

    /*
     *    The maximum concurrent ternminals default is 0 which will be displayed as
     *    <Calc> by SLB during listing.
     */
    (*int_prm_ptr).prm_max_concurrentl = 0;

    /*
     *    The Enable tracing mask field internal default is zero.
     */
    (*int_prm_ptr).prm_trace_maskl = 0;

    /*
     *    We also have special UNIX defaults for file specifications.
     */
#if defined(UNIX) || defined(WIN32)
    (void) strcpy ((*int_prm_ptr).prm_message_logc52, "lgn.mlg");
    (void) strcpy ((*int_prm_ptr).prm_statistics_filec52, "lgn.sta");
#endif

    /*
     *    Set up the new message template defaults.
     */
    (void) strcpy ((*int_prm_ptr).prm_failtcc84, "^D  ^L  FAIL ^C");
    (void) strcpy ((*int_prm_ptr).prm_passtcc84, "^D  ^L  PASS ^C");
    (void) strcpy ((*int_prm_ptr).prm_comptestc84,
                   "^D  ^T  Test completed with ^N test cases and ^A anomalies");
    (void) strcpy ((*int_prm_ptr).prm_abrttestc84,
                   "^D  ^T  Test aborted prematurely by ^S");
    /*
     *    Set up the default templates for the history and report files
     */
    (void) strcpy ((*int_prm_ptr).prm_histtemc84, "^D  ^T  ^X");
    (void) strcpy ((*int_prm_ptr).prm_reptemc84,  "^D  ^L  ^X");

    /*
     *    Script and Test Protection masks - clear masks indicate all access.
     */
    (*int_prm_ptr).prm_scp_protl = 0;
    (*int_prm_ptr).prm_tst_protl = 0;

    /*
     *    Process type defaults to DETACHED.
     */
    (*int_prm_ptr).prm_process_typel = PRTYP_DETACHED;

    /*
     *    ...and return after removing diagnostics added.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    SETUP_INT_TLV_DEF  - Setup the internal defaults for a TLV record */
/*    ==================                                        */
/*                                  */
/*    Calling Format:                           */
/*  SETUP_INT_TLV_DEF (diagstk_ptr, int_tlv_ptr);           */
/*                                  */
/*    Where:                                */
/*  diagstk_ptr = Pointer to the diagnostics stack to use       */
/*  int_tlv_ptr = Pointer to the Internal TLV Defaults record   */
/*                                  */
/*    Return Status                             */
/*  N/A                             */
/*                                  */
/*    This routine is called to initialize the TLV record to hold the   */
/*    internal default values.                      */
/*                                  */
/************************************************************************/

static void SETUP_INT_TLV_DEF   (struct diagstk *diagstk_ptr
                                 ,struct tlv     *int_tlv_ptr)
{
    /*
     *    Local Data
     */
    unsigned long   saved_diagctx;      /* Saved diagnostics context    */

    /*
     *    First things first, add a diagnostic message to the stack.
     */
    saved_diagctx = PSL_ADD_STATDIAG (diagstk_ptr ,"During SETUP_INT_TLV_DEF");


    /*
     *    The Logical node internal default is NULL. This gets displayed as <Local>
     */
    (void) memset ((*int_tlv_ptr).tlv_logical_nodec20 ,'\0'
                   ,sizeof ((*int_tlv_ptr).tlv_logical_nodec20));

    /*
     *    ...and return after removing diagnostics added.
     */
    PSL_REMOVE_DIAGNOSTICS (diagstk_ptr, saved_diagctx);
    return;
}

/************************************************************************/
/*                                  */
/*    VTEST_LOGIN_TYPE_DEF - Default routine for the LOGIN TYPE field   */
/*    ====================                                          */
/*                                  */
/*    Calling Format:                           */
/*  login_type = VTEST_LOGIN_TYPE_DEF (terminal_type);      */
/*                                  */
/*    Where:                                */
/*  terminal_type - long type of terminal               */
/*                                  */
/*    Return Status                             */
/*  login_type - Returned internal default login type.      */
/*                                  */
/*    This routine is called to determine the default value for the     */
/*    LOGIN_TYPE field in a Terminal Definition Record.         */
/*                                  */
/*    The LOGIN TYPE field is dependant on the TERMINAL TYPE.       */
/*    If the TERMINAL TYPE is REMOTE then the LOGIN TYPE is     */
/*    TELNET on UNIX, otherwise the LOGIN TYPE is AUTO      .   */
/*                                  */
/************************************************************************/

static    long  VTEST_LOGIN_TYPE_DEF (long terminal_type)

{
    /*
     *    If the terminal type is NOT remote the login type is 'Auto' otherwise it
     *    is 'Script'
     */
    if ((terminal_type != TOF_THRTTP_REMLAT)
            &&  (terminal_type != TOF_THRTTP_REMTCPIP))
    {
        return TOF_THRLOG_AUTO;
    }
    else
    {
        return TOF_THRLOG_TELNET;
    }
}

/************************************************************************/
/*                                  */
/*    VTEST_TIMING_DEF - Default routine for the TIMING field       */
/*    ================                                          */
/*                                  */
/*    Calling Format:                           */
/*  timing = VTEST_TIMING_DEF (terminal_type);          */
/*                                  */
/*    Where:                                */
/*  terminal_type - long type of terminal               */
/*                                  */
/*    Return Status                             */
/*  timing - Returned internal default timing value         */
/*                                  */
/*    This routine is called to determine the default value for the     */
/*    TIMING field in a Terminal Definition Record.             */
/*                                  */
/*    The TIMING field is dependant on the TERMINAL TYPE.       */
/*    If the TERMINAL TYPE is REMOTE then the TIMING is COARSE      */
/*    otherwise the TIMING is FINE.                 */
/*                                  */
/************************************************************************/

static    long  VTEST_TIMING_DEF (long terminal_type)

{
    /*
     *    If the terminal type is NOT remote the timing is 'Fine' otherwise it
     *    is 'Coarse'
     */
    if ((terminal_type != TOF_THRTTP_REMLAT)
            &&  (terminal_type != TOF_THRTTP_REMTCPIP))
    {
        return TOF_THRITM_FINE;
    }
    else
    {
        return TOF_THRITM_COARSE;
    }
}

/************************************************************************/
/*                                  */
/*    VTEST_PORT_DEF - Default routine for the TDF PORT NUMBER field    */
/*    ==============                                            */
/*                                  */
/*    Calling Format:                           */
/*  number = VTEST_PORT_DEF (login_type);               */
/*                                  */
/*    Where:                                */
/*  login type - long login type                    */
/*                                  */
/*    Return Status                             */
/*  number - Returned internal default port number          */
/*                                  */
/*    This routine is called to determine the default value for the     */
/*    PORT NUMBER field in a Terminal Definition Record.        */
/*                                  */
/*    The TIMING field is dependant on the LOGIN TYPE.          */
/*    If the LOGIN TYPE is Telnet then the PORT NUMBER is 23, if the    */
/*    LOGIN TYPE is RLOGIN the PORT NUMBER is 513 otherwise it is 0 */
/*                                  */
/************************************************************************/
long VTEST_PORT_DEF (long login_type)

{
    /*    If the LOGIN TYPE is Telnet then the PORT NUMBER is 23, if the
     *    LOGIN TYPE is RLOGIN the PORT NUMBER is 513 otherwise it is 0
     */
    if (login_type == TOF_THRLOG_TELNET)
    {
        return 23;
    }
    else if (login_type == TOF_THRLOG_RLOGIN)
    {
        return 513;
    }
    else
    {
        return 23;
    }
}
