/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECDLLFunctions.cpp
// -------------------
//
//    This is the implementation file for the CECDLLFunctions class
//
//=========================================================================


#include "stdafx.h"
#include "CECDLLFunctions.h"
#include "..\Interfaces\CyrPlugIns.h"
#include "..\Interfaces\CyrCtrls.h"
#include "..\Common\CECRegistryTools.h"
#include "Solutions.h"
#include "..\Common\CECCyranoHome.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECDLLFunctions::CECDLLFunctions()
{
    m_pTrace = NULL;
}

CECDLLFunctions::CECDLLFunctions(CECTrace* pTrace)
{
    // And set up our local pointer to the trace file
    m_pTrace = pTrace;
}

CECDLLFunctions::~CECDLLFunctions()
{
    for(int nDllNdx=0;nDllNdx<m_PlugInDLLs.GetSize();nDllNdx++)
        delete m_PlugInDLLs[nDllNdx];
    m_PlugInDLLs.RemoveAll();
    m_arStudioDLLs.RemoveAll();

    m_pTrace = NULL;
}


//
// @mfunc   Initialize the class with the appropriate DLLS from the specified initialization file
//
//
int CECDLLFunctions::Initialize     (const CString& strType, const CString& strSubType)
{
    //
    //    Translate the CYRANO_HOME environment variable. This gives us the base directory in which
    //    the plugin directories will be contained.
    //
    CECCyranoHome CyranoHome;
    if (CyranoHome.GetCyranoHome(m_CyranoHome) != 0) return -1;

    return LoadProductInfo(strType ,strSubType);
}


//
// @mfunc   GetFunctionPtr
//
//    Return a pointer to the specified function within either a specific or a generic DLL
//
FARPROC CECDLLFunctions::GetFunctionPtr (const CString& szFunctionName ,const CString& szSearchObject, BOOL bByType /*=TRUE*/) const
{
    FARPROC FunctionPtr = NULL;

    //
    //    First of all get a handle to the specified type of DLL (if specified at all)
    //
    HINSTANCE hDLL = GetDLL (szSearchObject, bByType);

    //
    //    ...then try to get a pointer to the specified function in the DLL
    //
    if (hDLL != NULL)
    {
        return GetFunction (hDLL ,szFunctionName);
    }

    //
    //    If we failed then check to see if we specified a type and if so check again without the type
    //
    if (!szSearchObject.IsEmpty())
    {
        HINSTANCE hDLL = GetDLL ("");

        if (hDLL != NULL)
        {
            FunctionPtr = GetFunction (hDLL ,szFunctionName);
        }
    }
    return FunctionPtr;
}


//
// @mfunc   GetDLL
//
//    Return the handle to the specified TYPE of DLL
//
HINSTANCE CECDLLFunctions::GetDLL   (const CString& szSearchObject, BOOL bByType /*=TRUE*/) const
{
    HINSTANCE hDLL = NULL;
    CECPlugInDLL* pPlugInDLL;

    //
    //    Get number of entries in the list
    //
    int LastItem = m_PlugInDLLs.GetUpperBound();

    //
    //    ..and loop through them looking for a DLL of the specified type
    //
    for (int i = 0; i <= LastItem && hDLL == NULL; i++)
    {
        pPlugInDLL = m_PlugInDLLs.GetAt(i);

        //
        //    And return the DLL depending on our search criteria
        //
        if (bByType)
        {
            if (szSearchObject == *pPlugInDLL->Type())
                hDLL = pPlugInDLL->hDLL();
        }
        else
        {
            if (szSearchObject == *pPlugInDLL->Title())
                hDLL = pPlugInDLL->hDLL();
        }
    }

    if (hDLL == NULL)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "GetDLL", "Failed to get a handle to <" + szSearchObject + ">");
    }

    return hDLL;
}


//
// @mfunc   GetFunction
//
//    Return a pointer to the specified function in the specified DLL
//
FARPROC CECDLLFunctions::GetFunction (HINSTANCE hDLL ,const CString& FunctionName) const
{
    FARPROC fProc = ::GetProcAddress(hDLL ,FunctionName);

    if (fProc == NULL)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "GetFunction", "Failed to get a handle to the function <" + FunctionName + ">");
    }

    return fProc;
}


//
// @mfunc   LoadProductInfo
//
// Finds and loads in all framework compliant product info structures so that we know
// a). what we are running and
// b). what we have to licence
//
int CECDLLFunctions::LoadProductInfo    (const CString strType ,const CString strSubType)
{
    if(strType.IsEmpty())
    {
        //      LoadAllPlugIns(,strSubType);
    }

    //
    //  ...load the specific product plugins.
    //

    int Status = LoadAllPlugIns (strType ,strSubType);
    return Status;
}

int CECDLLFunctions::LoadAllPlugIns (const CString& strType ,const CString& strSubType)
{

    //
    //    We do not know how many plugins have been registered - they are simply string values beneath
    //    this key. We therefore will iterate through these string values recovering the next plugin name
    //
    //    Note that there are two sets of plugins, Generic and product specific
    //
    CECRegistryTools reg;
    CString KeyName;
    CString PlugIn;
    CString SubType = strSubType;

    if (SubType.IsEmpty())
    {
        KeyName.Format ("%s\\%s\\PlugIns" ,SOLUTIONS_KEY ,strType);
    }
    else
    {
        KeyName.Format ("%s\\%s\\PlugIns\\%s" ,SOLUTIONS_KEY ,strType ,SubType);
    }

    //
    //    ...and open the plugins key
    //
    LONG lResult = reg.Open(HKEY_LOCAL_MACHINE, KeyName,KEY_READ);

    if (lResult != ERROR_SUCCESS)
    {
        AfxMessageBox ("FATAL Error while opening the PlugIns registry key");
        return -1;
    }

    //
    //    ...and iterate through its subkeys to identify the plugins.
    //
    TCHAR szBuffer[256];
    DWORD dwSize = 256;
    DWORD Index = 0;
    HKEY Key = reg.GetHKey();
    int     WorstStatus = 0;

    while (RegEnumKeyEx(Key, Index, szBuffer, &dwSize, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
    {
        m_nComponentCount++;
        Index++;

        //
        //    Format the name of the plugin given that it will be located in the \Plugins subdirectory
        //    of the CYRANO Solutions home directory
        //
#ifdef _DEBUG
        strcat (szBuffer ,"d");                     // Cater for debug images
#endif
        CString DLLName;
        DLLName.Format ("%sPlugIns\\%s.dll" ,m_CyranoHome ,szBuffer);


        //
        //    ...and attempt to load the DLL
        //
        CString RegisteredName = szBuffer;
        int Status  = LoadPlugIn (RegisteredName ,DLLName);
        if (Status != 0) WorstStatus = Status;

        //
        //    ...and loop for the next plugin (if any)
        //
        dwSize = 256;
    }

    //
    //    return the worst status which we have encountered.
    //
    return WorstStatus;
}


//
//    LoadPlugIn
//
//    Load a specified plugin and return its handle if successful
//
int     CECDLLFunctions::LoadPlugIn (const CString& RegisteredName ,const CString& DLLName)
{
    static int nGenericCount = 0;
    LPPRODUCTFUNC   lpfnGetProductInfo;
    LPCTRLSFUNC     lpfnGetCtrlsInfo;
    CString         RegisteredType;
    CString         RegisteredTitle = "";

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "LoadProductInfo", "Attempting to load <" + DLLName + ">");

    //
    //    If the file does not exist then we shall ignore this entry and remove it from the registry. This
    //    will allow us to handle un-installed products easily.
    //
    CFileStatus FileStatus;
    if (CFile::GetStatus(DLLName, FileStatus) == FALSE)
    {
#ifdef _DEBUG
        CString szMessage;
        szMessage.Format("The Plug-in DLL %s does not exist\n\nPlease check your installation",DLLName);
        AfxMessageBox (szMessage, MB_OK);
#endif
        return 1;
    }

    //
    //    ...and load in the library, saving its handle for later use.
    //
    HINSTANCE hDLL = AfxLoadLibrary(DLLName);
    if (hDLL == NULL)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "LoadProductInfo", "Failed to load <" + DLLName + ">");

        CString szMessage;
        szMessage.Format("Failed to load %s\n\nPlease check your installation",DLLName);
        AfxMessageBox (szMessage, MB_OK);
        return 1;
    }

    //
    //    Check for all DLL's which have the GetProductInfo function and if so add their names to the
    //    array of products.
    //
    lpfnGetProductInfo = (LPPRODUCTFUNC) GetProcAddress(hDLL, "GetProductInfo");
    struct PRODUCTINFO ProductInfo;

    //
    //    Check that we only load in products that are of the correct type
    //
    BOOL ProductDLL = FALSE;
    if (lpfnGetProductInfo != NULL && lpfnGetProductInfo)
    {
        // Get the product info
        BOOL bReturn = lpfnGetProductInfo(&ProductInfo);
        if (bReturn)
        {
            // Add this product to our internal list
            RegisteredType = ProductInfo.strDisplayName;
            RegisteredTitle = ProductInfo.strTitle;
            m_arProductArray.Add(RegisteredType);
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "LoadProductInfo", "<" + RegisteredName + "> registering as a Product DLL by name <" + RegisteredType + ">");
            ProductDLL = TRUE;
        }
    }

    if (!ProductDLL)
    {
        RegisteredType.Format("Generic%d" ,nGenericCount);
        nGenericCount++;
    }


    //
    //    If the DLL (product or extension) supports the GetCtrlsInfo function (that is have controls)
    //    then add it to the array of control DLL's
    //
    struct CTRLSINFO CtrlsInfo;
    CString strCtrlsDllType = "Generic Ctrls";

    lpfnGetCtrlsInfo = (LPCTRLSFUNC) GetProcAddress(hDLL, "GetCtrlsInfo");
    if (lpfnGetCtrlsInfo != NULL && lpfnGetCtrlsInfo)
    {
        BOOL bReturn = lpfnGetCtrlsInfo(&CtrlsInfo);
        if (bReturn)
        {
            RegisteredTitle = CtrlsInfo.strName;
            m_arControlsArray.Add(RegisteredType);
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "LoadProductInfo", "<" + RegisteredName + "> registering it controls by name <" + RegisteredType + ">");
        }
    }

    //
    //    Although this is not obvious we save ALL DLL's whether product or extension into the Studio DLL's
    //    array as this makes our searching easier later
    //
    m_arStudioDLLs.Add(RegisteredType);

    //
    //    All DLL's also exist in the Plug-In DLL's list
    //
    CString PlugInName = RegisteredName;
    CECPlugInDLL* pNewPlugInDLL =
        (CECPlugInDLL*) new CECPlugInDLL (hDLL ,PlugInName ,RegisteredType ,RegisteredTitle);
    m_PlugInDLLs.Add (pNewPlugInDLL);
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECDLLFunctions", "LoadProductInfo", "<" + RegisteredName + "> added as a Studio DLL" );

    return 0;
}

//
// GlobalFireEvent
// ===============
//
// This routine handles a change of project in the framework by firing an event
// to all the connected plug-ins.
//
LRESULT CECDLLFunctions::GlobalFireEvent (UINT nID, LONG lParam)
{
    for (int i = 0; i <= m_PlugInDLLs.GetUpperBound(); i++)
    {
        LPONEVENTFUNC   lpfnOnEvent;

        // Get a pointer to each DLL in turn
        CECPlugInDLL* pPlugInDLL = m_PlugInDLLs.GetAt(i);

        if (pPlugInDLL == NULL)
        {
            // Must of reached the end of the list
            break;
        }

        // See if it has an OnEvent function
        lpfnOnEvent = (LPONEVENTFUNC)::GetProcAddress(pPlugInDLL->hDLL(),"OnEvent");
        if (lpfnOnEvent)
        {
            // If it does then call it
            lpfnOnEvent(&nID, &lParam);
        }
    }
    return 0;
}


//
// SpecificFireEvent
// =================
//
// This routine fires an event to the specified named DLL
//
// @rdesc returns 0 if no DLL found, otherwise returns status from the message handler
//
//
LRESULT CECDLLFunctions::SpecificFireEvent (UINT nID, LONG lParam)
{
    LRESULT RetVal = 0;                     // Assume DLL does not exist;

    //
    //    The message to be fired is passed as the first argument, the second argument is the name
    //    of the DLL to fire the message to.with potentially trailing arguments separated by a '|'
    //
    CString*    pDllName = (CString*) lParam;
    CString Parameter = "";
    CString DllName = *pDllName;

    int Separator = pDllName->Find('|');
    if (Separator != -1)
    {
        Parameter = pDllName->Mid(Separator + 1);
        DllName = pDllName->Left(Separator);
    }

    CString DebugDllName = DllName + "d";

    for (int i = 0; i <= m_PlugInDLLs.GetUpperBound(); i++)
    {
        LPONEVENTFUNC   lpfnOnEvent;

        // Get a pointer to each DLL in turn
        CECPlugInDLL* pPlugInDLL = m_PlugInDLLs.GetAt(i);

        if (pPlugInDLL == NULL)
        {
            // Must of reached the end of the list
            break;
        }

        //    If it is not the named DLL then skip it.
        if ((!DllName.CompareNoCase(*pPlugInDLL->Name()))
                ||  (!DebugDllName.CompareNoCase(*pPlugInDLL->Name())))
        {

            // See if it has an OnEvent function
            lpfnOnEvent = (LPONEVENTFUNC)::GetProcAddress(pPlugInDLL->hDLL(),"OnEvent");
            if (lpfnOnEvent)
            {
                // If it does then call it
                return lpfnOnEvent(&nID, (LONG*) &Parameter);
            }
        }
    }
    return RetVal;
}

