/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TOptions.cpp
/**
 * Method defintions for TOptions class
 * This class is a random access holder for strings indexed by
 * AtvfKeys
 **/

// Local includes
#include "TOptions.h"

/**
 * Default Constructor
 **/
TOptions::TOptions()
{}

/**
 * Copy Constructor
 **/
TOptions::TOptions(const TOptions& options)
{
    // Loop through and delete all Atvf records and its list
    for(topt_cont_t::const_iterator i = options._opts.begin();
            i != options._opts.end();
            i++)
    {
        _opts[(*i).first] = new string(*((*i).second));
    }
}

/**
 * Destructor
 **/
TOptions::~TOptions()
{
    // Loop through and delete all Atvf records and its list
    for(topt_cont_t::iterator i = _opts.begin();
            i != _opts.end();
            delete (*(i++)).second)
        ; // No 'for' content
//    _opts.erase(_opts.begin(), _opts.end()); // no clear in g++
}

/**
 * Accessor to find the value of an option with a particular key
 **/
string TOptions::value(const string& key) const
{
    topt_cont_t::const_iterator i = _opts.find(AtvfKey(key));
    if(i == _opts.end())
    {
        return string("");
    }
    else
    {
        return *((*i).second);
    }
}

/**
 * Accessor to find out if an option with a particular key exists
 **/
bool TOptions::exists(const string& key) const
{
    topt_cont_t::const_iterator i = _opts.find(AtvfKey(key));
    if(i == _opts.end())
    {
        return false;
    }
    else
    {
        return true;
    }
}

/**
 * Accessor to find out if no options.
 **/
bool TOptions::empty() const
{
    return _opts.empty();
}

/**
 * Mutator to add another option to the set.  The passed in text
 * is interpreted so that the first 'word' within it is used as the key,
 * Where word is defined from sample begginning to first whitespace, and the
 * text immediately after the whitespace after the key until the end of
 * the string is the value.
 **/
void TOptions::add(const string& text)
{
    // Parse the text to extract the useful bits
    string::size_type pos = text.find_first_of(" \t");
    string key = text.substr(0, pos);
    string value;
    if(pos != string::npos)
    { // text not contain just key
        pos = text.find_first_not_of(" \t", pos);
        if(pos != string::npos)
        { // text not contain just key
            value = text.substr(pos);
        }
    }
    AtvfKey atvfKey(key);
    topt_cont_t::const_iterator i = _opts.find(atvfKey);
    if(i != _opts.end()) delete (*i).second;
    _opts[atvfKey] = new string(value);
}

/**
 * Mutator to add group of options to the set. 
 **/
void TOptions::add(const TOptions& opts)
{
    for(topt_cont_t::const_iterator i = opts._opts.begin();
            i != opts._opts.end();
            i++)
    {
        _opts[(*i).first] = new string(*((*i).second));
    }
}

/**
 * Friend operator <<
 **/
ostream& operator <<(ostream& os, const TOptions& top)
{
    os << "Options : ";

    if (!top.empty())
    {
        for (topt_cont_t::const_iterator optIt = top._opts.begin();
                optIt != top._opts.end(); ++optIt)
            (optIt == top._opts.begin()) ?
            os << (*optIt).first << " = " << *((*optIt).second):
            os << endl << "          " <<(*optIt).first << " = "<< *((*optIt).second);
    }
    else
        os << "<None>";

    return os;
}


