/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECOperand.cpp
#ifdef HPUX
#include <iostream.h>
#else
#include <iostream>
#endif
#include <string>

#include "CECOperand.h"
#include "cxxutils.h"
#include "TPLog.h"
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor to create a NULL operand object.
//
#ifndef TH_POOL_VER
CECOperand::CECOperand() :
        _addrM(addr_none), _dTyp(dtyp_none), _chrValP(0), _chrValLen (0),
        _fncCod(fnc_none), _intVal(0), _opr(opr_none), _varNamP(0), _varDims(0)
#else
CECOperand::CECOperand() :
        _addrM(addr_none), _dTyp(dtyp_none),
        _fncCod(fnc_none), _intVal(0), _opr(opr_none), _varDims(0)
#endif
{
    ; // Do nothing
} // End CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor to create an operand object from a function code.
//
// @parm    Function code.
//
#ifndef TH_POOL_VER
CECOperand::CECOperand(const FncCode fnccod) :
        _addrM(addr_function), _dTyp(dtyp_none), _chrValP(0), _chrValLen (0),
        _fncCod(fnccod), _intVal(0), _opr(opr_none), _varNamP(0), _varDims(0)
#else
CECOperand::CECOperand(const FncCode fnccod) :
        _addrM(addr_function), _dTyp(dtyp_none),
        _fncCod(fnccod), _intVal(0), _opr(opr_none), _varDims(0)
#endif
{
    switch (_fncCod)
    {
    case fnc_extract:
        _dTyp = dtyp_character;
        break;
    case fnc_locate:
        _dTyp = dtyp_integer;
        break;
    default:
        _dTyp = dtyp_none;
    }
    return;
} // End CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor to create an operand object from an operator.
//
// @parm    Operator.
//
#ifndef TH_POOL_VER
CECOperand::CECOperand(const Operator opr) :
        _addrM(addr_operator), _dTyp(dtyp_none), _chrValP(0), _chrValLen (0),
        _fncCod(fnc_none), _intVal(0), _opr(opr), _varNamP(0), _varDims(0)
#else
CECOperand::CECOperand(const Operator opr) :
        _addrM(addr_operator), _dTyp(dtyp_none),
        _fncCod(fnc_none), _intVal(0), _opr(opr), _varDims(0)
#endif
{
    ; // Do nothing
} // End CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor to create an operand object from an immediate integer
//			value.
//
// @parm    String from which operand to be created.
// @parm    Addressing mode for operand.
//

CECOperand::CECOperand(const string strval, const AddrMode addrm)
{
    if (addrm == addr_variable)		// Set variable operand specific data
    {
        _addrM = addrm;
        _dTyp = dtyp_none;

#ifndef TH_POOL_VER
        _chrValLen = 0;
        _chrValP = 0;
        size_t namlen = strval.length();			// Variable name length
        _varNamP = new char [namlen + 1];			// Space for name and NULL terminator
        memcpy (_varNamP, strval.c_str(), namlen);	// Copy name into new space
        *(_varNamP + namlen) = '\0';				// NULL terminate new name
#else
        _varNamP = strval;
#endif
    }
    else if (addrm == addr_immediate)	// Set Immediate operand specific data
    {
        _addrM = addrm;
        _dTyp = dtyp_character;
#ifndef TH_POOL_VER
        _chrValLen = (long) strval.length();			// Length of value
        _chrValP = new char [_chrValLen + 1];			// Space for value and NULL terminator
        memcpy (_chrValP, strval.c_str(), _chrValLen);	// Copy value into new space
        *(_chrValP + _chrValLen) = '\0';			// NULL terminate new value
        _varNamP = 0;
#else
        _chrValP = strval;
#endif
    }
    else				// Set Null operand specific data
    {
        _addrM = addr_none;
        _dTyp = dtyp_none;
#ifndef TH_POOL_VER
        _chrValLen = 0;
        _chrValP = 0;
        _varNamP = 0;
#endif
    }
    // Set remaining 'common' data.
    _intVal = 0;
    _opr = opr_none;
    _fncCod = fnc_none;
    _varDims = 0;
    return;
} // End CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor to create a variable operand object from a string
//			supplying the variable name, the data type and the number of
//			array dimensions (zero if not an array).
//
// @parm    Variable name for new variable operand.
// @parm    Data type of new variable operand.
//
#ifndef TH_POOL_VER
CECOperand::CECOperand(const string strval, const DataType dtyp, const long vardims) :
        _addrM(addr_variable), _dTyp(dtyp), _chrValP(0), _chrValLen (0), _fncCod(fnc_none),
        _intVal(0), _opr(opr_none), _varDims(vardims)
#else
CECOperand::CECOperand(const string strval, const DataType dtyp, const long vardims) :
_addrM(addr_variable), _dTyp(dtyp), _fncCod(fnc_none),
_intVal(0), _opr(opr_none), _varDims(vardims)
#endif
{
    // Save variable name.
#ifndef TH_POOL_VER
    size_t namlen = strval.length();			// Variable name length
    _varNamP = new char [namlen + 1];			// Space for name and NULL terminator
    memcpy (_varNamP, strval.c_str(), namlen);	// Copy name into new space
    *(_varNamP + namlen) = '\0';				// NULL terminate new name
#else
    _varNamP = strval;
#endif
    return;
} // End CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor to create an operand object from an immediate integer
//			value.
//
// @parm    Integer value from which to create operand.
//
#ifndef TH_POOL_VER
CECOperand::CECOperand(const long intval) :
        _addrM(addr_immediate), _dTyp(dtyp_integer), _chrValP(0), _chrValLen (0), _fncCod(fnc_none),
        _intVal(intval), _opr(opr_none), _varNamP(0), _varDims(0)
#else
CECOperand::CECOperand(const long intval) :
        _addrM(addr_immediate), _dTyp(dtyp_integer), _fncCod(fnc_none),
        _intVal(intval), _opr(opr_none), _varDims(0)
#endif
{
    ; // Do nothing
} // End CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   CECOperand copy constructor. This constructor creates an operand
//			object from another operand object. It does this by copying the
//			member data from the supplied object to the new object.
//
// @parm    CECOperand object from which to create operand.
//

#ifndef TH_POOL_VER
CECOperand::CECOperand(const CECOperand& opd) :
        _addrM(opd._addrM), _dTyp(opd._dTyp), _chrValP(0), _chrValLen (0), _fncCod(opd._fncCod),
        _intVal(opd._intVal), _opr(opd._opr), _varNamP(0), _varDims(opd._varDims)
#else
CECOperand::CECOperand(const CECOperand& opd) :
        _addrM(opd._addrM), _dTyp(opd._dTyp), _fncCod(opd._fncCod),
        _intVal(opd._intVal), _opr(opd._opr), _varDims(opd._varDims)
#endif
{
#ifndef TH_POOL_VER
    // If there is an immediate character value, allocate space for a new one and
    // copy from the input object to the new object.
    if (opd._chrValP != 0)
    {
        _chrValLen = opd._chrValLen;						// Length of value
        _chrValP = new char [_chrValLen + 1];				// Allocate space for new value
        memcpy (_chrValP, opd._chrValP, _chrValLen + 1);	// Copy value into new space
    }
    // If there is a variable name, allocate space for a new one and
    // copy from the input object to the new object.

    if (opd._varNamP != 0)
    {
        size_t namlen = strlen (opd._varNamP);	// Length of name
        _varNamP = new char [namlen + 1];		// Allocate space for new name
        strcpy (_varNamP, opd._varNamP);		// Copy name into new space
    }

#else
_chrValP = opd._chrValP;
    _varNamP = opd._varNamP;
#endif

    return;
}



CECOperand& CECOperand::operator=(const CECOperand& opd)
{
    _addrM = opd._addrM;
    _dTyp = opd._dTyp;
    _fncCod = opd._fncCod;
    _intVal = opd._intVal;
    _opr = opd._opr;
    _varDims = opd._varDims;

#ifndef TH_POOL_VER
    if (_chrValP != 0)
    {
        delete [] _chrValP;
        _chrValP = 0;
    }
    if (_varNamP != 0)
    {
        delete [] _varNamP;
        _varNamP = 0;
    }

    // If there is an immediate character value, allocate space for a new one and
    // copy from the input object to the new object.
    if (opd._chrValP != 0)
    {
        _chrValLen = opd._chrValLen;						// Length of value
        _chrValP = new char [_chrValLen + 1];				// Allocate space for new value
        memcpy (_chrValP, opd._chrValP, _chrValLen + 1);	// Copy value into new space
    }



    // If there is a variable name, allocate space for a new one and
    // copy from the input object to the new object.

    if (opd._varNamP != 0)
    {
        size_t namlen = strlen (opd._varNamP);	// Length of name
        _varNamP = new char [namlen + 1];		// Allocate space for new name
        strcpy (_varNamP, opd._varNamP);		// Copy name into new space
    }

#else
    _chrValP = opd._chrValP;
    _varNamP = opd._varNamP;
#endif

    return *this;
}



///////////////////////////////////////////////////////////////////////////////
// @mfunc   Destructor. If space has been allocated for a character value or a
//			variable name, then delete it.
//
CECOperand::~CECOperand()
{
#ifndef TH_POOL_VER
    // If there is a character value, delete the space allocated for it.
    if (_chrValP != 0) delete [] _chrValP;

    // If there is a variable name, delete the space allocated for it.
    if (_varNamP != 0) delete [] _varNamP;
#endif
    return;
} // End ~CECOperand()

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Accessor function to retrieve character value as a string.
//
// @rdesc	Character value as string.
//
string CECOperand::ChrVal() const
{
#ifndef TH_POOL_VER
    string retstr;			// Character value string to return

    if (_chrValP == 0)
    {
        retstr = string("");
    }
    else
    {
        retstr.append(_chrValP, _chrValLen);
    }

    return retstr;
#else
    return _chrValP;
#endif
}



///////////////////////////////////////////////////////////////////////////////
// @mfunc   Construct and return a string listing the operand contents. This
//			string can be used log diagnostic information.
//
// @rdesc	String listing of the operand contents.
//
string CECOperand::GetContent()
{
    string val_addrm = "";
    string val_dtyp = "";
    string val_chrval = "None";
    string val_opr = "";
    string val_intval = "";
    string val_varnam = "None";
    string val_vardims = "";
    string val_fnc = "";

#ifndef CYR_NO_STD_NAMESPACE
    // Set addressing mode
    switch (_addrM)
    {
    case addr_none:
        val_addrm = "None";
        break;
    case addr_immediate:
        val_addrm = "Immediate";
        break;
    case addr_operator:
        val_addrm = "Operator";
        break;
    case addr_variable:
        val_addrm = "Variable";
        break;
    default:
        val_addrm = "Unknown (" + int_to_string(_addrM) + ")";
    }

    // Set data type
    switch (_dTyp)
    {
    case dtyp_none:
        val_dtyp = "None";
        break;
    case dtyp_integer:
        val_dtyp = "Integer";
        break;
    case dtyp_character:
        val_dtyp = "Character";
        break;
    default:
        val_dtyp = "Unknown (" + int_to_string(_dTyp) + ")";
    }

    // Set integer value, character value, variable name and variable dimensions.
    val_intval = int_to_string (_intVal);
#ifndef TH_POOL_VER
    if (_chrValP != 0) val_chrval = string(_chrValP);
    if (_varNamP != 0) val_varnam = string(_varNamP);
#else
    if (_chrValP.empty() == FALSE) val_chrval = _chrValP;
    if (_varNamP.empty() == FALSE) val_varnam = _varNamP;

#endif
    val_vardims = int_to_string(_varDims);

    // Set operator.
    switch (_opr)
    {
    case opr_none:
        val_opr = "None";
        break;
    case opr_not:
        val_opr = "Not";
        break;
    case opr_and:
        val_opr = "And";
        break;
    case opr_or:
        val_opr = "Or";
        break;
    case opr_eq:
        val_opr = "Eq";
        break;
    case opr_ne:
        val_opr = "Ne";
        break;
    case opr_lt:
        val_opr = "Lt";
        break;
    case opr_le:
        val_opr = "Le";
        break;
    case opr_gt:
        val_opr = "Gt";
        break;
    case opr_ge:
        val_opr = "Ge";
        break;
    case opr_contains:
        val_opr = "Contains";
        break;
    case opr_not_contains:
        val_opr = "Not_contains";
        break;
    case opr_add:
        val_opr = "Add";
        break;
    case opr_sub:
        val_opr = "Sub";
        break;
    case opr_mul:
        val_opr = "Mul";
        break;
    case opr_div:
        val_opr = "Div";
        break;
    case opr_mod:
        val_opr = "Mod";
        break;
    default:
        val_opr = "Unknown (" + int_to_string(_opr) + ')';
    }

    // Set function code.
    switch (_fncCod)
    {
    case fnc_none:
        val_fnc = "None";
        break;
    case fnc_extract:
        val_fnc = "~Extract";
        break;
    case fnc_locate:
        val_fnc = "~Locate";
        break;
    default:
        val_fnc = "Unknown (" + int_to_string(_fncCod) + ')';
    }

#endif
    return ("OPERAND: Addrm: " + val_addrm + "; Dtyp: " + val_dtyp +
            "; ChrVal: " + val_chrval + "; IntVal: " + val_intval +
            "; Opr: " + val_opr + "; VarNam: " + val_varnam +
            "; VarDims: " + val_vardims + "; FncCod: " + val_fnc + '.');
} // End GetContent()


///////////////////////////////////////////////////////////////////////////////
// @mfunc   Accessor function to retrieve variable name as a string.
//
// @rdesc	Variable name as string.
//
string CECOperand::VarNam() const
{
#ifndef TH_POOL_VER
    string retstr;			// Variable name string to return

    if (_varNamP == 0)
    {
        retstr = string("");
    }
    else
    {
        retstr = string(_varNamP);
    }

    return retstr;
#else
    return _varNamP;
#endif
}


