/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CECOperand.h
// @module  CECOperand class | This module defines the interface to the
//          CECOperand class.
///////////////////////////////////////////////////////////////////////////////

#if !defined (CYR_CECOPERAND)
#define CYR_CECOPERAND

#if defined(WIN32)
# if _MSC_VER >= 1200   // MSC version 6 and above
#pragma warning(disable:4786)
# endif
#endif

#include <psl_macros.h>
#include <string>
#include <list>
#include <vector>
#include <stack>

#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif

// Operand addressing modes.
enum AddrMode {addr_none, addr_immediate, addr_operator, addr_variable,
               addr_function};

// Operand data types.
enum DataType {dtyp_none, dtyp_integer, dtyp_character};

// Codes representing operators.
enum Operator {opr_none, opr_not, opr_and, opr_or, opr_eq, opr_ne, opr_lt, opr_le,
               opr_gt, opr_ge, opr_contains, opr_not_contains, opr_add, opr_sub,
               opr_mul, opr_div, opr_mod};

// Codes representing functions.
enum FncCode {fnc_none, fnc_locate, fnc_extract};

///////////////////////////////////////////////////////////////////////////////
// @class   Operand | Intrnal class to hold operand as either a PSL_LONGWORD_T
//			or a string expression string in SCL format.
//
class CECOperand
{
    // @access Public Members.
public:
    // @cmember Constructor to create a 'NULL' operand.
    CECOperand();
    // @cmember Constructor to create an operand from a function code.
    CECOperand(const FncCode fnccod);
    // @cmember Constructor to create an operand from an operator.
    CECOperand(const Operator opr);
    // @cmember Constructor to create an operand from a string value.
    CECOperand(const string strval, const AddrMode addr);
    // @cmember Constructor to create a variable operand.
    CECOperand(const string strval, const DataType dtyp);
    // @cmember Constructor to create a variable operand.
    CECOperand(const string strval, const DataType dtyp, const long vardims);
    // @cmember Constructor to create an operand from an integer value.
    CECOperand(const long intval);
    // @cmember CECOperand copy constructor.
    CECOperand(const CECOperand& opd);
    CECOperand& operator=(const CECOperand&);
    // @cmember Destructor.
    virtual ~CECOperand();
    // @cmember Accessor function to retrieve address mode.
    AddrMode Addrm() const
    {
        return _addrM;
    };
    // @cmember Accessor function to retrieve character value as a string.
    string ChrVal() const;
    // @cmember Accessor function to retrieve data type.
    DataType DTyp() const
    {
        return _dTyp;
    };
    // @cmember Accessor function to retrieve function code.
    FncCode FncCod() const
    {
        return _fncCod;
    };
    // @cmember Accessor function to retrieve integer value.
    long IntVal() const
    {
        return _intVal;
    };
    // @cmember Return 'true' if character operand, 'false' otherwise.
    bool IsChr() const
    {
        return (_dTyp==dtyp_character)?true:false;
    };
    // @cmember Return 'true' if integer operand, 'false' otherwise.
    bool IsInt() const
    {
        return (_dTyp==dtyp_integer)?true:false;
    };
    // @cmember Accessor function to retrieve operator type.
    Operator Opr() const
    {
        return _opr;
    };
    // @cmember Get string listing operand content.
    string GetContent();
    // @cmember Accessor function to retrieve variable name as a string.
    string VarNam() const;
    // @cmember Accessor function to retrieve variable dimensions.
    long VarDims() const
    {
        return _varDims;
    };
    // @access Private Members.
private:
    // @cmember Addressing mode.
    AddrMode _addrM;
    // @cmember Data type.
    DataType _dTyp;
    // @cmember Length of immediate character value.
    long _chrValLen;
    // @cmember Value of immediate character operand as character array.
    // N.B.: This field MAY contain NULL characters - therefore we save its
    // length. A terminating NULL character is appended, BUT this is NOT
    // part of the data.
#ifndef TH_POOL_VER
    char *_chrValP;
#else
    string _chrValP;
#endif
    // @cmember Value of immediate integer operand.
    long _intVal;
    // @cmember Operator for operator operand.
    Operator _opr;
    // @cmember Name of variable for variable operand as character array.
#ifndef TH_POOL_VER
    char *_varNamP;
#else
    string _varNamP;
#endif
    // @cmember Number of array dimensions for variable operand.
    long _varDims;
    // @cmember Function code for function operand.
    FncCode _fncCod;
};

#endif // !defined(CYR_CECOPERAND)
