/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////////////////////////////////////////////////////////////
// @doc     CORBARepository
// @module  CORBARepository | This is to be used a a single Global Class
//  to handle all interfaces with the repository. After Creation it will be shared 
//  accross all threads and must be MT-safe. It also looks after any differences between
//  standard and Stand Alone behaviour, usually doing nothing in standalone mode
//  ALL public methods (except those involved with creation) must be protected with a mutex
//  lock, this should ensure that only 1 thread acesses this object at any 1 time. This should 
//  prevent the need for mutexes around all members but will require that no public method
//  calls another public method as this will result in a deadlock situation. Hopefully this 
//  will not present a problem due to the limited functionality of this class
//
//  The Repository should only ever throw CyrRepositoryExceptions
//  As a failure in the repository is fairly bad these exceptions are left to be caught 
//  either at a higher level within the thread/process or to be sent back to a TestInitiator 
///////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#include <assert.h>

#include "TPLog.h"

#include "CORBARepository.h"
#include "mdbexcept.h"
#include "mdbtypes2.h"
#include "cxxutils.h"
#include "CECResultsFile.h"
#ifndef CYRANO_TP_STANDALONE
#include "CyrVDK004/TimeService.h"
#endif

#ifdef HPUX
#   include <fstream.h>
#else
#   include <fstream>
#endif

#ifndef CYR_NO_STD_NAMESPACE
    using namespace std;
#endif

#ifdef WIN32
#   pragma warning(disable:4786)
#   pragma warning(disable:4503)
#endif

#define SERVERNAME_LITERAL "Server"

const string TESTRUNSTRING_SEP="~";
const string trc_prefix = "T* ";
const string wrn_prefix = "W* ";
const string err_prefix = "E* ";

#define ONE_MINUTE (60 * 1000)

#ifndef TH_POOL_VER
CTMutex CORBARepository::public_method_mutex;
#else
CFastLock *CORBARepository::s_pPublicMethodFl = NULL;
#endif

extern string exedir;//global string variable

#ifdef __DLLSA__
    extern string filevardir;//global string variable
#endif

CORBARepository* Repository=NULL;
//@mfunc    Constructor, only called from main, therefore inherently MT-safe in this instance
//@parm	    argc
//@parm	    argv
CORBARepository::CORBARepository(CORBAServer* corba,string node,string user,string pass,int pid,string exename):
m_DataServerNodeName(node)
,m_pid(pid)
,m_UserName(user)
,m_Password(pass)
,m_exename(exename)
#ifndef CYRANO_TP_STANDALONE
,m_Server(NULL)
,m_Project(NULL)
,m_RootFolder(NULL)
,m_TestFolder(NULL)
,m_ObjectCodeFolder(NULL)
,m_NodeFolder(NULL)
,m_DataFolder(NULL)
,m_TestStore(NULL)
,m_TestRun(NULL)
,m_corba(corba)
,m_LocalResults(false)
,m_DeleteLocalResults(true)
#endif
{
    char buff[20];
    itoa(m_pid, buff, 10);
    m_szPid = buff;

#ifndef CYRANO_TP_STANDALONE
    m_ParentTestRunList.clear();

    // let exceptions be captured higher up, not much ew can do if repository fails
    // We need to connect to the Server. To do this we first need to create 
    // an instance of the server object.
    // Create a server object
    CECInetInfo info;
    string underscore_ip_name=info.GetIPAddressAsString(m_DataServerNodeName);
	
    string factory=underscore_ip_name+".CyranoFactory";
    string dataservername=underscore_ip_name+"_CyrStProvider_001";
    string msg="Node "+underscore_ip_name+" is not running a DataServer";
    try
    {
	CORBA::Object_ptr obj;
	obj=m_corba->FindObjectWithName(Provider::CLASS_NAME,dataservername.c_str());
	if(CORBA::is_nil(obj))
	    throw CyrRepositoryException(msg.c_str(),"CORBARepository::CORBARepository",-1);
    }
    catch(...)//Assume anything bad is installation problem
    {
        throw CyrRepositoryException(msg.c_str(),"CORBARepository::CORBARepository",-1);
    }
    if(logr) logr->AddTrace(STARTUP,DEBUGLEVEL,"Create Repository Server"); 
    try
    {
		m_Server = new CECOrbServer(m_corba, underscore_ip_name.c_str(), user.c_str(), pass.c_str());
    }
    catch(const CyrRepositoryException &ex)
    {
		string msg;
		msg += "Error Creating Repository Server,";
        msg += ex.what();
        msg += ',';
        msg += ex.where();
        msg += ',';
        msg += ex.status();

		if(logr) logr->AddTrace(STARTUP, DEBUGLEVEL, msg.c_str()); 

		throw;
    }
    catch(const exception &ex)
    {
		string msg = "Error Creating Repository Server,";
        msg += ex.what();
		if(logr) logr->AddTrace(STARTUP,DEBUGLEVEL,msg.c_str());
		throw;
    }
    if(logr) logr->AddTrace(STARTUP,DEBUGLEVEL,"Repository Server Created");

	try
	{
		CORBA::Object_ptr obj;
		obj=m_corba->FindObjectWithName(TimeService::CLASS_NAME,underscore_ip_name.c_str());
		if(CORBA::is_nil(obj))
		    throw CyrRepositoryException("TimeService Not available","CORBARepository::CORBARepository",-1);
		TimeService_var TmS=TimeService::_narrow(obj);
		CTTime tm;
		unsigned long repo_time=TmS->GetTime();
		tm.now();
		m_timeoffset=tm.getSeconds()-repo_time;//no of seconds difference (approx) between machines add to all statistic times
		msg="Time offset on machine "+info.GetIPAddressAsString()+" set to "+int_to_string(m_timeoffset);
		logr->AddTrace(STARTUP,ERRORLEVEL,msg.c_str()); 
	}
	catch(...)
	{
		m_timeoffset = 0;
		if(logr) logr->AddTrace(STARTUP,DEBUGLEVEL,"Unable to connect to TimeService.");
	}
#endif
}

//@mfunc	Destructor
CORBARepository::~CORBARepository()
{
#ifndef CYRANO_TP_STANDALONE
    if(logr) logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Start Clear Down of Repository Objects"); 
    if(logr) logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Remove Object Code Folder"); 
    if(m_ObjectCodeFolder)
    {
		delete m_ObjectCodeFolder;
		m_ObjectCodeFolder = NULL;
    }

    if(m_NodeFolder)
    {
		delete m_NodeFolder;
		m_NodeFolder = NULL;
    }

    if(m_DataFolder)
    {
		delete m_DataFolder;
		m_DataFolder = NULL;
    }
    if(logr) logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Remove TestRun"); 
    if(m_TestRun)
    {
		// Before we remove TestRun store any local Test Results
		// by creating a TestResult file object.
		// Results are transferred back to the Server machine and 
		// stored into the repository.

		if(m_LocalResults)
		{
 		if(logr) logr->AddTrace(SHUTDOWN,TRACELEVEL,"Storing HTTP Test Results"); 
			try
			{
				// We need to pass in the name of the local results file with and without path

				CECResultsFile* RFile = new CECResultsFile(m_TestRun,
									m_LocalResultsFileName.c_str(),
									m_LocalResultsFullFileName.c_str());
#ifndef TH_POOL_VER
                RFile->Concatenate();
#endif
				RFile->Store(m_DeleteLocalResults);
				delete RFile;

				// Store URLs file
				string LocalURLFileName = m_LocalResultsFileName.substr(0, m_LocalResultsFileName.find_last_of(".")) + ".urls";
				string LocalURLFullFileName = m_LocalResultsFullFileName.substr(0, m_LocalResultsFullFileName.find_last_of(".")) + ".urls";
				CECResultsFile* RUrlFile = new CECResultsFile(m_TestRun,
									LocalURLFileName.c_str(),
									LocalURLFullFileName.c_str());
				RUrlFile->Store(m_DeleteLocalResults);
				delete RUrlFile;
			}
			catch(CyrRepositoryException &ex)
			{
				string msg = "Error storing results file," + ex.where();
				if(logr) logr->AddTrace(STARTUP, DEBUGLEVEL, msg.c_str());
			}

		}

	if(logr) logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Store Tracing"); 
	if(logr) logr->Store(m_TestRun);
	delete m_TestRun;
	m_TestRun = NULL;
    }
    for(TestRunList_t::iterator i=m_ParentTestRunList.begin();i!=m_ParentTestRunList.end();i++)
    {
		delete (*i);
    }
    if(m_TestFolder)
    {
	delete m_TestFolder;
	m_TestFolder = NULL;
    }
    if(m_TestStore)
    {
	delete m_TestStore;
	m_TestStore = NULL;
    }
    if(m_RootFolder)
    {
	delete m_RootFolder;
	m_RootFolder = NULL;
    }
    if(m_Project)
    {
	delete m_Project;
	m_Project = NULL;
    }
    if(m_Server)
    {
	delete m_Server;
	m_Server = NULL;
    }

#endif
}

#ifdef TH_POOL_VER
BOOL CORBARepository::InitClass()
{
    assert(s_pPublicMethodFl == NULL);
    if(s_pPublicMethodFl == NULL)
        s_pPublicMethodFl = new CFastLock();

    if(s_pPublicMethodFl == NULL)
        return FALSE;
    else
        return TRUE;
}

void CORBARepository::UninitClass()
{
    if(s_pPublicMethodFl != NULL)
    {
        delete s_pPublicMethodFl;
        s_pPublicMethodFl = NULL;
    }
}
#endif

//@mfunc    Create the Global Repository Objects required down to TestRun
//	    Should only be called in 'main' and should therefore be MT-safe(more correctly only one thread will call it)
//@parm	    Name of Project to retrieve Tests From
//@parm	    TestName within this project
//@parm	    A pipe separated list of parents for this test run, only required if this is to be a sub test run
int CORBARepository::Connect(const string& ProjectName ,const string& TestName,const string& TestRunString)
{
#ifndef CYRANO_TP_STANDALONE
    // Get a handle to the specified project from the repository. First allocate the object
    if(!m_Server) if(logr) logr->AddTrace(REPOSITORY,ERRORLEVEL,"No Server Object");

    m_Project = new CECOrbProject(m_Server,ProjectName.c_str());

    m_RootFolder = new CECOrbFolder(m_Project);

    //If first TestManager refresh repository
    //if(TestRunString=="")m_RootFolder->Refresh();
    // Get the Folder holding all the tests for the project
    // Always check that the pointer is valid
    m_TestFolder = new CECTestFolder(m_RootFolder);

    // Get the Folder holding all the tof files for the project
    // Always check that the pointer is valid
    m_ObjectCodeFolder = new CECObjectCodeFolder(m_RootFolder);

    m_NodeFolder = new CECNodeFolder(m_RootFolder);

    m_DataFolder = new CECDataFolder(m_RootFolder);

    // Get Test Run Object
    // We have been passed a test object - Allocate a test run for it.
    MakeActualTestRun(TestRunString,TestName);
    //Connect Global Log ObjectLog after retreiving 
	//details from repository (after creation of TestRun) 
    if(logr) logr->AddTrace(REPOSITORY,ERRORLEVEL,"Connect Tracing Object to repository");
    if(logr) (*logr)->Connect(m_Project);
    if(logr) logr->AddTrace(REPOSITORY,ERRORLEVEL,"Tracing Object connected to repository");
#endif//CYRANO_TP_STANDALONE
    return 0;
}
string CORBARepository::GetFile(const string& ShortFileName,const string& LongFilename,CECOrbFolder* Folder)
{
#ifndef CYRANO_TP_STANDALONE
	CECOrbFile*	FileEx;
	// Get the actual TOF file detailing the test run from the object folder
	// and copy this to local disk space for later access.
	FileEx = new CECOrbFile(Folder, ShortFileName.c_str());
	if(logr) logr->AddTrace(DBGSTD,DEBUGLEVEL,string("Got file :"+ShortFileName).c_str());
	// Validate the file exchange pointer
	if (FileEx == NULL)
	{
		return "";
	}

	// Copy the file requested to the local disk space.
	FileEx->CheckOut(LongFilename.c_str(),false);
	delete FileEx;
	if(logr) logr->AddTrace(DBGSTD,DEBUGLEVEL,string("Checked Out file :"+ShortFileName).c_str());
#endif
	return LongFilename;
}
/////////////////////////////////////////////////////////////////////////
//@mfunc    Get Test Tof File from repository ObjectCode folder for specified test
//	    and copy to local directory
//	    Must be MT-safe
//@parm	    Tof File Base Name (no extension)/TestName
//@rdesc    Full Path name of copied file
string  CORBARepository::GetTofFile(const string& inFilename)
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
	if(logr) logr->AddTrace(DBGSTD,DEBUGLEVEL,string("Get file :"+inFilename).c_str());
	string Filename = inFilename+GetTofFileExtension();
        string FilenamePath = exedir + FN_PATHSEP + Filename;
	return GetFile(Filename,FilenamePath,m_ObjectCodeFolder);
}
/////////////////////////////////////////////////////////////////////////
//@mfunc    Get Thread Tof File from repository ObjectCode folder for specified test
//	    and copy to local directory
//	    Must be MT-safe
//@parm	    Tof File Base Name (no extension)
//@rdesc    Full Path name of copied file
string  CORBARepository::GetThreadFile(const string& inFilename)
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
	if(logr) logr->AddTrace(DBGSTD,DEBUGLEVEL,string("Get Thread:"+inFilename).c_str());
	string Filename = inFilename+".tho";
        string FilenamePath = exedir + FN_PATHSEP + Filename;
	return GetFile(Filename,FilenamePath,m_ObjectCodeFolder);
}
string  CORBARepository::GetVariableFile(const string& inFilename)
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
	string Filename = inFilename+".fvr";

#ifdef __DLLSA__
    string FilenamePath = filevardir + FN_PATHSEP + Filename;
#else
    string FilenamePath = exedir + FN_PATHSEP + Filename;
#endif
	return GetFile(Filename,FilenamePath,m_DataFolder);
}

/////////////////////////////////////////////////////////////////////////
//@mfunc    Get Node  File from repository Node folder for specified Node
//	    and copy to local directory
//	    Must be MT-safe
//@parm	    Logical Node Name
//@rdesc    Full Path name of copied file
#ifndef CYRANO_TP_STANDALONE
CECNodeDefinition CORBARepository::GetNodeDefinition(const string& LogicalNodeName)
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
	if(logr) logr->AddTrace(DBGSTD,DEBUGLEVEL,string("Get Node:"+LogicalNodeName).c_str());
	if(LogicalNodeName==""||LogicalNodeName=="<local>"||LogicalNodeName=="localhost")
	{
	    CECNodeDefinition NodeDef("localhost");
	    return NodeDef;
	}
	else
	{
	    string FilenamePath = exedir + FN_PATHSEP + LogicalNodeName;
	    GetFile(LogicalNodeName,FilenamePath,m_NodeFolder);
	    char* nodename=new char[256];
	    ifstream nodefile(FilenamePath.c_str());
	    nodefile.getline(nodename ,256);
#ifdef UNIX
	    nodename[strlen(nodename)-1]='\0';
#endif
	    string node=nodename;
	    CECNodeDefinition NodeDef(node);
	    delete[] nodename;
	    return NodeDef;
	}
}
#endif
/////////////////////////////////////////////////////////////////////////
//@mfunc    Add a Timer Output to this TestRun
//@parm	    String Details required!
int CORBARepository::TimerOutput(const string& TimerName,long ElapsedTime,const string& ScriptName,const string& ThreadID,const string& TestName,int no_threads )
{
#ifndef CYRANO_TP_STANDALONE
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    //Make String Test|Thread|Script|Timer
    string TimerString;
    TimerString = TestName+"|"+ThreadID+"|"+ScriptName+"|"+TimerName;


    CECTimerOutput *pTimer = new CECTimerOutput(m_timeoffset, TimerString.c_str(), ElapsedTime, no_threads );
	
    // check creation of timer, if NULL then return error
    if (pTimer == NULL)
	return -1;
    m_TestRun->AddTestOutput( pTimer);
    delete pTimer;
#endif
    return 0;
}
//@mfunc    Add A Log Message
//@parm	    message to log
//@parm	    destination type e.g. AUDIT, REPORT etc.
int CORBARepository::LogMessage	(const string& Msg, int LogFile,const string& scriptname,const string& ThreadID,int objindex)
{

#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    string strmsg;
    strmsg.reserve(255);

    char buff[20];

    strmsg += m_exename;
    strmsg += '|';
    strmsg += m_szPid;
    strmsg += '|';
    strmsg += ThreadID;
    strmsg += '|';
    strmsg += scriptname;
    strmsg += '|';
    itoa(objindex, buff, 10);
    strmsg += buff;
    strmsg += '|';
    strmsg += Msg;

    if(logr) logr->AddTrace(AUDIT, AUDITLEVEL, strmsg.c_str());

#ifndef CYRANO_TP_STANDALONE
    try
    {
        switch (LogFile)
        {
	    case 1:
            m_TestRun->AddTestOutput(&CECTestHistory(m_timeoffset, strmsg.c_str()));
            break;
	    case 2:
            m_TestRun->AddTestOutput(&CECTestReport(m_timeoffset, strmsg.c_str()));
            break;
	    case 0:
            m_TestRun->AddTestOutput(&CECTestLog(m_timeoffset, strmsg.c_str()));
            break;
	    default:
            m_TestRun->AddTestOutput(&CECTestLog(m_timeoffset, strmsg.c_str()));
            break;
        }
    }
    catch(...)
    {
	    if(logr) logr->AddTrace(AUDITFAIL, AUDITLEVEL, strmsg.c_str());
    }
#endif

    return 0;
}
#ifndef CYRANO_TP_STANDALONE
string  CORBARepository::GetProjectName()
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    string ProjectName=m_Project->GetName();
    return ProjectName;
}
string  CORBARepository::GetUserName()
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    return m_UserName;
}
string  CORBARepository::GetPassword()
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    return m_Password;
}

string  CORBARepository::GetDataServerNodeName()
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    return m_DataServerNodeName;
}

CECOrbTestRun* CORBARepository::GetParentTestRunFromString(string TestRunString)
{
    //initialise positions in string before going through it
    int startpos=0;
    //Find ServerName if problems then throw exception
    int pipe_pos=startpos;
    startpos=TestRunString.find_first_of(TESTRUNSTRING_SEP,pipe_pos);
    string ServerName=TestRunString.substr(pipe_pos,startpos-pipe_pos);
    if(ServerName!=SERVERNAME_LITERAL)
	throw mdbParseException("Sub TestRuns Could not be created");//this has all gone horribly wrong shouldn't really go any further
    //Find ProjectName (should be same as one we're using)if problems then throw exception
    pipe_pos=++startpos;
    startpos=TestRunString.find_first_of(TESTRUNSTRING_SEP,pipe_pos);
    string ProjectName=TestRunString.substr(pipe_pos,startpos-pipe_pos);
    if(ProjectName!=m_Project->GetName())
	throw mdbParseException("Sub TestRuns Could not be created, problem with project name");
    //Find Test folder Name (should be Tests)if problems then throw exception
    pipe_pos=++startpos;
    startpos=TestRunString.find_first_of(TESTRUNSTRING_SEP,pipe_pos);
    string TestFolderName=TestRunString.substr(pipe_pos,startpos-pipe_pos);
    if(TestFolderName!=m_TestFolder->GetName())
	throw mdbParseException("Sub TestRuns Could not be created, problem with TestFolder name");
    //Done a few checks on previous parts of string now parse the rest, the next bit is the parent Test
    //after that they are all parent testruns
    pipe_pos=++startpos;
    startpos=TestRunString.find_first_of(TESTRUNSTRING_SEP,pipe_pos);
    string ParentTestName=TestRunString.substr(pipe_pos,startpos-pipe_pos);
    //Got Parent Test name so create parent teststore from this
    m_TestStore = new CECTestStore(m_TestFolder,ParentTestName.c_str());

    //Make a list of all test run names to be used differently by TestManagers/Executers
    //This list will be used by a virtually inherited method to connect/create the TestProceses
    //TestRun that will be used for the storage of data
    vector<string> TestRunNameList;
    //Find the rest of the test runs
    while(startpos<TestRunString.length())
    {
        pipe_pos=++startpos;
        startpos=TestRunString.find_first_of(TESTRUNSTRING_SEP,pipe_pos);
        string TestRunName=TestRunString.substr(pipe_pos,startpos-pipe_pos);
	TestRunNameList.push_back(TestRunName);
    }
    return CreateParentTestRuns(TestRunNameList,TestRunString);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc Return string identifying the test run within the test hierarchy
// @rdesc TestRunString
// just as an aside so that it is commented somewhere, I have used the term TestRunString quite a lot
// this is a string of pipe separated names from which a full repository hierarchy can be derived for a test run
// e.g. Server|Project|TestFolder|TestName|TestRun1|TestRun2|TestRun3.....
// This rather conveniently means that each TestManager has a CORBARepository that has direct access
// to all the parent CORBARepository Objects which should come in useful for commands such as LogToCentral.
// this should also be useful for the naming of 'Global'(both very global and quite global) variables
string  CORBARepository::GetTestRunString()
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    string test_run_string=SERVERNAME_LITERAL;
    test_run_string+= TESTRUNSTRING_SEP + m_Project->GetName() + TESTRUNSTRING_SEP + m_TestFolder->GetName();
    if(m_ParentTestRunList.empty())
    {
	test_run_string += TESTRUNSTRING_SEP + m_TestStore->GetName() + TESTRUNSTRING_SEP + m_TestRun->GetName();
    }
    else
    {
	test_run_string += TESTRUNSTRING_SEP + m_TestStore->GetName();
	TestRunList_t::iterator i;
	for(i=m_ParentTestRunList.begin();i!=m_ParentTestRunList.end();i++)
	{
	    test_run_string += TESTRUNSTRING_SEP + (*i)->GetName();
	}
	test_run_string += AppendOwnTestRunString();//Add own bit if you have one, bit contrived I know!
    }
    return test_run_string;
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Return string identifying the Primary (i.e. first) test run within the test hierarchy
//	    Could be changed to call GetTestRunString and break after first TestRun
// @rdesc   TestRunString
string  CORBARepository::GetPrimaryTestRunString()
{
#ifndef TH_POOL_VER
	CTLockGuard<CTMutex> guard(public_method_mutex);
#else
	CLock guard(s_pPublicMethodFl);
#endif
    string test_run_string=SERVERNAME_LITERAL;
    test_run_string+= TESTRUNSTRING_SEP + m_Project->GetName() + TESTRUNSTRING_SEP + m_TestFolder->GetName() ;
    if(m_ParentTestRunList.empty())
    {
	test_run_string += TESTRUNSTRING_SEP + m_TestStore->GetName() + TESTRUNSTRING_SEP + m_TestRun->GetName();
    }
    else
    {
	test_run_string += TESTRUNSTRING_SEP + m_TestStore->GetName();
	TestRunList_t::iterator i;
	i=m_ParentTestRunList.begin();
	test_run_string += TESTRUNSTRING_SEP + (*i)->GetName();
    }
    return test_run_string;
}
#endif //#ifndef CYRANO_TP_STANDALONE


///////////////////////////////////////////////////////////////////////////////
// @mfunc  Tells the CORBARepository that there is a local results file
//	    
// @parm   Fully qualified Filename for results file
// @parm   Filename for results file
void CORBARepository::SetLocalResultsFile(const char* FullFileName,const char* Filename, bool del )
{
	// remember the filename
	m_LocalResultsFileName = Filename;
	m_LocalResultsFullFileName = FullFileName;

	// set the flag to tell the Repository to deal with this on shut down
	m_LocalResults = true;
	m_DeleteLocalResults = del;
}
