/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     OtaExecuterImpl
// @module  OtaExecuterImpl |
//
//
///////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#ifdef UNIX
using namespace std;
#endif

#include <TECORBARepository.h>
#include <stdio.h>

#include "OtaDefs.h"
#include <psl_macros.h>
#include <target_rtndef.h>
#include "OtaExecuter_i.h"
#include <TPLog.h>
#include <cxxutils.h>
#include <CECInetInfo.h>
#include "OtaQuery.h"
#include "OtaExec.h"

#include "OtaSnmp.h"

#ifdef WIN32
#include "OtaNTPerf.h"
#endif

#ifdef UNIX
#include <time.h>
#endif

///////////////////////////////////////////////////////////////////////////////
#include <CECSimpleParse.h>
#include <RegExp.h>

extern string exedir;
extern TScriptRule *rule;

CORBAServer *pCServer = NULL;
void* Stop(void);
extern CTSemaphore blocking_semaphore;
void OtaExecuterImpl_ExecutionThread(OtaExecuterImpl* OTA)
{
    OTA->ExecutionThread();
}
void OtaExecuterImpl_ExecuteQuery(OtaExecuterImpl* OTA,int index)
{
    OTA->ExecuteQuery(index);
}
/////////////////////////////////////////////////////////////////////////////
// OtaExecuterImpl  Constructor
///////////////////////////////////////////////////////////////////////////////
// @mfunc Constructor
OtaExecuterImpl::OtaExecuterImpl(CORBAServer * pserver,
                                 string & regname,
                                 string node,
                                 int pid,
                                 string exename):m_pServer(pserver),
        m_RegisterName(regname),
        m_nodename(node),
        m_pid(pid), m_exename(exename), m_ParentTestLevel(TestLevel::_nil())
{
    pCServer = pserver;
    m_pServer->ActivateObject(this);    
    m_pServer->NameObject(this->_this(),TestExecuter::CLASS_NAME, m_RegisterName.c_str());    

    // Global CORBA Respository class
    m_threadstarted=false;
    Repository = NULL;      //Make sure is null if not used
}

/////////////////////////////////////////////////////////////////////////////
// OtaExecuterImpl  Destructor
///////////////////////////////////////////////////////////////////////////////
// @mfunc Destructor

OtaExecuterImpl::~OtaExecuterImpl()
{
    // Delete the repository first as this takes some time - depending
    // on the size of the logging files etc.
    if (Repository)
    {
        delete    Repository;
    }

    //Finished shutting down , inform TestLevel that we are complete
    if (!CORBA::is_nil(m_ParentTestLevel))
    {
        m_ParentTestLevel->TestExecuterFinished();
    }

    // delete the CORBAinterface pointer
    // dispose of the m_OtaExecuterStatus Obj
    try
    {
        m_pServer->UnNameObject(TestExecuter::CLASS_NAME, m_RegisterName.c_str());  

    }
    catch(...)
    {
        //if we can't de-register probably means either everything
        //has gone anyway or we are already de-registered
    }

}


CORBA::Long OtaExecuterImpl::Initialise(const char *node, const char *user,
                                        const char *password,
                                        const char *TestRunString)
{
    m_TestRunString = TestRunString;    //store for use later

    //Create New repository
    try
    {
        Repository =
            new TECORBARepository(m_pServer, node, user, password,
                                  m_nodename.c_str(), m_pid, m_exename);
    }
    catch(CyrRepositoryException & e)
    {
        OtaAddTrace(CYR_OTA, TRACE_PARTIALDIAG,
                    "Failed to create repository");
        Stop();
        throw     CyrProcessException(e.what().c_str(), e.where().c_str(), e.status());
    }
    catch(exception & e)
    {
        OtaAddTrace(CYR_OTA, TRACE_PARTIALDIAG,
                    "Failed to create repository");
        Stop();
        throw     CyrProcessException(e.what(), "OtaExecuterImpl::Initialise",

                                      -1);
    }
    catch(...)
    {
        OtaAddTrace(CYR_OTA, TRACE_PARTIALDIAG,
                    "Failed to create repository");
        Stop();
        throw     CyrProcessException("UNKNOWN",
                                      "OtaExecuterImpl::Initialise", -1);
    }

    return 0;
}

/////////////////////////////////////////////////////////////////////////////
// ExecuteThread
///////////////////////////////////////////////////////////////////////////////
// @mfunc
// @param
// @param
// @param
// @rdesc long
//
long      OtaExecuterImpl::ExecuteThread(const char *ProjectName,
        const char *TestName,
        TestLevel_ptr & Parent,
        const ThreadDef & thrdef)
{
    static bool bThreadAttached = false;
    m_threadstarted=false;
    //
    //   Msg to identify the start of the execute thread routine.
    //
    string    msg =
        string("Execute Thread, ProjectName: ") + ProjectName +
        " TestName: " + TestName;

    OtaAddTrace(STARTUP, DEBUGLEVEL, msg);

    //
    //    If we have already been through this code then throw a CyrNoMoreThreads exception as OTA
    //    only supports a single thread
    //
    if (bThreadAttached)
    {
        OtaAddTrace(CYR_OTA, TRACE_PARTIALDIAG, "ExecuteThread called twice");
        Stop();

        throw     CyrNoFreeThreadException();
    }

    //Set up Parent TestLevel (should only be done once)
    if (CORBA::is_nil(m_ParentTestLevel))
    {
        if (!CORBA::is_nil(Parent))
            m_ParentTestLevel = TestLevel::_duplicate(Parent);
    }

    bThreadAttached = true;
    //

    //
    //    Connect to the test run in the repository
    //
    OtaAddTrace(STARTUP, DEBUGLEVEL, "Connect to TestRun");
    try
    {
        long      status =
            Repository->Connect(ProjectName, TestName, m_TestRunString);

        if (status == -1)
        {
            OtaAddTrace(CYR_OTA, TRACE_PARTIALDIAG,
                        "Failed to connect to repository");
            Stop();
            // Get Test has failed, set error message and throw exception
            CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
            CORBA::String_var what = "GetTest Failed";
            throw     CyrProcessException(what, where, -1);
        }
    }
    catch(CyrRepositoryException & e)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to connect to repository");
        Stop();
        CORBA::String_var what = e.what().data();
        CORBA::String_var where = e.where().data();
        throw     CyrProcessException(what, where, e.status());
    }
    catch(exception & e)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to connect to repository");
        Stop();
        CORBA::String_var what = e.what();
        CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
        throw     CyrProcessException(what, where, -1);
    }
    catch(CORBA::Exception&)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to connect to repository");
        Stop();
        CORBA::String_var what = "CORBA Exception";
        CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
        throw     CyrProcessException(what, where, -1);
    }
    catch(...)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to connect to repository");
        Stop();
        CORBA::String_var what = "UNKNOWN";
        CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
        throw     CyrProcessException(what, where, -1);
    }

    try
    {
        m_testRun = Repository->GetTestRun();
    }
    catch(...)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to GetTestRun");
        Stop();
        CORBA::String_var what = "UNKNOWN";
        CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
        throw     CyrProcessException(what, where, -1);
    }

    //
    //    Access the repository and retrieve the TOF test description file by copying it to local disk space.
    //    The full path / location of the TOF file is returned.
    //
    string    FullTestName;

    try
    {
        OtaAddTrace(DBGSTD, DEBUGLEVEL, "Get the File: " + string(TestName));
        FullTestName = Repository->GetTofFile(TestName);
        OtaAddTrace(CYR_OTA, TRACE_PARTIALDIAG,
                    "Using File: " + string(FullTestName));
    }
    catch(...)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to get TOF file");
        Stop();
        CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
        CORBA::String_var what = "GetTestFile Failed";
        throw     CyrProcessException(what, where, -1);
    }

    if (FullTestName.size() == 0)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Failed to get TOF file");
        Stop();
        CORBA::String_var where = "OtaExecuterImpl::ExecuteThread";
        CORBA::String_var what = "GetTestFile Failed";
        throw     CyrProcessException(what, where, -1);
    }

    // Call function to parse TOF file
    ParseTof(FullTestName.c_str());

    // Log the configuration
    LogConfig();

    // Start execution thread
    m_status = ACTIVE;
    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Starting execution thread");

    m_executor = new CTThread(makeFunctorA1(OtaExecuterImpl_ExecutionThread,OtaExecuterImpl*, this));
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is created as a separate thread.  It is responsible for
//          the constant execution of queries.  It can only be stopped by the
//          stop method.
void      OtaExecuterImpl::ExecutionThread()
{
    m_threadstarted=true;
    OtaAddTrace(DBGSTD, ERRORLEVEL,
                "OtaExecuterImpl::ExecutionThread entered");

    // Get access to this thread function
    int       index = 0;

    m_threads = 0;



    // Set environment variables
    SetEnvironment();

    // Whilst the application is active we loop through all the queries.
    // Each query is executed in its own thread and the results stored
    // in the correct array.
    while (m_status == ACTIVE)
    {
        {
            char buff[ 128 ];
            sprintf( buff, "m_status == ACTIVE loop, m_threads %d", m_threads );
            OtaAddTrace(DBGSTD, ERRORLEVEL, buff );
        }

        while (index < m_query.size())
        {

            OtaQuery *pQuery = m_query[index];

            if (!pQuery->IsActive())
            {
                // Get current time before query and check that query needs to be done
                CTTime now;
                now.now();
                long      start = now.getSeconds();
                long      interval =
                    start - pQuery->GetLastTimestamp().getSeconds();

                if (interval >= pQuery->GetInterval())
                {
                    // Execute query
                    CTThread thr (makeFunctorA2 (OtaExecuterImpl_ExecuteQuery, OtaExecuterImpl*, this,int, index));
                }
            }
            // Move to next one
            index++;

            // delay so all performance monitors don't run at once
            // use as little code as possible to avoid Heisenburg effects
#if defined WIN32
            //OtaAddTrace(DBGSTD, ERRORLEVEL, "about to sleep for 200ms" );
            Sleep(200); // Milliseconds
            //OtaAddTrace(DBGSTD, ERRORLEVEL, "    sleep done" );
#elif defined UNIX
// TBD check whether this is process or thread specific.
            // Posix spec says thread safe.
            // Solaris says thread safe
            // Linux and HP say process.
            // empirical test on Linux (2.2.15/gcc 2.91.66) looks OK
            // HP-UX 11 and SunOs 5.6 work if -lpthread specified
            {
                struct timespec ts;

                ts.tv_sec = 0;
                ts.tv_nsec = 200 * 1000;    // nanoseconds;
                nanosleep(&ts, NULL);
            }
#else
#error platform unknown
#endif
        }

        // Sleep for a second and check if thread has been cancelled
        CTCurrentThread::sleep(ONE_SECOND);
        OtaAddTrace(DBGSTD, ERRORLEVEL, "about to serviceCancellation");
        // Reset counter if necessary
        index++;
        if (index > m_query.size())
            index = 0;
    }
    //When No longer Active and thread stopping
    while (m_threads > 0)
    {
        {
            char      buff[128];

            sprintf(buff, "RWCancellation caught - m_threads %d",
                    m_threads);
            OtaAddTrace(DBGSTD, ERRORLEVEL, buff);
        }

        CTCurrentThread::sleep(ONE_SECOND);
    }

    OtaAddTrace(DBGSTD, ERRORLEVEL,
                "end of while( m_status == ACTIVE ) loop");
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is used as a separate thread that executes a single
//          query.  All relevant information can be found in the query itself.
// @parm    Index of query to execute.
void      OtaExecuterImpl::ExecuteQuery(int index)
{
    string    msg;

    // Acquire mutex on thread count and increment it
    m_mutex.lock();
    m_threads++;
    m_mutex.unlock();

    const char * const catchmsg = "Caught Exception in OtaExecuterImpl::ExecuteQuery: ";
    bool caught = false;
    try
    {
        // Get correct query out of array
        OtaQuery *pQuery = m_query[index];

        if ((pQuery != NULL) && (!pQuery->IsDisabled()))
        {
            pQuery->SetActive(true);

            OtaQueryHandler *pHandler = pQuery->GetHandler();

            // Run query
            CTTime now;
            now.now();
            long      start = now.getSeconds();
            long      simRes = 0;
            string    detRes = "";
            bool      bStatus;
            string    msg;

            // If SQL query, acquire mutex to prevent more than one query per
            // target at a time
            OtaTarget *pTarget = pQuery->GetTarget();

            if (pTarget != NULL)
            {
                pTarget->m_mutex.lock();
            }

            if (pQuery->IsDetailQuery())
            {
                msg = string("Executing detail query \"") +
                      pQuery->GetDescription() + "\"";

                OtaAddTrace(DBGSTD, DEBUGLEVEL, msg);
                bStatus = pHandler->RunDetailedQuery(pQuery->GetQuery(), &detRes);
            }
            else
            {
                msg = string("Executing simple query \"") +
                      pQuery->GetDescription() + "\"";

                OtaAddTrace(DBGSTD, DEBUGLEVEL, msg);
                bStatus = pHandler->RunSimpleQuery(pQuery->GetQuery(), &simRes);
            }

            // If SQL query, release mutex
            if (pTarget != NULL)
            {
                pTarget->m_mutex.unlock();
            }

            // If query failed then disable it
            if (!bStatus)
            {
                string    msg =
                    string("Disabling query \"") + pQuery->GetDescription() + "\"";

                OtaAddTrace(CYR_OTA, ERRORLEVEL, msg);
                pQuery->Disable();
            }
            else
            {
                // Get current time after query
                CTTime now;
                now.now();
                long      stop = now.getSeconds();

                // Add new test result
                if (pQuery->IsDetailQuery())
                {
                    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Adding detail test result");
                    pQuery->AddResult(start, stop, detRes.c_str());

                    // Log result
                    msg = "Value for " + pQuery->GetDescription() + " = " + detRes;
                    OtaAddTrace(CYR_OTA, TRACE_RESULTS, msg);
                }
                else
                {
                    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Adding simple test result");
                    pQuery->AddResult(m_testRun, start, stop, simRes);

                    // Log result
                    char      cval[32];

                    sprintf(cval, "%d", simRes);
                    msg = "Value for " + pQuery->GetDescription() + " = " + cval;
                    OtaAddTrace(CYR_OTA, TRACE_RESULTS, msg);
                }

                pQuery->SetActive(false);
            }
        }
    }
#if 1
    OTACATCHALL
#else
    catch(...)
    {
        OtaAddTrace(CYR_OTA, TRACE_RESULTS,
                    "Caught exception in OtaExecuterImpl::ExecuteQuery" );
    }
#endif

    // Acquire mutex on query and decrement thread count
    m_mutex.lock();
    m_threads--;
    m_mutex.unlock();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Stop this OTA Test Executer - tidily.
//
void OtaExecuterImpl::Stop()
{
    static bool stopping = false;
    size_t    i;

    if (stopping) return;

    stopping = true;
    OtaAddTrace(DBGSTD, ERRORLEVEL, "Stop invoked for OTA Test Executer");

    // Assume test is successful for now.
    m_status = SUCCESS;

    // Cancel execution thread and wait for it to terminate.
    if(m_threadstarted)
    {
        OtaAddTrace(DBGSTD, ERRORLEVEL, "about to join");
        CTCurrentThread::join(*m_executor);
        {
            char      buff[128];
            sprintf(buff, "joined - m_query.size() %d", m_query.size());
            OtaAddTrace(DBGSTD, ERRORLEVEL, buff);
        }
    }

    // Store test results and delete queries
    for (i = 0; i < m_query.size(); i++)
    {
        // Get next entry
        {
            char      buff[128];
            sprintf(buff, "storing query %d", i);
            OtaAddTrace(DBGSTD, ERRORLEVEL, buff);
        }

        OtaQuery *pQuery = m_query[i];

        // Store detail query results in Test Run - note that we stored simple
        // results as we went along
        if (pQuery->IsDetailQuery())
        {
            pQuery->StoreResults(m_testRun);
        }

        // Delete query object.
        OtaAddTrace(DBGSTD, ERRORLEVEL, "Deleting query object");
        delete pQuery;
    }

    OtaAddTrace(DBGSTD, ERRORLEVEL,
                "done loop storing results - about to delete");

    // Delete target objects
    for (i = 0; i < m_targets.size(); i++)
    {
        {
            char      buff[128];
            sprintf(buff, "deleting query %d", i);
            OtaAddTrace(DBGSTD, ERRORLEVEL, buff);
        }

        // Get next target
        OtaAddTrace(DBGSTD, ERRORLEVEL, "Deleting target object");
        OtaTarget *pTarget = m_targets[i];
    }

    // Allow main thread to complete
    blocking_semaphore.unlock();

    OtaAddTrace(DBGSTD, DEBUGLEVEL, "about to return from OtaExecuterImpl::Stop()");
    return;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Kill this OTA Test Executer.
//
//          Note that no return is made from this routine - an explit exit is
//          executed instead.
//
//          The Stop() method should be used in order to perform an orderly
//          shutdown. This method should be used for emergency shutdowns only.
//
void OtaExecuterImpl::Kill()
{
    OtaAddTrace(SHUTDOWN, ERRORLEVEL, "Kill invoked for OTA Test Executer");
    OtaAddTrace(SHUTDOWN, ERRORLEVEL, "Kill for OTA Test Executer complete - about to exit process");
    exit(0);
}

/////////////////////////////////////////////////////////////////////////////
// ParseTof
///////////////////////////////////////////////////////////////////////////////
// @mfunc Parse TOF file and set up data for queries.
// @param
// @param
void      OtaExecuterImpl::ParseTof(const char *TofFile)
{
    // Error information
    const char *where = "OtaExecuterImpl::ParseTof";
    const int status = CYR_SCRIPT_INVALID;

    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Parsing TOF file");

    try
    {
        const AtvfKey typeOption("Options");
        const AtvfKey typeEnv("Environment");
        const AtvfKey typeTarget("Target");
        const AtvfKey typeSimpleQuery("SimpleQuery");
        const AtvfKey typeDetailQuery("DetailQuery");
        size_t    index = -1;

        // Open file and start parsing TOF
        AtvfInput in(TofFile);
        TScript   script(rule, in);

        // Loop through all records
        for (size_t i = 0; i < script.length(); i++)
        {
            TRecord  *record = script[i];

            // Read any general options
            if (record->isType(typeOption))
                ReadOptions(record);

            // Get environment information
            if (record->isType(typeEnv))
                ReadEnvironment(record);


            // Get simple query
            if (record->isType(typeSimpleQuery))
            {
                ReadQuery(record, false);
            }

            // Get detail query (unless detail queries disabled)
            if ((record->isType(typeDetailQuery)) && (m_details))
            {
                ReadQuery(record, true);
            }
        }
    }
    catch(mdbRuntimeError ex)
    {
        OtaAddTrace(CYR_OTA, ERRORLEVEL, "Error reading TOF file");
        Stop();
        throw     CyrProcessException(ex.what().c_str(), where, status);
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method extracts any general options information from the TOF
//          script and adds them to the list of test executor creation
//          parameters.
// @parm    Pointer to the TOF record.
void      OtaExecuterImpl::ReadOptions(TRecord * record)
{
    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Reading options");
    // Extract tracing level
    const TParam *tparm = record->param("tracing");

    if (tparm != NULL)
    {
        m_tracing = tparm->intVal();
    }

    // Extract detail level setting
    tparm = record->param("dodetails");
    if (tparm != NULL)
    {
        m_details = (tparm->strVal() != "0");
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Given a TOF record, this method will extract any information that
//          relates to the execution environment of the query.  The information
//          is placed in the parameter list that will be used to create the
//          query object.
// @parm    Pointer to the TOF record.
void      OtaExecuterImpl::ReadEnvironment(TRecord * record)
{
    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Reading environment");
    const TParam *tparm = record->param("env");

    if (tparm != NULL)
    {
        string    environment = tparm->strVal().c_str();

        m_environments.push_back(environment);
    }
}


///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method reads a query record from the TOF file.  This will
//          extract any query information and build a query object from it.  A
//          query object contains all the information required to execute an
//          OTA query.
// @parm    Pointer to the query record.
void      OtaExecuterImpl::ReadQuery(TRecord * record, bool detail)
{
    OtaAddTrace(DBGSTD, DEBUGLEVEL, "Reading query");
    const char *where = "OtaExecuterImpl::ReadQuery";
    const TParam *tparm;
    int       querytype = 0;
    int       targettype = 0;
    bool      delta = false;
    string    desc = "";
    int       interval = 5;
    string    query = "";
    string    target = "";
    OtaTarget *pTarget = NULL;
    OtaQueryHandler *pHandler;

    // Check if query is enabled - if not, just return
    tparm = record->param("enabled");
    if (tparm != NULL)
    {
        if (tparm->intVal() == 0)
            return;
    }

    // Get type of query
    tparm = record->param("type");
    if (tparm != NULL)
    {
        querytype = tparm->intVal();
    }

    // Determine if query is a delta query or not
    if (!detail)
    {
        tparm = record->param("delta");
        if (tparm != NULL)
        {
            delta = (tparm->intVal() != 0);
        }
    }

    // Extract description field
    tparm = record->param("heading");
    if (tparm != NULL)
    {
        desc = tparm->strVal().c_str();
    }

    // Extract query interval
    tparm = record->param("frequency");
    if (tparm != NULL)
    {
        interval = tparm->intVal();
    }

    // Extract actual query
    tparm = record->param("text");
    if (tparm != NULL)
    {
        query = tparm->strVal().c_str();
    }

    // If SQL query extract target name
    if (querytype == QUERY_SQL)
    {
        tparm = record->param("tgtname");
        if (tparm != NULL)
        {
            target = tparm->strVal().c_str();
        }

        //Locate target record
        int       index = 0;

        for (index = 0; index < m_targets.size(); index++)
        {
            pTarget = m_targets[index];
            if (pTarget->m_name == target)
                break;
        }
        if (pTarget == NULL)
        {
            OtaAddTrace(CYR_OTA, ERRORLEVEL, "No target for SQL query");
            Stop();
            CORBA::String_var where = "OtaExecuterImpl::ReadQuery";
            CORBA::String_var what = "No target for SQL query";
            throw     CyrProcessException(what, where, -1);
        }
    }

    // determine handler
    switch (querytype)
    {
    case QUERY_EXEC:
        {
            pHandler = new OtaExec();
            break;
        }
    case QUERY_NTPERF:
        {
#ifdef WIN32
            pHandler = new OtaNTPerform();
            break;
#else
            OtaAddTrace(CYR_OTA, ERRORLEVEL,
                        "NT Query invalid on this platform");
            Stop();
            CORBA::String_var where = "OtaExecuterImpl::ReadQuery";
            CORBA::String_var what = "NT Query invalid on this platform";
            throw     CyrProcessException(what, where, -1);

            break;
#endif
        }
    case QUERY_SNMP:
        {
            pHandler = new OtaSnmp();
            break;
        }
    default:
        {
            OtaAddTrace(CYR_OTA, ERRORLEVEL, "Invalid query type");
            Stop();
            CORBA::String_var where = "OtaExecuterImpl::ReadQuery";
            CORBA::String_var what = "Invalid query type";
            throw     CyrProcessException(what, where, -1);

            break;
        }
    }

    // Create a new query object
    OtaQuery *qo = new OtaQuery(m_testRun, query, m_nodename, desc,
                                interval, delta, detail, querytype, pTarget,

                                pHandler);

    // Append object to list of queries
    m_query.push_back(qo);
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method sets environment variables from the list specified
void      OtaExecuterImpl::SetEnvironment()
{
    for (size_t i = 0; i < m_environments.size(); i++)
    {
        // Get next entry
        string    environment = m_environments[i];

        // Set environment
        if (environment.length() > 0)
        {
            OtaAddTrace(DBGSTD, DEBUGLEVEL,
                        "Setting environment variable: " + environment);
            int       status = putenv(environment.c_str());

            if (status != 0)
            {
                OtaAddTrace(DBGSTD, ERRORLEVEL,
                            "Error setting environment variable");
            }
        }
    }
}

/////////////////////////////////////////////////////////////////////////////
// LogConfig
///////////////////////////////////////////////////////////////////////////////
// @mfunc Log the configuration.
// @param
// @param
void      OtaExecuterImpl::LogConfig()
{
    char      msg[256];

    sprintf(msg, "Trace level = %d", m_tracing);
    OtaAddTrace(CYR_OTA, TRACE_CONFIG, msg);
    if (m_details)
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "Details enabled");
    }
    else
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "Details disabled");
    }

    if (m_environments.size() > 0)
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "Environment variables:");
        for (size_t i = 0; i < m_environments.size(); i++)
        {
            string    environment = m_environments[i];

            OtaAddTrace(CYR_OTA, TRACE_CONFIG, environment);
        }
    }
    else
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG,
                    "No environment variables defined");
    }

    if (m_targets.size() > 0)
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "Targets:");
        for (size_t i = 0; i < m_targets.size(); i++)
        {
            OtaTarget *pTarget = m_targets[i];

            pTarget->Log();
        }
    }
    else
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "No targets defined");
    }

    if (m_query.size() > 0)
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "Queries:");
        for (size_t i = 0; i < m_query.size(); i++)
        {
            OtaQuery *pQuery = m_query[i];

            pQuery->Log();
        }
    }
    else
    {
        OtaAddTrace(CYR_OTA, TRACE_CONFIG, "No queries defined");
    }

}
char     *OtaExecuterImpl::GetNodeName()
{
    CECInetInfo inet;

    return CORBA::string_dup(inet.GetHostName().c_str());
}

///////////////////////////////////////////////////////////////////////////////
// HELPER FUNCTIONS
// ================
///////////////////////////////////////////////////////////////////////////////
int       server_specific_init(void)
{
    return 0;
}

// Perform any server specific one time cleanup
int       server_specific_cleanup(void)
{
    return 0;
}

