/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "AsyncVirtualUserController.h"

#ifdef USE_PERF_DATA
#   include "PerfCounterKeys.hpp"
#endif

CFastLock *CVirtualUsersGroup::s_pUsersSl = NULL;


///////////////////////////////////////////////////////////////////////////////
//
// Class CAsyncVirtualUserController
//
///////////////////////////////////////////////////////////////////////////////

#ifdef USE_PERF_DATA
LONG *CAsyncVirtualUserController::m_spActiveUsers = NULL;
#endif

CAsyncVirtualUserController::CAsyncVirtualUserController(TScriptRule* pRule, LPCSTR zTraceList, LPCSTR zTraceFileName, DWORD dwViewSize)
    : CThreadPoolController(), m_scriptCache(pRule), m_pTimerQueue(NULL), m_dwFailedUsersCount(0L), m_dwSuccessfullUsersCount(0L)
{
    m_pTrace = NULL;
    if(zTraceFileName != NULL && zTraceList != NULL)
    {
        string s;
        istrstream is(zTraceList);
        BOOL bContinue = TRUE;

        m_pTrace = new CTrace(zTraceFileName, dwViewSize);
        if(m_pTrace)
        {
            GetTraceStream(m_pTrace) << "Trace started." << "\r\n" << endl;;
            ReleaseTraceStream();

            while(bContinue)
            {
                is >> s;
                if(is.fail() == FALSE)
                {
                    (*m_pTrace).AddTrace(s.c_str());
                }
                else
                {
                    bContinue = FALSE;
                }
            }
        }
    }
}

CAsyncVirtualUserController::~CAsyncVirtualUserController()
{
    if(m_pTimerQueue)
        delete m_pTimerQueue;

    if(m_pTrace != NULL)
        delete m_pTrace;
}

DWORD CAsyncVirtualUserController::Init(DWORD dwThreadPoolSize, DWORD dwNumberOfConcurrentThreadsInIocp)
{
#ifdef USE_PERF_DATA
    m_spActiveUsers = g_pPerformance->GetCounterValue(PRFCNT_OPENSTA_THREADPOOL_ACTIVEUSERS);
#endif
    DWORD res = CThreadPoolController::Init(dwThreadPoolSize, dwNumberOfConcurrentThreadsInIocp);

    m_VirtualUsersList.Init(m_pIocp);

    if(res == 0)
    {
        m_pTimerQueue = new CTimerQueue(*m_pIocp, INFINITE, m_pTrace);

        if(m_pTimerQueue == NULL)
            return -1; // TODO : positionner numro d'erreur.

        m_pTimerQueue->Init();
    }
    return res;
}

// Code inspired from Cthreadcontrol::Execute(...).
int CAsyncVirtualUserController::StartNewUsersGroup(DWORD nbUsers, const string &szTofFile
#ifndef CYRANO_TP_STANDALONE
                                                    , TestLevel_ptr pTL, const ThreadDef &thrdef
#endif
                                                    )
{
    // Alloc nbUsers clients.
    CVirtualUsersGroup *pGroup = m_VirtualUsersList.CreateNewGroup(*m_pIocp, *this, m_pTimerQueue, 4, nbUsers, szTofFile.c_str(), 
#ifndef CYRANO_TP_STANDALONE
        pTL, thrdef, 
#endif
        m_pTrace);

    if(pGroup != NULL)
    {
        // Start processing.
        // TODO: what should we do if did not receive EVENT_TYPE_THREADPOOL_START yet?
        pGroup->StartUsers();
#ifdef USE_PERF_DATA
#   ifdef LOCK_COUNTERS
        InterlockedExchangeAdd(m_spActiveUsers, nbUsers);
#   else
        *m_spActiveUsers += nbUsers;
#   endif
#endif
        return 0;
    }
    else
    {
        // TODO: memory error, file not found...
        assert(0);
        return -1;
    }
}

void CAsyncVirtualUserController::CancelAllUsers()
{
    m_VirtualUsersList.CancelAllUsers();
}

void CAsyncVirtualUserController::OnAllRequestsFinished(DWORD dwVuId)
{
//    CLock lock (m_sl);    // OCO: not used

#ifdef USE_PERF_DATA
#   ifdef LOCK_COUNTERS
    InterlockedDecrement(m_spActiveUsers);
#   else
    --*m_spActiveUsers;
#   endif
#endif

    DWORD dwRemainingUsers = m_VirtualUsersList.DropVirtualUser(dwVuId);

    if(dwRemainingUsers == 0)
    {
        // All VU have been processed. Bye bye everybody!
        if(m_pTrace && m_pTrace->IsTrace("full"))
        {
            GetTraceStream(m_pTrace) << "CAsyncVirtualUserController::OnAllRequestFinished: user " << dwVuId << " terminated. Simulation is over.\r\n";
            ReleaseTraceStream();
        }
        Stop(); // TODO: It's not our job. Instead, should signal the VU manager, which sould make the decision (because we don't know if all the users have been created).
    }
    else
    {
        if(m_pTrace && m_pTrace->IsTrace("full"))
        {
            GetTraceStream(m_pTrace) << "CAsyncVirtualUserController::OnAllRequestFinished: user " << dwVuId << " terminated. Remaining " << dwRemainingUsers << " users.\r\n";
            ReleaseTraceStream();
        }
    }
}

void CAsyncVirtualUserController::OnAllRequestsCancelled(DWORD dwVuId)
{
    OnAllRequestsFinished(dwVuId);
}

void CAsyncVirtualUserController::OnEvent(DWORD /*dwNumberOfBytesTransferred*/, BOOL bIsIo, DWORD dwEventType)
{
    if(bIsIo == TRUE)
    {
        // Should not receive any IOs there.
        assert(0);
    }
    else
    {
        switch(dwEventType)
        {
        case EVENT_TYPE_THREADPOOL_START:
            {
                // Nothing to do.
                break;
            }

        case EVENT_TYPE_THREADPOOL_STOP_THREAD:
            {
                if(m_pTrace && m_pTrace->IsTrace("errors"))
                {
                    GetTraceStream(m_pTrace) << "CAsyncVirtualUserController::OnEvent: ERROR: do not handle STOP THREAD events YET." << "\r\n";
                    ReleaseTraceStream();
                }
                break;
            }

        case EVENT_TYPE_TIMER:
            {
                if(m_pTrace && m_pTrace->IsTrace("errors"))
                {
                    GetTraceStream(m_pTrace) << "CAsyncVirtualUserController::OnEvent: ERROR: do not handle timer events." << "\r\n";
                    ReleaseTraceStream();
                }
                break;
            }

        default:
            {
                if(m_pTrace && m_pTrace->IsTrace("errors"))
                {
                    GetTraceStream(m_pTrace) << "CAsyncVirtualUserController::OnEvent: ERROR: unknown type." << "\r\n";
                    ReleaseTraceStream();
                }
                break;
            }
        };
    }
}

void CAsyncVirtualUserController::OnEventFailed(DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD dwEventType, DWORD dwError)
{
    // TODO
    if(m_pTrace && m_pTrace->IsTrace("errors"))
    {
        GetTraceStream(m_pTrace) << "CAsyncVirtualUserController::OnEventFailed: ERROR." << "\r\n";
        ReleaseTraceStream();
    }
}



///////////////////////////////////////////////////////////////////////////////
//
// Class CVirtualUsersGroup
//
///////////////////////////////////////////////////////////////////////////////

CVirtualUsersGroup::CVirtualUsersGroup(DWORD nbUsers)
    : m_VirtualUsersArray(NULL), m_bStarted(FALSE), m_pIocp(NULL), m_pTrace(NULL)
{
    assert(s_pUsersSl != NULL);
    CreateUsers(nbUsers);
}

BOOL CVirtualUsersGroup::CreateUsers(DWORD nbUsers)
{
    BOOL bCouldCreate;

    CLock lock (*s_pUsersSl);
    if(nbUsers != 0)
    {
        m_VirtualUsersArray = new CAsyncVirtualUser[nbUsers];
    }

    if(m_VirtualUsersArray != NULL)
    {
        m_dwVirtualUsersCount = nbUsers;
        bCouldCreate = TRUE;
    }
    else
    {
        m_dwVirtualUsersCount = 0;
        bCouldCreate = FALSE;
    }

    return bCouldCreate;
}

void CVirtualUsersGroup::CancelAllUsers()
{
    CLock lock (*s_pUsersSl);
    for(DWORD dw=0L ; dw<m_dwVirtualUsersCount ; dw++)
    {
        m_VirtualUsersArray[dw].CancelAllQueries();
    }
}

void CVirtualUsersGroup::InitUsers(CIOCP& iocp, CAsyncVirtualUserController& controller, CTimerQueue* pTimerQueue, DWORD dwParallelism, const string &szTofFile, 
#ifndef CYRANO_TP_STANDALONE
                                   TestLevel_ptr pTL, const ThreadDef &thrdef,
#endif
                                   CTrace* pTrace)
{
    static DWORD dwManagerID = 0;
    m_pTrace = pTrace;

    m_pIocp = &iocp;

    CLock lock (*s_pUsersSl);
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        m_VirtualUsersArray[dw].Init(controller, pTimerQueue, dwManagerID, dwParallelism, pTrace);

#ifndef CYRANO_TP_STANDALONE
        m_VirtualUsersArray[dw].Initialise(szTofFile, pTL->TestName(), thrdef);
#else
        m_VirtualUsersArray[dw].Initialise(szTofFile);
#endif

        dwManagerID++;
    }
}

// Start the users if they have not been started yet.
void CVirtualUsersGroup::StartUsers()
{
    CLock lock (*s_pUsersSl);
    if(m_bStarted == FALSE)
    {
        for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
        {
            if(m_pTrace && m_pTrace->IsTrace("full"))
            {
                GetTraceStream(m_pTrace) << "Posting a start message to " << &m_VirtualUsersArray[dw] << "\r\n" << endl;;
                ReleaseTraceStream();
            }

            if(m_pIocp->PostCompletedIO(0, (IocpEventDispatcher*) &m_VirtualUsersArray[dw], (OVERLAPPED*)CAsyncVirtualUser::EVENT_TYPE_VIRTUAL_USER_START) != 0)
            {
                assert(0);
                if(m_pTrace && m_pTrace->IsTrace("full"))
                {
                    GetTraceStream(m_pTrace) << "Start users ERROR." << "\r\n" << endl;;
                    ReleaseTraceStream();
                }
            }
        }

        m_bStarted = TRUE;
    }
}

// Delete all the users from that group.
void CVirtualUsersGroup::DropUsers()
{
    CLock lock (*s_pUsersSl);
    if(m_VirtualUsersArray != NULL)
    {
        delete [] m_VirtualUsersArray;
        m_VirtualUsersArray = NULL;
        m_dwVirtualUsersCount = 0;
    }
}

// Append the pipe-separated list of all unique identifiers for that group's users (formerly Threads).
// Code based on TestExecuterImpl::GetThreadIdents().
void CVirtualUsersGroup::AppendThreadIdents(string &str) const
{
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        str += m_VirtualUsersArray[dw].GetThreadID() + '|';
    }
}

DWORD CVirtualUsersGroup::GetMatchingThreadsCount(const std::string &sname) const
{
    DWORD dwThreadsCount = 0;
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        if(m_VirtualUsersArray[dw].ScriptName() == sname)
            dwThreadsCount++;
    }

    return dwThreadsCount;
}

#ifndef CYRANO_TP_STANDALONE
DWORD CVirtualUsersGroup::GetTestCaseResults(TestCaseStatus Status) const
{
    DWORD dwRes = 0;

    // Ask all the groups to return their working threads UIDs to the end of 'str'.
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        dwRes += m_VirtualUsersArray[dw].GetTestCaseResults(Status);
    }

    return dwRes;
}
#endif

// Return thread current scriptname based on unique ID.
BOOL CVirtualUsersGroup::GetVuScriptName(const char *ident, string &name) const
{
    // Browse all the threads in that group in search for the 'ident' thread and return its ScriptName.
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        if(m_VirtualUsersArray[dw].GetThreadID() == ident)
        {
            name = m_VirtualUsersArray[dw].ScriptName();
            return TRUE;
        }
    }

    return FALSE;
}

// Return a thread's execution phase based on its unique ID.
BOOL CVirtualUsersGroup::GetVuStatus(const char *ident, EVuExecPhase &phase) const
{
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        if(m_VirtualUsersArray[dw].GetThreadID() == ident)
        {
            phase = m_VirtualUsersArray[dw].GetStatus();
            return TRUE;
        }
    }

    return FALSE;
}

BOOL CVirtualUsersGroup::GetVuStartTime(const char *ident, ULONG &ulTime) const
{
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        if(m_VirtualUsersArray[dw].GetThreadID() == ident)
        {
            ulTime = m_VirtualUsersArray[dw].GetStartTime();
            return TRUE;
        }
    }

    return FALSE;
}

#ifndef CYRANO_TP_STANDALONE
DWORD CVirtualUsersGroup::AppendThreadInformation(ThreadInfoList &TIList, DWORD dwStartPos) const
{
    for(DWORD dw=0 ; dw<m_dwVirtualUsersCount ; dw++)
    {
        m_VirtualUsersArray[dw].GetVUInfo(TIList[dwStartPos + dw]);
    }

    return m_dwVirtualUsersCount;
}
#endif


///////////////////////////////////////////////////////////////////////////////
//
// Class CVirtualUsersList
//
///////////////////////////////////////////////////////////////////////////////

CVirtualUsersList::CVirtualUsersList()
    : m_dwVirtualUsersCount(0), m_pIocp(NULL)
{
}

CVirtualUsersList::~CVirtualUsersList()
{
    DeleteGroups();
}

DWORD CVirtualUsersList::Init(CIOCP* pIocp)
{
    m_pIocp = pIocp;

    return 0;
}

CVirtualUsersGroup *CVirtualUsersList::CreateNewGroup(CIOCP& iocp, CAsyncVirtualUserController& controller, CTimerQueue* pTimerQueue, DWORD dwParallelism, DWORD nbUsers, const string &szTofFile, 
#ifndef CYRANO_TP_STANDALONE
                                                      TestLevel_ptr pTL, const ThreadDef &thrdef, 
#endif
                                                      CTrace* pTrace)
{
    // TODO : pb if nbUsers is small.
    CVirtualUsersGroup *pNewGroup = new CVirtualUsersGroup;
    
    if(pNewGroup != NULL)
    {
        if(pNewGroup->CreateUsers(nbUsers) == TRUE)
        {
            InterlockedExchangeAdd((LONG*)&m_dwVirtualUsersCount, (LONG)nbUsers);

            {
                CLock guard (m_userGroupSl);
                m_GroupsList.push_back(pNewGroup);
            }

            pNewGroup->InitUsers(iocp, controller, pTimerQueue, dwParallelism, szTofFile, 
#ifndef CYRANO_TP_STANDALONE
                                pTL, thrdef,
#endif
                                pTrace);
        }
        else
        {
            delete pNewGroup;
        }
    }

    return pNewGroup;
}

void CVirtualUsersList::CancelAllUsers()
{
    CLock lock(m_userGroupSl);

    for(vector<CVirtualUsersGroup*>::iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        (*i)->CancelAllUsers();
    }
}

DWORD CVirtualUsersList::DropVirtualUser(DWORD dwVuId)
{
    // TODO: should really drop the VU or at least the matching CVirtualUsersGroup when that group is empty.
    return InterlockedDecrement((LONG*)&m_dwVirtualUsersCount);
}

// Start all the clients that have not been started yet.
void CVirtualUsersList::StartAll()
{
    CLock lock(m_userGroupSl);

    for(vector<CVirtualUsersGroup*>::iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        (*i)->StartUsers();
    }
}

// Optimises the memory buffer.
void CVirtualUsersList::SetMinSize(DWORD dwMinSize)
{
    // TODO
}

void CVirtualUsersList::DeleteGroups()
{
    CLock lock(m_userGroupSl);

    for(vector<CVirtualUsersGroup*>::iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        (*i)->DropUsers();
    }

    m_GroupsList.clear();
}

// Get pipe-separated list of all unique identifiers for users (formerly Threads) in this Test executer.
// Code based on TestExecuterImpl::GetThreadIdents().
void CVirtualUsersList::GetThreadIdents(string &str) const
{
    str.erase();

    // Ask all the groups to append their working threads UIDs to the end of 'str'.
    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        (*i)->AppendThreadIdents(str);
    }

    str += '|';
}

DWORD CVirtualUsersList::GetMatchingThreadsCount(const std::string &sname) const
{
    DWORD dwThreadsCount = 0;

    // Ask all the groups to return their working threads UIDs to the end of 'str'.
    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        dwThreadsCount += (*i)->GetMatchingThreadsCount(sname);
    }

    return dwThreadsCount;
}

#ifndef CYRANO_TP_STANDALONE
DWORD CVirtualUsersList::GetTestCaseResults(TestCaseStatus Status) const
{
    DWORD dwRes = 0;

    // Ask all the groups to return their working threads UIDs to the end of 'str'.
    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        dwRes += (*i)->GetTestCaseResults(Status);
    }

    return dwRes;
}
#endif

// Return thread current scriptname based on unique ID.
string CVirtualUsersList::GetVuScriptName(const char *ident) const
{
    string scriptName;

    // Browse all the groups in search for the 'ident' thread and return its ScriptName.
    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        if((*i)->GetVuScriptName(ident, scriptName) == TRUE)
            return scriptName;
    }

    // Could not find that user.
    assert(0);
    return string();
}

// Browse the groups until we find 'ident' and return its status.
EVuExecPhase CVirtualUsersList::GetVuStatus(const char *ident) const
{
    EVuExecPhase execPhase;

    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        if((*i)->GetVuStatus(ident, execPhase) == TRUE)
            return execPhase;
    }

    // Could not find that user.
    assert(0);
    return EXEC_OVER;
}

// Browse all the groups in search for the 'ident' thread and return its ScriptName.
ULONG CVirtualUsersList::GetVuStartTime(const char *ident) const
{
    ULONG time;

    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        if((*i)->GetVuStartTime(ident, time) == TRUE)
            return time;
    }

    // Could not find that user.
    assert(0);
    return 0;
}

#ifndef CYRANO_TP_STANDALONE
// Pack info about all the virtual users.
DWORD CVirtualUsersList::GetThreadInformation(ThreadInfoList &TIList) const
{
    DWORD dwUsersCount = GetUsersCount();
    DWORD dwPosInList = 0;

    TIList.length(dwUsersCount);
    for(vector<CVirtualUsersGroup*>::const_iterator i=m_GroupsList.begin() ; i<m_GroupsList.end() ; i++)
    {
        dwPosInList += (*i)->AppendThreadInformation(TIList, dwPosInList);
    }
    
    return dwUsersCount;
}
#endif
