/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _OPENSTA_IOCP_H_
#define _OPENSTA_IOCP_H_

////////////////////////////////////////////////////////////////////////////////
// I/O completion port definition file.
//
// Return values: unless otherwise stated, a DWORD return value is a
// GetLastError() error code.
////////////////////////////////////////////////////////////////////////////////

#include <winbase.h>
#include "Trace.hpp"

// This class is the secondary interface used to distribute completion status information
// from the IOCP's I/O completion queue.
// If dwEventType > 0xFFFF, then it is actually a real OVERLAPPED
// pointer, and bIsIo is set to TRUE. Otherwise, bIsIo is FALSE and the value
// is a user-defined event type (valid range: 0 - 0xFFFF).
// THREAD SAFE.
struct IocpEvent
{
    // Standard event types. Reserved range: 0 - 0x7FFF.
    // Framework users may create additional types in the range 0x8000 - 0xFFFF.
    enum StandardEventTypes
    {
        EVENT_TYPE_THREADPOOL_START         = 0x1,  // To be handled by user.
        EVENT_TYPE_THREADPOOL_STOP_THREAD   = 0x2,  // Automatically handled by framework.
        EVENT_TYPE_TIMER                    = 0x3,  // To be handled by user.
    };

    // Methods.
    virtual void OnEvent      (DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD dwEventType) = 0;
    virtual void OnEventFailed(DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD dwEventType, DWORD dwError) = 0;
};

// This class is the main interface for completion status information distribution.
// Its role is to allow implementations to change the routing of completion status
// information at run time.
// THREAD SAFE.
struct IocpEventDispatcher
{
    // Retrieves the secondary completion status information distribution interface.
    virtual IocpEvent* GetIocpEventTarget() = 0;
};

// This class wraps a Win32 I/O completion port.
// It manages the instant count of outstanding I/Os, i. e., entries in the
// IOCP's I/O completion queue.
// THREAD SAFE, except for the Init() method.
class CIOCP
{
public:
    //// --------------------------------------------------------------------------------
    //// Construction / Destruction.
    //// --------------------------------------------------------------------------------
    CIOCP(DWORD dwNumberOfConcurrentThreads = 0, CTrace* pTrace = NULL);
    ~CIOCP();

    //// --------------------------------------------------------------------------------
    //// User methods.
    //// --------------------------------------------------------------------------------

    // Actually create the Win32 IOCP.
    DWORD Init();

    // Get the number of outstanding IOs in the IOCP's I/O completion queue.
    DWORD GetOutstandingIoCount() const
    {
#ifdef USE_PERF_DATA
        return (*m_pIOCPQueueLengthPerfCount);
#else
        return 0;
#endif
    }

    // Get the peak number of outstanding IOs in the IOCP's I/O completion queue.
    DWORD GetPeakOustandingIos() const {return m_dwPeakOutstandingIOs;}

    // Clear the number of outstanding IOs and returns the new value.
    void ClearOutstandingIoCount() const
    {
#ifdef USE_PERF_DATA
#   ifdef LOCK_COUNTERS
        InterlockedExchange(m_pIOCPQueueLengthPerfCount, 0);
#   else
        *m_pIOCPQueueLengthPerfCount = 0;
#   endif
#endif
    }

    // Increment the number of outstanding IOs and returns the new value.
    DWORD IncrementOutstandingIoCount() const
    {
#ifdef USE_PERF_DATA
#   ifdef LOCK_COUNTERS
        DWORD dwIos = (DWORD)InterlockedIncrement(m_pIOCPQueueLengthPerfCount);
#   else
        DWORD dwIos = (DWORD)++(*m_pIOCPQueueLengthPerfCount);
#   endif
        if(dwIos > m_dwPeakOutstandingIOs)
            InterlockedExchange((LPLONG)&m_dwPeakOutstandingIOs, dwIos);
        return dwIos;
#else
        return 0;
#endif
    }

    // Decrement the number of outstanding IOs and returns the new value.
    DWORD DecrementOutstandingIoCount() const
    {
#ifdef USE_PERF_DATA
#   ifdef LOCK_COUNTERS
        return (DWORD)InterlockedDecrement(m_pIOCPQueueLengthPerfCount);
#   else
        DWORD dwCnt = (DWORD)--(*m_pIOCPQueueLengthPerfCount);
        if(dwCnt > 0x80000000)
            dwCnt = 0;  // Assume a collision.
        return dwCnt;
#   endif
#else
        return 0;
#endif
    }

    // Returns the number of concurrent threads.
    DWORD GetNbConcurrentThreads() const {return m_dwNumberOfConcurrentThreads;}

    // Explicitely append an entry to the IOCP's I/O completion queue.
    // Return value: 0 = success, other = GetLastError() failure code.
    DWORD PostCompletedIO(DWORD dwNumberOfBytes, IocpEventDispatcher *pEventDispatcher, OVERLAPPED *pOverlapped = NULL);

    // Extracts the next completed I/O entry from the IOCP's I/O completion queue.
    // Internally calls GetQueuedCompletionStatus().
    DWORD GetNextCompletionStatus(DWORD *pdwBytesTransferred, IocpEventDispatcher*& pEventDispatcher, LPOVERLAPPED *ppOverlapped, DWORD dwMilliseconds);

    // HANDLE cast operator.
    operator HANDLE() const {return m_hIOCP;}

protected:
    //// --------------------------------------------------------------------------------
    //// Data.
    //// --------------------------------------------------------------------------------
    HANDLE  m_hIOCP;                        // Handle of the Win32 IOCP kernel object.
    DWORD   m_dwPeakOutstandingIOs;         // Peak number of outstanding IOs in the IOCP's I/O completion queue.
    DWORD   m_dwNumberOfConcurrentThreads;  // Number of concurrent threads for this IOCP.
    CTrace *m_pTrace;                       // The ostream trace support object.

    LONG *  m_pIOCPQueueLengthPerfCount;    // Pointer to performance counter storing the completion queue length.

private:
};

#endif  // _OPENSTA_IOCP_H_
