/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _OPENSTA_REGISTRY_H_
#define _OPENSTA_REGISTRY_H_

////////////////////////////////////////////////////////////////////////////////
// OpenSTA
//
// Registry key object definition file.
//
// Return values: unless otherwise stated, a DWORD return value is a
// GetLastError() error code.
////////////////////////////////////////////////////////////////////////////////

// This class implements a registry key object.
class CRegistryKey
{
public:
    //// Construction/Destruction.
    CRegistryKey() : m_hKey(NULL) {}
    CRegistryKey(PCWSTR zSubkey, HKEY hParentKey = HKEY_LOCAL_MACHINE, BOOL bReadOnly = TRUE)
        : m_hKey(NULL) {Open(zSubkey, hParentKey, bReadOnly);}
    CRegistryKey(PCWSTR zSubkey, const CRegistryKey& parentKey, BOOL bReadOnly = TRUE)
        : m_hKey(NULL) {Open(zSubkey, parentKey, bReadOnly);}
    ~CRegistryKey(){}

    //// Opening and closing.
    DWORD Open(PCWSTR zSubkey, HKEY hParentKey = HKEY_LOCAL_MACHINE, BOOL bReadOnly = TRUE)
    {
        DWORD dwRet;
        Close();
        if(bReadOnly)
            dwRet = RegOpenKeyExW(hParentKey, zSubkey, 0, KEY_QUERY_VALUE, &m_hKey);
        else
            dwRet = RegCreateKeyExW(hParentKey, zSubkey, 0, NULL,
                REG_OPTION_NON_VOLATILE, KEY_QUERY_VALUE | KEY_SET_VALUE | KEY_CREATE_SUB_KEY,
                NULL, &m_hKey, NULL);
        if(dwRet != ERROR_SUCCESS)
            m_hKey = NULL;
        return dwRet;
    }
    DWORD Open(PCWSTR zSubkey, const CRegistryKey& parentKey, BOOL bReadOnly = TRUE)
    {
        return Open(zSubkey, parentKey.m_hKey, bReadOnly);
    }
    BOOL IsOpen() const {return m_hKey != NULL;}
    DWORD Close()
    {
        DWORD dwRet = ERROR_SUCCESS;
        if(m_hKey)
        {
            dwRet = RegCloseKey(m_hKey);
            m_hKey = NULL;
        }
        return dwRet;
    }

    //// Value get.
    DWORD GetValue(PCWSTR zValueName, DWORD& dwValue) const
    {
        DWORD dwSize = sizeof(DWORD);
        return RegQueryValueExW(m_hKey, zValueName, NULL, NULL, (PBYTE)&dwValue, &dwSize);
    }
    DWORD GetValue(PCWSTR zValueName, PWSTR zValue, DWORD dwSize) const // For strings and multistrings.
        {return RegQueryValueExW(m_hKey, zValueName, NULL, NULL, (PBYTE)zValue, &dwSize);}
    DWORD GetValueSize(PCWSTR zValueName, DWORD& dwValueSize) const
        {return RegQueryValueExW(m_hKey, zValueName, NULL, NULL, NULL, &dwValueSize);}


    //// Value set.
    DWORD SetValue(PCWSTR zValueName, DWORD dwValue) const
        {return RegSetValueExW(m_hKey, zValueName, 0, REG_DWORD, (const BYTE*)&dwValue, sizeof(dwValue));}
    DWORD SetValue(PCWSTR zValueName, PCWSTR zValue, BOOL bMultiString = FALSE) const
    {
        DWORD dwSize;
        if(bMultiString)
        {
            for(PCWSTR pV = zValue; *pV != 0; pV += lstrlenW(pV)+1);
            dwSize = (pV - zValue + 1) * sizeof(WCHAR);
        }
        else
            dwSize = (lstrlenW(zValue) + 1) * sizeof(WCHAR);
        return RegSetValueExW(m_hKey, zValueName, 0, (bMultiString ? REG_MULTI_SZ : REG_SZ), (const BYTE*)zValue, dwSize);
    }

private:
    HKEY m_hKey;

    //// Copy forbidden.
    CRegistryKey(const CRegistryKey&);
    void operator=(const CRegistryKey&);
};

#endif  // _OPENSTA_REGISTRY_H_
