/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __SHARED_MEM_BUFFS__
#define __SHARED_MEM_BUFFS__

#include "FastLock.hpp"

// ----------------------------------------------------------------------------
// Class used to allocate buffers from the virtual memory and share them
// between 'clients'. Used to avoid allocating and freeing memory from the heap.
// Must be allocated after CFastLock::Init().
// ----------------------------------------------------------------------------

class CSharedMemBuffs
{
public:
    CSharedMemBuffs(DWORD dwMaxBuffersCount, DWORD dwMaxBufferSize, DWORD dwGrowingBuffCount);
    ~CSharedMemBuffs();

    PBYTE   AllocDataBuffer();
    void    ReturnDataBuffer(PBYTE pReturnedBuff);

protected:
    // Helpers.
    BOOL CommitNextRegion();

protected:
    // Data.
    CFastLock *     m_pFl;
    DWORD           m_dwMaxBuffCount;

    PBYTE           m_pvVirtualMemBuff;         // Pointer to the virtual memory buffer.
    DWORD           m_dwCommittedBuffs;
    DWORD           m_dwGrowingBuffCount;       // Number of buffers that must be allocated each time.
    DWORD           m_dwMaxBufferSize;

    stack<PBYTE>    m_freeBuffStack;

private:
    // Forbidden operator.
    const CSharedMemBuffs &operator=(const CSharedMemBuffs &);
};


// ----------------------------------------------------------------------------
// Allocate a virtual memory area capable of storing dwMaxBuffersCount buffers, 
// each of which capable of storing dwMaxBufferSize bytes. dwGrowingBuffCount 
// buffers will be committed from that area at a time.
//
// Note the size of the committed memory must be a multiple of the system page 
// size though dwMaxBufferSize might not be.
// ----------------------------------------------------------------------------
inline CSharedMemBuffs::CSharedMemBuffs(DWORD dwMaxBuffersCount, DWORD dwMaxBufferSize, DWORD dwGrowingBuffCount)
    :   m_dwMaxBuffCount(dwMaxBuffersCount), m_dwMaxBufferSize(dwMaxBufferSize), m_dwCommittedBuffs(0), 
        m_dwGrowingBuffCount(dwGrowingBuffCount)
{
    m_pFl = new CFastLock();
    assert(m_pFl);

    // Create the memory pool.
    DWORD dwAllocSize = dwMaxBuffersCount * dwMaxBufferSize;
    m_pvVirtualMemBuff = (PBYTE)VirtualAlloc(NULL, dwAllocSize, MEM_RESERVE, PAGE_READWRITE);
    assert(m_pvVirtualMemBuff);

    // Pre-allocate buffers.
    CommitNextRegion();
}

inline CSharedMemBuffs::~CSharedMemBuffs()
{
    if(VirtualFree(m_pvVirtualMemBuff, 0, MEM_RELEASE) == FALSE)
    {
        DWORD dwErr = GetLastError();
        assert(0);
    }

    if(m_pFl != NULL)
        delete m_pFl;
}

// ----------------------------------------------------------------------------
// Return a buffer from the pool. Eventually grows the pool.
// ----------------------------------------------------------------------------
inline PBYTE CSharedMemBuffs::AllocDataBuffer()
{
    CLock guard(m_pFl);

    if(m_freeBuffStack.empty())
    {
        if(CommitNextRegion() == FALSE)
            return FALSE;
    }

    PBYTE pBuff = m_freeBuffStack.top();
    m_freeBuffStack.pop();

    return pBuff;
}

// ----------------------------------------------------------------------------
// Return a buffer to the pool.
// ----------------------------------------------------------------------------
inline void CSharedMemBuffs::ReturnDataBuffer(PBYTE pReturnedBuff)
{
    CLock guard(m_pFl);

    m_freeBuffStack.push(pReturnedBuff);
}

// ----------------------------------------------------------------------------
// Allocate new buffers from the VM.
// ----------------------------------------------------------------------------
inline BOOL CSharedMemBuffs::CommitNextRegion()
{
    // Allocate new buffers from m_pvVirtualMemBuff.
    DWORD dwCommitBegin = m_dwCommittedBuffs * m_dwMaxBufferSize;
    DWORD dwToCommitSize = m_dwMaxBufferSize * m_dwGrowingBuffCount;

    if(VirtualAlloc((LPVOID)((PBYTE)m_pvVirtualMemBuff + dwCommitBegin), dwToCommitSize, MEM_COMMIT, PAGE_READWRITE) == NULL)
        return FALSE;

    PBYTE iterator = m_pvVirtualMemBuff + dwCommitBegin;

    // Add the new buffers to the free buffers stack.
    for(DWORD dw=0 ; dw<m_dwGrowingBuffCount ; dw++)
    {
        m_freeBuffStack.push(iterator);
        iterator += m_dwMaxBufferSize;
    }

    m_dwCommittedBuffs += m_dwGrowingBuffCount;

    return TRUE;
}

#endif  // __SHARED_MEM_BUFFS__