/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#include "Statistics.hpp"
#include "Fastlock.hpp"

using namespace std;

BOOL CStatistics::Init(const string &zStatsFileName, const string &zUrlsFileName, DWORD dwViewSize)
{
    if(sm_pWriteSl == NULL)
        sm_pWriteSl = new CFastLock();
    if(sm_pWriteSl == NULL)
        return FALSE;

	sm_tOffset = 0;

    // Query for the performance timer resolution (counts per second) to help compute elapsed time.
    if(QueryPerformanceFrequency(&sm_liPerfCtrFrequency) == 0)
    {
        // Perf. counters not supported by hardware. Will use GetSystemTimeAsFileTime.
    }

    SYSTEM_INFO sysInfo;
    GetSystemInfo(&sysInfo);

    if(dwViewSize == 0)
    {
        dwViewSize = sysInfo.dwAllocationGranularity;
    }
    else
    {
        // Make sure the specified dwViewSize is a multiple of sysInfo.dwAllocationGranularity.
        if(dwViewSize == 0 || dwViewSize % sysInfo.dwAllocationGranularity != 0)
        {
            // Error: bad parameter.
            return FALSE;
        }
    }

#ifndef CYRANO_TP_STANDALONE
    if((CreateMemMappedFile(sm_statsFile, zStatsFileName, dwViewSize) && CreateMemMappedFile(sm_urlsFile, zUrlsFileName, dwViewSize)) == FALSE)
    {
        InterlockedExchange((PLONG)&sm_bErrorDontWrite, TRUE); // Won't dump anything to the files.
        return FALSE;
    }
    else
    {
        return TRUE;
    }
#else
    // Won't dump the stats file.
    return TRUE;
#endif
}

BOOL CStatistics::CreateMemMappedFile(SFileInfo &fileInfo, const string &zFileName, DWORD dwViewSize)
{
    fileInfo.m_dwCurrViewOffset = 0;
    fileInfo.m_dwViewSize = dwViewSize;

    // Output to a file. Map it into memory.
    fileInfo.m_hFile = CreateFile(zFileName.c_str(), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if(fileInfo.m_hFile != INVALID_HANDLE_VALUE)
    {
        HANDLE hMap = CreateFileMapping(fileInfo.m_hFile, NULL, PAGE_READWRITE, 0, dwViewSize, NULL);
        if(hMap != NULL)
        {
            fileInfo.m_pView = (LPSTR)MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, 0);
            CloseHandle(hMap);
            if(fileInfo.m_pView == NULL)
            {
                CloseHandle(fileInfo.m_hFile);
                fileInfo.m_hFile = NULL;

                return FALSE;
            }
            else
                return TRUE;
        }
        else
        {
            CloseHandle(fileInfo.m_hFile);
            fileInfo.m_hFile = NULL;

            return FALSE;
        }
    }
    else
    {
        fileInfo.m_hFile = NULL;
        return FALSE;
    }
}

void CStatistics::ReleaseFile(SFileInfo &fileInfo)
{
    if(fileInfo.m_pView && fileInfo.m_hFile)
    {
        // File output: adjust file size and close.
        UnmapViewOfFile(fileInfo.m_pView);
        SetFilePointer(fileInfo.m_hFile, fileInfo.m_dwFileSize + fileInfo.m_dwCurrViewOffset, NULL, FILE_BEGIN);
        SetEndOfFile(fileInfo.m_hFile);
        CloseHandle(fileInfo.m_hFile);
    }

    if(sm_pWriteSl != NULL)
    {
        delete sm_pWriteSl;
        sm_pWriteSl = NULL;
    }
}

void CStatistics::Release()
{
    ReleaseFile(sm_statsFile);
    ReleaseFile(sm_urlsFile);
}


// --------------------------------------------------------------------------------
// EnlargeView
//
// Because of MapViewOfFile requirements linked to the memory granularity, must 
// fill the rest of the current view, then move the view/enlarge the file, and 
// finally copy the remaining data to the new view.
//
// WARNING: fileInfo.m_dwViewSize must be a multiple of system's memory allocation 
// granularity otherwise next call to EnlargeView would fail on MapViewOfFile.
// --------------------------------------------------------------------------------
BOOL CStatistics::EnlargeView(SFileInfo &fileInfo, CONST VOID *pData, DWORD dwDataSize)
{
    // Fill the remaining of the current view.
    DWORD dwViewSizeLeft = fileInfo.m_dwViewSize - fileInfo.m_dwCurrViewOffset;
    CopyMemory( fileInfo.m_pView + fileInfo.m_dwCurrViewOffset, 
                pData, 
                dwViewSizeLeft);

    // Adjust the size of the file.
    fileInfo.m_dwFileSize += fileInfo.m_dwViewSize;

    // Enlarge the file and move the view.
    UnmapViewOfFile(fileInfo.m_pView);

    HANDLE hMap = CreateFileMapping(fileInfo.m_hFile, NULL, PAGE_READWRITE, 0, fileInfo.m_dwFileSize + fileInfo.m_dwViewSize, NULL);
    if(hMap == NULL)
    {
        CloseHandle(fileInfo.m_hFile);
        fileInfo.m_hFile = NULL;

        // TODO: tester si l'interface graphique saura ouvrir le fichier malgr que la SOutputStats n'est pas complet.
        // Sinon, redimensionner le fichier en lui retirant dwViewSizeLeft bytes.

        return FALSE;
    }
    else
    {
        fileInfo.m_pView = (LPSTR)MapViewOfFile(hMap, FILE_MAP_WRITE, 0, fileInfo.m_dwFileSize, fileInfo.m_dwViewSize);
        CloseHandle(hMap);

        if(fileInfo.m_pView == NULL)
        {
            DWORD dwErr = GetLastError();   // TODO: tracer ces erreurs ?
            CloseHandle(fileInfo.m_hFile);
            fileInfo.m_hFile = NULL;

            return FALSE;
        }
        else
        {
            // Dump the remaining data.
            CopyMemory( fileInfo.m_pView, 
                        (CHAR *)pData + dwViewSizeLeft, 
                        dwDataSize - dwViewSizeLeft);

            fileInfo.m_dwCurrViewOffset = dwDataSize - dwViewSizeLeft;

            return TRUE;
        }
    }
}

// TODO: une optimisation possible est de commencer par rserver la zone d'criture sous le couvert de la CS,
// puis de rendre la main avant de faire les CopyMemory. Attention: il ne faut pas qu'un autre thread appelle
// EnlargeView entre-temps.
void CStatistics::DumpData()
{
    if(!sm_bErrorDontWrite)
    {
        CLock lock (sm_pWriteSl);

        // WARNING: assumes that 'sizeof SOutputStats' is smaller than m_dwViewSize.

        // Write statistics.
        if(sm_statsFile.m_dwCurrViewOffset + sizeof SOutputStats > sm_statsFile.m_dwViewSize)
        {
            if(EnlargeView(sm_statsFile, &m_statsData, sizeof SOutputStats) == FALSE)
                InterlockedExchange((PLONG)&sm_bErrorDontWrite, TRUE);
        }
        else
        {
            CopyMemory(sm_statsFile.m_pView + sm_statsFile.m_dwCurrViewOffset, &m_statsData, sizeof SOutputStats);
            sm_statsFile.m_dwCurrViewOffset += sizeof SOutputStats;
        }

        // Write urls.
        if(sm_urlsFile.m_dwCurrViewOffset + m_szURL.size() + 1 > sm_urlsFile.m_dwViewSize)
        {
            if(EnlargeView(sm_urlsFile, m_szURL.c_str(), m_szURL.size() + 1) == FALSE)
                InterlockedExchange((PLONG)&sm_bErrorDontWrite, TRUE);
        }
        else
        {
            CopyMemory(sm_urlsFile.m_pView + sm_urlsFile.m_dwCurrViewOffset, m_szURL.c_str(), m_szURL.size());
            sm_urlsFile.m_dwCurrViewOffset += m_szURL.size();
            sm_urlsFile.m_pView[sm_urlsFile.m_dwCurrViewOffset] = '\0';
            sm_urlsFile.m_dwCurrViewOffset++;
        }
    }

    m_statsData.m_dwDataFlow = 0;
}

void CStatistics::OnRequestStart()
{
    m_statsData.m_dwDataFlow = 0;
    time(&m_statsData.m_ttTimeStamp);

	m_statsData.m_ttTimeStamp = m_statsData.m_ttTimeStamp - sm_tOffset;

    if(QueryPerformanceCounter(&m_liRequestStartCounter) == 0)
    {
        // Hardware does not provide a high-resolution performance timer.
        GetSystemTimeAsFileTime(&m_ftRequestSystemTime);
    }
}

void CStatistics::OnRequestOver()
{
    LARGE_INTEGER liNowCounter;

    if(QueryPerformanceCounter(&liNowCounter) != 0)
    {
        LARGE_INTEGER liDuration;
        assert(liNowCounter.QuadPart > m_liRequestStartCounter.QuadPart);
        liDuration.QuadPart = ((liNowCounter.QuadPart - m_liRequestStartCounter.QuadPart) * 1000i64) / sm_liPerfCtrFrequency.QuadPart;
        m_statsData.m_ctElapsedTime = (clock_t) liDuration.QuadPart;
    }
    else
    {
        // Hardware does not provide a high-resolution performance timer.

        FILETIME   ftNow;
        unsigned __int64    qwNow;
        unsigned __int64    qwStart;
        unsigned __int64    qwDuration;

        GetSystemTimeAsFileTime(&ftNow);

        // Compute the time elapsed since request started.

        qwNow = ftNow.dwHighDateTime;
        qwNow <<= 32;
        qwNow |= ftNow.dwLowDateTime;

        qwStart = m_ftRequestSystemTime.dwHighDateTime;
        qwStart <<= 32;
        qwStart |= m_ftRequestSystemTime.dwLowDateTime;

        qwDuration = (qwNow - qwStart) / 10000;

        assert(qwDuration <= INT_MAX);
        m_statsData.m_ctElapsedTime = (clock_t)qwDuration;
    }
}

void CStatistics::SetTimeOffset(long t_offset)
{
	sm_tOffset = t_offset;
}

CStatistics::SFileInfo CStatistics::sm_statsFile    = { NULL, NULL, 0, 0, 0 };
CStatistics::SFileInfo CStatistics::sm_urlsFile     = { NULL, NULL, 0, 0, 0 };

CFastLock *     CStatistics::sm_pWriteSl            = NULL;
LARGE_INTEGER   CStatistics::sm_liPerfCtrFrequency;

#ifndef CYRANO_TP_STANDALONE
BOOL            CStatistics::sm_bErrorDontWrite     = FALSE;
#else
// Don't write the data to the file.
BOOL            CStatistics::sm_bErrorDontWrite     = TRUE;
#endif

long			CStatistics::sm_tOffset = 0;