/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _OPENSTA_TRACE_H_
#define _OPENSTA_TRACE_H_

////////////////////////////////////////////////////////////////////////////////
// Trace definition file.
//
// Return values: unless otherwise stated, a DWORD return value is a
// GetLastError() error code.
////////////////////////////////////////////////////////////////////////////////

#include <sstream>
#include <string>
#include <set>
#include <assert.h>
#include "FastLock.hpp"

#include "TPLog.h"
/*extern const char* STARTUP;         // Startup trace messages
extern const char* SHUTDOWN;        // shutdown trace messages
extern const char* SCRIPTCOMMAND;   // script command
extern const char* DBGSTD;          // non-specific debug message
extern const char* AUDITFAIL;       // message failed to get to Audit type logging
extern const char* NOTE;            // Note message
extern const char* USERTRACE;       // Trace Message
extern const char* AUDIT;           // Copy of Audit message
extern const char* REPOSITORY;      // Repository Debug messages
extern const char* CONNECTION;
extern const char* REQUEST;
extern const char* FULL;
extern const char* SYNC;

extern const int   ERRORLEVEL;      // Important Trace error
extern const int   WARNINGLEVEL;
extern const int   DEBUGLEVEL;      // Sort of mid range level
extern const int   DETAILLEVEL;     // Very specific Trace messages
extern const int   AUDITLEVEL;      // Level at which Audit logs are traced
extern const int   TRACELEVEL;      // A level to use for trace messages
extern const int   NOTELEVEL;       // A level to use for Note messages
*/

// This class implements the behavior needed to produce traces to an output stream.
// It can manage up to 64 simultaneous traces.
// The output stream is the standard output or a file.
class CTrace
{
public:
    //// --------------------------------------------------------------------------------
    //// Construction / Destruction.
    //// --------------------------------------------------------------------------------

    // zFileName is the path of a file to write to. That file is written to as a
    // memory-mapped file, to ensure quick memory write operations, lazily written
    // to the actual file by system threads.
    // The special value "stdout" for zFileName routes the output to the standard
    // output, in which case the write operations are slow.
    CTrace(LPCSTR zFileName, DWORD dwViewSize = 10485760)
        : m_fileName(zFileName), m_pView(NULL), m_dwViewSize(dwViewSize), m_dwOffset(0),
          m_dwViewNum(0),
          m_sl(4000)
    {
        if(m_fileName == "stdout")
        {
            // Output to the standard output.
            m_hFile = GetStdHandle(STD_OUTPUT_HANDLE);
        }
        else
        {
            // Output to a file. Map it into memory.
            m_hFile = CreateFile(zFileName, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ, NULL,
                CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS, NULL);
            if(m_hFile != INVALID_HANDLE_VALUE)
            {
                HANDLE hMap = CreateFileMapping(m_hFile, NULL, PAGE_READWRITE, 0,
                    m_dwViewSize, NULL);
                if(hMap != NULL)
                {
                    m_pView = (LPSTR)MapViewOfFile(hMap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
                    if(m_pView == NULL)
                    {
                        CloseHandle(m_hFile);
                        m_hFile = NULL;
                    }
                    CloseHandle(hMap);
                }
                else
                {
                    CloseHandle(m_hFile);
                    m_hFile = NULL;
                }
            }
            else
                m_hFile = NULL;
        }
    }

    ~CTrace()
    {
        if(m_pView && m_hFile)
        {
            // File output: adjust file size and close.
            UnmapViewOfFile(m_pView);
            SetFilePointer(m_hFile, m_dwViewNum * m_dwViewSize + m_dwOffset, NULL, FILE_BEGIN);
            SetEndOfFile(m_hFile);
            CloseHandle(m_hFile);
        }
    }


    //// --------------------------------------------------------------------------------
    //// Configuration. These methods are not thread-safe, and should be
    //// called to configure the trace object.
    //// --------------------------------------------------------------------------------

    // Enabling a trace.
    CTrace& AddTrace(LPCSTR zTrace) {m_traces.insert(std::string(zTrace)); return *this;}

    // Disabling a trace.
    void RemoveTrace(LPCSTR zTrace) {m_traces.erase(std::string(zTrace));}


    //// --------------------------------------------------------------------------------
    //// User methods. These methods are thread-safe, and are available
    //// when an trace object is configured.
    //// --------------------------------------------------------------------------------

    // This class allows to acquire a lock on the stream, to guard against
    // access from concurrent threads. Writes to the stream can then be
    // performed as transactions.
    // A Lock object is to be instanciated on the stack, so that the lock
    // is released upon scope exit.
    class Lock
    {
    public:
        Lock(CTrace& trace, std::ostream*& pOstream) : m_trace(trace)
        {
            m_trace.m_sl.Lock();
            pOstream = &m_ostream;
        }
        Lock(CTrace* pTrace, std::ostream*& pOstream) : m_trace(*pTrace)
        {
            m_trace.m_sl.Lock();
            pOstream = &m_ostream;
        }
        ~Lock()
        {
            // Write the data to the output.
            m_trace.Write(m_ostream.str().c_str());
            m_trace.m_sl.Unlock();
        }
    private:
        //// --------------------------------------------------------------------------------
        //// Forbidden.
        //// --------------------------------------------------------------------------------
        void operator=(const Lock&);
        Lock(const Lock&);

        //// --------------------------------------------------------------------------------
        //// Data
        //// --------------------------------------------------------------------------------
        CTrace&            m_trace;
        std::ostringstream m_ostream;
    };

    friend class Lock;

    // Checking whether a trace is enabled.
    // Multiple traces can be specified by separating them with blank (space) characters.
    // The function performs a logical OR between the traces if bOR is TRUE, otherwise,
    // a logical AND.
    BOOL IsTrace(LPCSTR zTrace, BOOL bOR = TRUE) const
    {
        BOOL bRet = FALSE;
        if(!m_traces.empty())
        {
            bRet = !bOR;
            std::istringstream is(zTrace);
            std::string s;
            BOOL bContinue = TRUE;
            while(bContinue)
            {
                is >> s;
                if(!s.empty())
                {
                    BOOL bFound = m_traces.find(s) != m_traces.end();
                    if(bOR && bFound)
                    {
                        // If the trace is found, stop searching.
                        bRet = TRUE;
                        bContinue = FALSE;
                    }
                    else if(!bFound)
                    {
                        // If the trace is NOT found, stop searching.
                        bRet = FALSE;
                        bContinue = FALSE;
                    }
                }
                else
                    bContinue = FALSE;
            }
        }
        return bRet;
    }

    // Dump to ostream if the trace is set.
    template <class T>
    CTrace& TraceIf(LPCSTR zTrace, const T& t, BOOL bNewLine = FALSE)
    {
        if(!m_traces.empty() && IsTrace(zTrace))
        {
            std::ostream* pOstream = NULL;
            Lock lock(this, pOstream);

            *pOstream << t;
            if(bNewLine)
                *pOstream << "\r\n";
        }
        return *this;
    }

    // Dump always (ignoring the traces).
    template <class T>
    CTrace& Trace(const T& t, BOOL bNewLine = FALSE)
    {
        std::ostream* pOstream = NULL;
        Lock lock(this, pOstream);

        *pOstream << t;
        if(bNewLine)
            *pOstream << "\r\n";
        return *this;
    }

    // Dump a newline.
    CTrace& TraceNL()
    {
        std::ostream* pOstream = NULL;
        Lock lock(this, pOstream);

        *pOstream << "\r\n";
        return *this;
    }

private:
    //// --------------------------------------------------------------------------------
    //// Helpers.
    //// --------------------------------------------------------------------------------

    // Writing a string to the output.
    void Write(const char* zString)
    {
        DWORD dwLen = lstrlen(zString);
        Write(zString, dwLen);
    }

    void Write(const char* zString, DWORD dwLen)
    {
        if(m_pView == NULL)
        {
            if(m_hFile != NULL)
            {
                //// Output to standard output.

                DWORD dw;
                WriteFile(m_hFile, zString, dwLen, &dw, NULL);
            }
        }
        else
        {
            //// Output to file.

            BOOL bNewView = m_dwOffset + dwLen > m_dwViewSize;
            DWORD dwToWrite = bNewView ? m_dwViewSize - m_dwOffset : dwLen;

            if(dwToWrite > 0)
                CopyMemory(m_pView + m_dwOffset, (CONST VOID*)zString, dwToWrite);

            // If a new view is needed, then replace the old one.
            if(bNewView)
            {
                // Must enlarge the file and map a new view.
                UnmapViewOfFile(m_pView);
                DWORD dwCurrentFileSize = GetFileSize(m_hFile, NULL);
                HANDLE hMap = CreateFileMapping(m_hFile, NULL, PAGE_READWRITE, 0, dwCurrentFileSize + m_dwViewSize, NULL);
                if(hMap != NULL)
                {
                    m_pView = (LPSTR)MapViewOfFile(hMap, FILE_MAP_ALL_ACCESS, 0, dwCurrentFileSize, m_dwViewSize);
                    if(m_pView == NULL)
                    {
                        CloseHandle(m_hFile);
                        m_hFile = NULL;
                    }
                    else
                    {
                        // Write the second chunk.
                        m_dwViewNum++;
                        CopyMemory(m_pView, (CONST VOID*)(zString + dwToWrite), dwLen - dwToWrite);
                        m_dwOffset = dwLen - dwToWrite;
                    }
                    CloseHandle(hMap);
                }
                else
                {
                    CloseHandle(m_hFile);
                    m_hFile = NULL;
                }
            }
            else
                m_dwOffset += dwToWrite;
        }
    }

    //// --------------------------------------------------------------------------------
    //// Data.
    //// --------------------------------------------------------------------------------
    std::string             m_fileName;     // Name of the output file. "stdout" if the
                                            // output is the standard output.
    HANDLE                  m_hFile;        // Output file's handle.
    const DWORD             m_dwViewSize;   // Size of the mapped view of the output file.
    LPSTR                   m_pView;        // The mapped view of the output file, or NULL
                                            // if the output is the standard output.
    DWORD                   m_dwViewNum;    // View number.
                                            // or 0 if the output is the standard output.
    DWORD                   m_dwOffset;     // Current offset to which data must be written,
                                            // or 0 if the output is the standard output.
    std::set<std::string>   m_traces;       // The set of traces. Should not be changed
                                            // after trace object configuration.
    CFastLock               m_sl;           // To guard against concurrent access.

    //// --------------------------------------------------------------------------------
    //// Forbidden.
    //// --------------------------------------------------------------------------------
    CTrace(const CTrace& trace);
    CTrace& operator=(const CTrace& trace);
};

// Tracing the file name.
#define TraceFileName()             Trace(__FILE__)
#define TraceFileNameNL()           Trace(__FILE__, TRUE)
#define TraceFileNameIf(zTrace)     TraceIf(zTrace, __FILE__)
#define TraceFileNameIfNL(zTrace)   TraceIf(zTrace, __FILE__, TRUE)

// Tracing the line #.
#define TraceLineNumber()           Trace(__LINE__)
#define TraceLineNumberNL()         Trace(__LINE__, TRUE)
#define TraceLineNumberIf(zTrace)   TraceIf(zTrace, __LINE__)
#define TraceLineNumberIfNL(zTrace) TraceIf(zTrace, __LINE__, TRUE)

// Tracing the compilation date.
#define TraceDate()                 Trace(__DATE__)
#define TraceDateNL()               Trace(__DATE__, TRUE)
#define TraceDateIf(zTrace)         TraceIf(zTrace, __DATE__)
#define TraceDateIfNL(zTrace)       TraceIf(zTrace, __DATE__, TRUE)

// Tracing the compilation time.
#define TraceTime()                 Trace(__TIME__)
#define TraceTimeNL()               Trace(__TIME__, TRUE)
#define TraceTimeIf(zTrace)         TraceIf(zTrace, __TIME__)
#define TraceTimeIfNL(zTrace)       TraceIf(zTrace, __TIME__, TRUE)

// Tracing the date and time of the last modification.
#define TraceTimestamp()            Trace(__TIMESTAMP__)
#define TraceTimestampNL()          Trace(__TIMESTAMP__, TRUE)
#define TraceTimestampIf(zTrace)    TraceIf(zTrace, __TIMESTAMP__)
#define TraceTimestampIfNL(zTrace)  TraceIf(zTrace, __TIMESTAMP__, TRUE)

// Macro to simplify acquiring a trace stream lock.
// This allows to write:
//   GetTraceStream(m_pTrace) << "abc" << "def";
//   ReleaseTraceStream();
#define GetTraceStream(pTrace)      {std::ostream *pOstream = NULL; \
                                    CTrace::Lock lock(pTrace, pOstream); \
                                    *pOstream
#define ReleaseTraceStream()        }


#endif  // _OPENSTA_TRACE_H_
